var express = require('express');
var router = express.Router();
const Validator = require('Validator');
const chat_model = require('./chat_model');
var common = require('../../../utilities/common');
var validate_header = require('../../../middleware/validate-header');
var uniqueMiddleware = require('../../../middleware/uniqueMiddleware');
const httpStatusCode = require('http-status-codes');
var { t } = require('localizify');

/* routes are starting here */
router.post('/create_chat_room', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        sender_id: "required",
        receiver_id: "required"
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.create_chat_room(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }

            validate_header.send_response(res, response_data);
        });
    }

});

/* create group */
router.post('/create_group', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        chat_name: "required",
        // profile_image: "required",
        users: "required|array"
    }
    const messages = {
        'required': t('field_requird'),
    }

    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.create_group(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }

            validate_header.send_response(res, response_data);
        });
    }
});

/* chat room list */
router.post('/get_chat_room_list', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    chat_model.get_chat_room_list(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
            // console.log('response get_chat_room_list===========',response)
        }

        validate_header.send_response(res, response_data);
    });
});

/* chat room list */
router.post('/get_unread_chat_history', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        chat_room_id: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.get_unread_chat_history(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }

            validate_header.send_response(res, response_data);
        });
    }
});

/* chat room list */
router.post('/get_chat_history', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        chat_room_id: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.get_chat_history(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }

            validate_header.send_response(res, response_data);
        });
    }
});

/* sendMessage */
router.post('/sendMessage', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        message_type: "required",
        message: "required",
        sender_id: "required",
    }
    // if (request.chat_room_id = undefined) {
    //     rules.receiver_id = "required"
    // }
    // if (request.receiver_id = undefined) {
    //     rules.chat_room_id = "required"
    // }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.sendMessage(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }

            validate_header.send_response(res, response_data);
        });
    }
});

/* Send Emit from admin */
router.post('/send_emit', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        emit_type: "required",
        emit_user: "required",
        emit_response: "required"
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.send_emit(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});

/* Get group detail */
router.post('/chat_room_detail', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        chat_room_id: "required"
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.chat_room_detail(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});

/* update group name and profile pic */
router.post('/update_group', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        chat_room_id: "required",
        chat_name: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.update_group(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});

/* add users in group */
router.post('/add_user_in_group', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        users: "required",
        chat_room_id: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.add_user_in_group(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});

/* make group admin */
router.post('/make_admin', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        group_user_id: "required",
        chat_room_id: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.make_admin(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});

/* remove user from group */
router.post('/remove_user', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        group_user_id: "required",
        chat_room_id: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.remove_user(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});

/* delete group */
router.post('/delete_conversation', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        chat_room_id: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.delete_conversation(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});

/* delete group */
router.post('/delete_group', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        chat_room_id: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.delete_group(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});
/* Clear Chat */
router.post('/clear_chat', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        chat_room_id: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.clear_chat(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});
/* Make/Remove starred message */
router.post('/remove_starred', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        message_ids: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.remove_starred(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});
router.post('/make_starred', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        message_ids: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.make_starred(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});
/* Starred message list */
router.post('/starred_message_list', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    var rules = {
        chat_room_id: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.starred_message_list(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});
/*Message Viewed User List */
router.post('/message_viewed_user', (req, res) => {
    var request = req.body
    request.user_id = req.user_id;
    console.log("message_viewed_user", request);
    var rules = {
        message_id: "required",
    }
    const messages = {
        'required': t('field_requird'),
    }
    var v = Validator.make(request, rules, messages);
    if (v.fails()) {
        var error
        const errors = v.getErrors();
        for (var key in errors) {
            error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data);
    } else {
        chat_model.message_viewed_user(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        });
    }
});

module.exports = router;