let responseCode = require('../../../config/response-error-code');
const constant = require('../../../config/constant');
let moment = require('moment');
const con = require('../../../config/database');
const asyncLoop = require('node-async-loop')
let { t } = require('localizify');
const interestModel = require('../../../model/interests');
const questionModel = require('../../../model/questions');
const optionsModel = require('../../../model/questions_answers');
const loveLanguageModel = require('../../../model/love_languages');
const bloodModel = require('../../../model/bloods');
const relationshipsModel = require('../../../model/relationships');
const otherGendersModel = require('../../../model/other_genders');
const hashtagsModel = require('../../../model/hashtags');
const postProfileModel = require('../../../model/postProfiles');
const reactionsModel = require('../../../model/reactions');
const reportReasonModel = require('../../../model/report_reason');
const backgroundColorModel = require('../../../model/background_color');
let common_model = {
    getQuestion: (req, callback) => {
        questionModel.find({ is_delete: "0", is_active: "1" }, (err, questions) => {
            if (err) {
                callback(responseCode.OPERATION_FAILED, err, []);
            } else {
                let response_data = [];
                if (questions.length > 0) {
                    asyncLoop(questions, (item, next) => {
                        optionsModel.find({ is_delete: "0", question_id: item._id }, (_err, _option) => {
                            if (_err) {
                                next(_err);
                                return;
                            } else {
                                let data = {};
                                data._id = item._id;
                                data.question = item.question;
                                data.answers = _option;
                                response_data.push(data);
                                next();
                            }
                        })
                    }, () => {
                        callback(responseCode.SUCCESS, t('data_found'), response_data);

                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);

                }
            }
        })

    },
    getInterests: (req, callback) => {
        try {
            let filters = { is_active: '1', is_deleted: '0' }
            if (req.search != undefined && req.search != '') {
                filters.name = { $regex: '.*' + req.search + '.*', $options: 'i' }
            }
            interestModel.find(filters, { _id: 1, image: { $concat: [constant.s3Bucket_interest, "$image"] }, name: 1 }, (err, all_interests) => {
                if (all_interests.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), all_interests);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            });

        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, [])
        }
    },
    getLoveLanguage: (req, callback) => {
        try {
            loveLanguageModel.find({ is_active: '1', is_deleted: '0' }, { _id: 1, name: 1, description: 1, image: { $concat: [constant.s3Bucket_loveLanguage, "$image"] }, activeImage: { $concat: [constant.s3Bucket_loveLanguage, "$activeImage"] } }, (err, all_languages) => {
                if (all_languages.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), all_languages);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            });
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, [])
        }
    },
    getBloods: async (req, callback) => {
        try {
            bloodModel.find({ is_active: '1', is_deleted: '0' }, (err, all_bloods) => {
                if (all_bloods.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), all_bloods);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            });
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, [])
        }
    },
    getOtherGenders: async (req, callback) => {
        try {
            otherGendersModel.find({ is_deleted: '0' }, (err, allGenders) => {
                if (allGenders.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), allGenders);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            });
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, [])
        }
    },
    addRelationship: (req, callback) => {
        let addData = {
            name: req.name
        }
        relationshipsModel.create(addData, (_err, _response) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('relationship_added'), { relation_id: _response._id });
            }
        })
    },
    getRelationship: (req, callback) => {
        relationshipsModel.find({ is_delete: '0' }, (_err, _rows) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                if (_rows.length == 0) {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);/*   no need to show this message */
                } else {
                    callback(responseCode.SUCCESS, t('data_found'), _rows);
                }
            }
        })
    },
    getProfileForPost: (req, callback) => {
        postProfileModel.find({ is_delete: '0' }, (_err, _rows) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                if (_rows.length == 0) {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);/*   no need to show this message */
                } else {
                    callback(responseCode.SUCCESS, t('data_found'), _rows);
                }
            }
        })
    },
    searchHashtag: (req, callback) => {
        try {
            hashtagsModel.aggregate([
                {
                    $match: {
                        hashtag: { $regex: '.*' + req.hashtag + '.*', $options: 'i' },
                        is_active: "1",
                        is_delete: "0"
                    }
                },
                {
                    $lookup: {
                        from: 'post_hatshtags',
                        localField: '_id',
                        foreignField: 'hashtag_id',
                        as: 'postHashtag'
                    }
                },
                {
                    $addFields: {
                        totalPost: { $size: '$postHashtag' },
                    }
                },
                {
                    $project: {
                        _id: 1,
                        hashtag: 1,
                        totalPost: 1
                    }
                }
            ]).then(_rows => {
                if (_rows.length == 0) {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);/*   no need to show this message */
                } else {
                    callback(responseCode.SUCCESS, t('data_found'), _rows);
                }
            })

        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    getReactions: (req, callback) => {
        reactionsModel.find({}, { _id: 1, name: 1, image: { $concat: [constant.s3Bucket_emoji, "$image"] } }, (_err, _rows) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                if (_rows.length == 0) {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);/*   no need to show this message */
                } else {
                    callback(responseCode.SUCCESS, t('data_found'), _rows);
                }
            }
        })
    },
    storyReportReasonList: (req, callback) => {
        reportReasonModel.find({}, { _id: 1, name: 1 }, (_err, _rows) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                if (_rows.length == 0) {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);/*   no need to show this message */
                } else {
                    callback(responseCode.SUCCESS, t('data_found'), _rows);
                }
            }
        })
    },
    colorList: (req, callback) => {
        backgroundColorModel.find({}, { _id: 1, color_name: 1 }, (_err, _rows) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                if (_rows.length == 0) {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);/*   no need to show this message */
                } else {
                    callback(responseCode.SUCCESS, t('data_found'), _rows);
                }
            }
        })
    },
    getLoveLanguages: (req, callback) => {
        loveLanguageModel.find({ is_active: '1', is_deleted: '0' }, { image: { $concat: [constant.s3Bucket_loveLanguage, "$image"] }, activeImage: { $concat: [constant.s3Bucket_loveLanguage, "$activeImage"] } }, (_err, all_languages) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, []);
            } else if (all_languages.length > 0) {
                callback(responseCode.SUCCESS, t("data_found"), all_languages);
            }
            else {
                callback(responseCode.NO_DATA_FOUND, t("data_not_found"), []);
            }
        })
    }
};
module.exports = common_model;