const responseCode = require('../../../config/response-error-code');
const con = require('../../../config/database');
const common = require('../../../utilities/common');
const validate_header = require('../../../middleware/validate-header');
const constant = require('../../../config/constant');
const moment = require('moment');
const asyncLoop = require('node-async-loop')
const userModel = require('../../../model/user');
const followersModel = require('../../../model/followers');
const familyModel = require('../../../model/family');
const goalsAndDreamsModel = require('../../../model/goals/goals_and_dreams')
const goalsCatogaryModel = require('../../../model/goals/goal_categories')
const goalRequestModel = require('../../../model/goals/goal_request')
const progressNoteModel = require('../../../model/goals/progress_notes')
const notiLibrary = require('../../../config/notification')
const mongoose = require('mongoose');
const { t } = require('localizify');
const _ = require('lodash');
let goals_model = {
    /**
     * @description API method for add goal category
     */
    addGoalCategory: (req, callback) => {
        try {
            let addData = {
                user_id: req.user_id,
                name: req.name,
                image: req.image
            }
            goalsCatogaryModel.create(addData, async (_err, _result) => {
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, {});
                } else {
                    let category = await goalsCatogaryModel.findOne({ _id: _result._id }, {
                        _id: 1,
                        name: 1,
                        image: {
                            $cond: {
                                if: { $ne: ['$image', ''] }, then: {
                                    $concat: [constant.s3Bucket_goals_category, '$image']
                                }, else: ''
                            }
                        },
                    })
                    callback(responseCode.SUCCESS, t('goal_category_added'), category);
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});

        }

    },
    /**
     * @description API method for goals category listing
     */
    goalCategoriesList: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            let response_data = [];
            let condition = {}
            let search = (req.search != undefined && req.search != '') ? req.search : '';
            goalsCatogaryModel.aggregate([
                {
                    $match: {
                        $or: [
                            { 'name': { $regex: '.*' + search + '.*', $options: 'i' } }
                        ]
                    }
                },
                {
                    $lookup: {
                        from: "goals_and_dreams",
                        localField: "_id",
                        foreignField: "goal_category_id",
                        as: "totalGoals"
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        _id: 1,
                        name: 1,
                        image: {
                            $cond: {
                                if: { $ne: ['$image', ''] }, then: {
                                    $concat: [constant.s3Bucket_goals_category, '$image']
                                }, else: ''
                            }
                        },
                        totalUsers: { $size: '$totalGoals' }
                    }
                },
                { $skip: start },
                { $limit: limit }
            ], (_err, _result) => {
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, []);
                } else if (_result.length > 0) {
                    asyncLoop(_result, async (item, next) => {
                        const count = await goalsAndDreamsModel.aggregate([
                            {
                                $match: {
                                    goal_category_id: item._id
                                }
                            },
                            {
                                $group: {
                                    _id: { goal_category_id: '$goal_category_id', user_id: '$user_id' },
                                    uniqueGoals: { $first: '$_id' } // Group by category and user, consider only unique goals
                                }
                            },
                            {
                                $group: {
                                    _id: '$_id.goal_category_id',
                                    totalGoalCount: { $sum: 1 } // Count the unique goals per category
                                }
                            },
                            {
                                $project: {
                                    totalGoalCount: 1
                                }
                            }
                        ]);
                        const totalGoals = count.length > 0 ? count[0].totalGoalCount : 0;
                        item.totalUsers = totalGoals;
                        next();
                    }, () => {
                        callback(responseCode.SUCCESS, t('data_found'), _result);
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },

    categoryDetail: (req, callback) => {
        try {
            const goal_category_id = mongoose.Types.ObjectId(req.goal_category_id);
            goalsCatogaryModel.aggregate([
                {
                    $match: {
                        _id: goal_category_id
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'goalUsers.user_id',
                        foreignField: '_id',
                        as: 'userProfile'
                    }
                },
                {
                    $project: {
                        _id: 1,
                        name: 1,
                        image: {
                            $cond: {
                                if: { $ne: ['$image', ''] }, then: {
                                    $concat: [constant.s3Bucket_goals_category, '$image']
                                }, else: ''
                            }
                        }
                    }
                }
            ], async (_err, _result) => {
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, {});
                } else if (_result.length > 0) {
                    const count = await goalsAndDreamsModel.aggregate([
                        {
                            $match: {
                                goal_category_id: _result[0]._id
                            }
                        },
                        {
                            $group: {
                                _id: { goal_category_id: '$goal_category_id', user_id: '$user_id' },
                                uniqueGoals: { $first: '$_id' } // Group by category and user, consider only unique goals
                            }
                        },
                        {
                            $group: {
                                _id: '$_id.goal_category_id',
                                totalGoalCount: { $sum: 1 } // Count the unique goals per category
                            }
                        },
                        {
                            $project: {
                                totalGoalCount: 1
                            }
                        }
                    ]);
                    const totalGoals = count.length > 0 ? count[0].totalGoalCount : 0;
                    _result[0].totalUsers = totalGoals;
                    callback(responseCode.SUCCESS, t('data_found'), _result[0]);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
                }
            })
        } catch (error) {
            console.log(error)
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },

    getGoalCategoryUsers: async (req, callback) => {
        try {
            const goal_category_id = mongoose.Types.ObjectId(req.goal_category_id);
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            let categoryDetail = await goalsCatogaryModel.find({ _id: goal_category_id });
            if (categoryDetail.length > 0) {
                goalsAndDreamsModel.aggregate([
                    {
                        $match: {
                            goal_category_id: goal_category_id
                        }
                    },
                    {
                        $project: {
                            user_id: 1,
                            goal_category_id: 1,
                            // Calculate the progress percentage based on start_date and end_date
                            progressPercentage: {
                                $cond: {
                                    if: { $gt: [{ $subtract: [new Date(), '$start_date'] }, 0] },
                                    then: {
                                        $cond: {
                                            if: { $gt: [{ $subtract: ['$end_date', '$start_date'] }, 0] },
                                            then: {
                                                $cond: {
                                                    if: { $lt: [{ $subtract: [new Date(), '$end_date'] }, 0] },
                                                    then: {
                                                        $round: [{
                                                            $multiply: [
                                                                { $divide: [{ $subtract: [new Date(), '$start_date'] }, { $subtract: ['$end_date', '$start_date'] }] },
                                                                100
                                                            ]
                                                        }, 2]
                                                    },
                                                    else: 100
                                                }
                                            },
                                            else: 100
                                        }
                                    },
                                    else: 0
                                }
                            }
                        }
                    },
                    {
                        $group: {
                            _id: { userId: '$user_id', categoryId: '$goal_category_id' },
                            highestProgress: { $max: { $round: ['$progressPercentage', 2] } }
                        }
                    },
                    {
                        $lookup: {
                            from: 'users',
                            localField: '_id.userId',
                            foreignField: '_id',
                            as: 'user'
                        }
                    },
                    {
                        $unwind: '$user'
                    },
                    {
                        $lookup: {
                            from: 'goal_categories',
                            localField: '_id.categoryId',
                            foreignField: '_id',
                            as: 'category'
                        }
                    },
                    {
                        $unwind: '$category'
                    },

                    {
                        $project: {
                            _id: 0,
                            goal_category_id: '$_id.categoryId',
                            categoryName: '$category.name',
                            user_id: '$user._id',
                            username: '$user.username',
                            user_id: '$user._id',
                            first_name: '$user.first_name',
                            last_name: '$user.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$user.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$user.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$user.email',
                            highestProgress: 1,
                            is_admin: {
                                $cond: {
                                    if: { $eq: ['$user._id', '$category.user_id'] }, then: true,
                                    else: false
                                }
                            }
                        }
                    },
                    {
                        $sort: { highestProgress: -1 }
                    },
                    { $skip: start },
                    { $limit: limit }
                ], (_err, _result) => {
                    if (_err) {
                        callback(responseCode.OPERATION_FAILED, _err, {});
                    } else if (_result.length > 0) {
                        callback(responseCode.SUCCESS, t('data_found'), _result);
                    } else {
                        callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
                    }
                })
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {});

            }
        } catch (error) {
            console.log(error)
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    /**
     * @description API method for goals listing
     */
    goalsAndDreamsList: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        let response_data = [];
        let search = (req.search != undefined && req.search != '') ? req.search : '';
        const currentTime = new Date();
        let condition = {
            is_delete: "0",
            $or: [
                { 'goal_title': { $regex: '.*' + search + '.*', $options: 'i' } }
            ]
        }
        let join = [
            { $eq: ['$request_user_id', req.user_id] }, // Check if the current user follows the author
            { $eq: ['$goal_id', '$$authorID'] },
        ]
        if (req.isMyGoalList != undefined && req.isMyGoalList == '1') {
            condition['user_id'] = req.user_id;
        } else {
            condition['user_id'] = { $ne: req.user_id }
            condition['privacy'] = { $ne: 'OnlyMe' }
            join.push({ $eq: ['$status', 'Accept'] })
        }
        console.log("condition", condition)
        console.log("join", join)
        goalsAndDreamsModel.aggregate([
            {
                $match: condition
            },
            {
                $lookup: {
                    from: 'goal_requests',
                    let: { authorID: '$_id' },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $and: join
                                }
                            }
                        }
                    ],
                    as: 'requests'
                }
            },
            {
                $match: {
                    'requests.0': { $exists: true }, // Ensures there is at least one accepted request
                },
            },
            {
                $lookup: {
                    from: "goal_categories",
                    localField: "goal_category_id",
                    foreignField: "_id",
                    as: "category"
                }
            },
            {
                $unwind: '$category'
            },
            {
                $lookup: {
                    from: "users",
                    localField: "user_id",
                    foreignField: "_id",
                    as: "profile"
                }
            },
            {
                $unwind: '$profile'
            },
            {
                $addFields: {
                    totalDuration: { $subtract: ['$end_date', '$start_date'] },
                    elapsedTime: { $subtract: [currentTime, '$start_date'] }
                }
            },
            {
                $addFields: {
                    // progressPercentage: {
                    //     $multiply: [
                    //         { $divide: ['$elapsedTime', '$totalDuration'] },
                    //         100
                    //     ]
                    // },
                    progressPercentage: {
                        $cond: {
                            if: { $gt: [{ $subtract: [new Date(), '$start_date'] }, 0] },
                            then: {
                                $cond: {
                                    if: { $gt: [{ $subtract: ['$end_date', '$start_date'] }, 0] },
                                    then: {
                                        $cond: {
                                            if: { $lt: [{ $subtract: [new Date(), '$end_date'] }, 0] },
                                            then: {
                                                $round: [{
                                                    $multiply: [
                                                        { $divide: [{ $subtract: [new Date(), '$start_date'] }, { $subtract: ['$end_date', '$start_date'] }] },
                                                        100
                                                    ]
                                                }, 2]
                                            },
                                            else: 100
                                        }
                                    },
                                    else: 100
                                }
                            },
                            else: 0
                        }
                    }
                }
            },
            { $sort: { createdAt: -1 } },
            {
                $project: {
                    _id: 1,
                    goal_title: 1,
                    user_id: 1,
                    goal_description: 1,
                    share_with_all: 1,
                    goal_type: 1,
                    start_date: 1,
                    end_date: 1,
                    privacy: 1,
                    is_reminder: 1,
                    music: 1,
                    category: {
                        category_id: '$category._id',
                        name: '$category.name',
                        image: {
                            $cond: {
                                if: { $ne: ['$category.image', ''] }, then: {
                                    $concat: [constant.s3Bucket_goals_category, '$category.image']
                                }, else: ''
                            }
                        },
                    },
                    profile: {
                        user_id: '$profile._id',
                        username: '$profile.username',
                        user_id: '$profile._id',
                        first_name: '$profile.first_name',
                        last_name: '$profile.last_name',
                        profile_image:
                        {
                            $cond: {
                                if: { $ne: ['$profile.profile_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                }, else: ''
                            }
                        },
                        email: '$profile.email'
                    },
                    // progressPercentage: 1
                    // progressPercentage: {
                    //     $cond: {
                    //         if: { $lte: [{ $round: ['$progressPercentage', 2] }, 0] }, then: 0, else: {
                    //             if: { $gte: [{ $round: ['$progressPercentage', 2] }, 100] }, then: 100, else: { $round: ['$progressPercentage', 2] }
                    //         }
                    //     }
                    // }
                }
            },
            { $skip: start },
            { $limit: limit }
        ], (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, []);
            } else if (_result.length > 0) {
                _result.map((item) => {
                    item.progressPercentage = goals_model.calculateElapsedPercentage(item.start_date, item.end_date);
                })
                callback(responseCode.SUCCESS, t('data_found'), _result);
            } else {
                callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
            }
        })
    },
    /**
     * @description API method for goals listing
     */
    goalDreamRequestList: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        let response_data = [];
        let search = (req.search != undefined && req.search != '') ? req.search : '';
        goalRequestModel.aggregate([
            {
                $match: {
                    status: "Pending",
                    request_user_id: req.user_id
                }
            },
            {
                $lookup: {
                    from: 'goals_and_dreams',
                    let: { authorID: '$goal_id' },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $and: [
                                        { $eq: ['$_id', '$$authorID'] },
                                        { $eq: ['$is_delete', '0'] }
                                    ]
                                }
                            }
                        }
                    ],
                    as: 'goalDream'
                }
            },
            {
                $unwind: '$goalDream'
            },
            {
                $lookup: {
                    from: "goal_categories",
                    localField: "goalDream.goal_category_id",
                    foreignField: "_id",
                    as: "category"
                }
            },
            {
                $unwind: '$category'
            },
            {
                $lookup: {
                    from: "users",
                    localField: "goalDream.user_id",
                    foreignField: "_id",
                    as: "profile"
                }
            },
            {
                $unwind: '$profile'
            },

            { $sort: { createdAt: -1 } },
            {
                $project: {
                    goal_request_id: '$_id',
                    _id: '$goalDream._id',
                    goal_title: '$goalDream.goal_title',
                    goal_description: '$goalDream.goal_description',
                    share_with_all: '$goalDream.share_with_all',
                    goal_type: '$goalDream.goal_type',
                    start_date: '$goalDream.start_date',
                    end_date: '$goalDream.end_date',
                    privacy: '$goalDream.privacy',
                    is_reminder: '$goalDream.is_reminder',
                    addToCalender: '$goalDream.addToCalender',
                    music: '$goalDream.music',
                    category: {
                        category_id: '$category._id',
                        name: '$category.name',
                        image: {
                            $cond: {
                                if: { $ne: ['$category.image', ''] }, then: {
                                    $concat: [constant.s3Bucket_goals_category, '$category.image']
                                }, else: ''
                            }
                        },
                    },
                    profile: {
                        user_id: '$profile._id',
                        username: '$profile.username',
                        user_id: '$profile._id',
                        first_name: '$profile.first_name',
                        last_name: '$profile.last_name',
                        profile_image:
                        {
                            $cond: {
                                if: { $ne: ['$profile.profile_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                }, else: ''
                            }
                        },
                        email: '$profile.email'
                    },

                }
            },
            { $skip: start },
            { $limit: limit }
        ], (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, []);
            } else if (_result.length > 0) {
                callback(responseCode.SUCCESS, t('data_found'), _result);
            } else {
                callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
            }
        })

    },
    goalDelete: async (req, callback) => {
        try {
            const goalId = mongoose.Types.ObjectId(req.goal_id)

            await goalRequestModel.deleteMany({ goal_id: goalId });
            await goalsAndDreamsModel.deleteOne({ _id: goalId });
            callback(responseCode.SUCCESS, t('goal_deleted'), {});

        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
        }
    },
    /**
     * @description API method for goals detail
     */
    goalDetail: (req, callback) => {
        const currentTime = new Date();
        let goalId = mongoose.Types.ObjectId(req.goal_id)
        let condition = {
            is_delete: "0",
            _id: goalId
        }
        goalsAndDreamsModel.aggregate([
            {
                $match: condition
            },
            {
                $lookup: {
                    from: "goal_categories",
                    localField: "goal_category_id",
                    foreignField: "_id",
                    as: "category"
                }
            },
            {
                $unwind: '$category'
            },
            {
                $lookup: {
                    from: "users",
                    localField: "user_id",
                    foreignField: "_id",
                    as: "profile"
                }
            },
            {
                $unwind: '$profile'
            },
            {
                $addFields: {
                    progressPercentage: {
                        $cond: {
                            if: { $gt: [{ $subtract: [new Date(), '$start_date'] }, 0] },
                            then: {
                                $cond: {
                                    if: { $gt: [{ $subtract: ['$end_date', '$start_date'] }, 0] },
                                    then: {
                                        $cond: {
                                            if: { $lt: [{ $subtract: [new Date(), '$end_date'] }, 0] },

                                            then: {
                                                $round: [{
                                                    $multiply: [
                                                        { $divide: [{ $subtract: [new Date(), '$start_date'] }, { $subtract: ['$end_date', '$start_date'] }] },
                                                        100
                                                    ]
                                                }, 2]
                                            },
                                            else: 100
                                        }
                                    },
                                    else: 100
                                }
                            },
                            else: 0
                        }
                    }
                }
            },
            {
                $project: {
                    _id: 1,
                    goal_title: 1,
                    goal_description: 1,
                    share_with_all: 1,
                    goal_type: 1,
                    start_date: 1,
                    end_date: 1,
                    privacy: 1,
                    is_reminder: 1,
                    addToCalender: 1,
                    goal_category_id: 1,
                    music: 1,
                    category: {
                        category_id: '$category._id',
                        name: '$category.name',
                        image: {
                            $cond: {
                                if: { $ne: ['$category.image', ''] }, then: {
                                    $concat: [constant.s3Bucket_goals_category, '$category.image']
                                }, else: ''
                            }
                        },
                    },
                    profile: {
                        user_id: '$profile._id',
                        username: '$profile.username',
                        user_id: '$profile._id',
                        first_name: '$profile.first_name',
                        last_name: '$profile.last_name',
                        profile_image:
                        {
                            $cond: {
                                if: { $ne: ['$profile.profile_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                }, else: ''
                            }
                        },
                        email: '$profile.email'
                    },
                    progressPercentage: 1
                }
            }
        ], async (_err, _result) => {
            // console.log("_result", _result) progressPercentage: { $round: ['$progressPercentage', 2] }
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else if (_result.length > 0) {
                const count = await goalsAndDreamsModel.aggregate([
                    {
                        $match: {
                            goal_category_id: _result[0].goal_category_id
                        }
                    },
                    {
                        $group: {
                            _id: { goal_category_id: '$goal_category_id', user_id: '$user_id' },
                            uniqueGoals: { $first: '$_id' } // Group by category and user, consider only unique goals
                        }
                    },
                    {
                        $group: {
                            _id: '$_id.goal_category_id',
                            totalGoalCount: { $sum: 1 } // Count the unique goals per category
                        }
                    },
                    {
                        $project: {
                            totalGoalCount: 1
                        }
                    }
                ]);
                const totalGoals = count.length > 0 ? count[0].totalGoalCount : 0;
                _result[0].totalUsers = totalGoals;
                if (_result[0].share_with_all != '1') {
                    let allUsers = await goalRequestModel.find({ goal_id: _result[0]._id }, { _id: 0, user_id: '$request_user_id' });
                    // console.log("allUsers", allUsers)
                    if (allUsers.length > 0) {
                        let ids = [];
                        allUsers.map(user => ids.push(user.user_id));
                        _result[0].allUsers = ids;
                        callback(responseCode.SUCCESS, t('data_found'), _result[0]);
                    } else {
                        _result[0].allUsers = [];
                        callback(responseCode.SUCCESS, t('data_found'), _result[0]);
                    }

                } else {
                    _result[0].allUsers = [];
                    callback(responseCode.SUCCESS, t('data_found'), _result[0]);
                }
            } else {
                callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
            }
        })
    },
    /**
     * @description API method for creating goals
     */
    editGoalsAndDreams: (req, callback) => {
        let goalId = mongoose.Types.ObjectId(req.goal_id)
        let editData = {
            goal_title: req.goal_title,
            goal_type: req.goal_type,
            start_date: req.start_date,
            end_date: req.end_date,
            privacy: req.privacy,
            goal_category_id: req.goal_category_id,
            is_reminder: req.is_reminder,
            addToCalender: req.addToCalender,
            share_with_all: (req.share_with_all != undefined && req.share_with_all != '0') ? req.share_with_all : '0',
            goal_description: (req.goal_description != undefined && req.goal_description != '') ? req.goal_description : '',
            music: (req.music != undefined && req.music != '') ? req.music : '',
        }
        goalsAndDreamsModel.updateOne({ _id: goalId }, editData, async (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                // console.log("_result", _result)
                if (req.privacy == 'OnlyMe') {
                    goalRequestModel.deleteMany({
                        goal_id: _result._id,
                        request_user_id: { $ne: req.user_id },
                        status: { $ne: 'Accept' }
                    }, async (err, _result) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('goals_and_dreams_edited'), {});
                        } else {
                            callback(responseCode.OPERATION_FAILED, _err, {});
                        }
                    })
                } else if (req.privacy == 'FollowingWith') {
                    goalRequestModel.deleteMany({
                        goal_id: _result._id,
                        status: { $ne: 'Accept' }
                    }, async (err, _result) => {
                        if (!err) {
                            req.goal_id = _result._id
                            await goals_model.editGoalFollowingWith(req);
                            callback(responseCode.SUCCESS, t('goals_and_dreams_edited'), {});
                        } else {
                            callback(responseCode.OPERATION_FAILED, _err, {});
                        }
                    })

                } else if (req.privacy == 'FamilyOnly') {
                    goalRequestModel.deleteMany({
                        goal_id: _result._id,
                        status: { $ne: 'Accept' }
                    }, async (err, _result) => {
                        if (!err) {
                            req.goal_id = _result._id
                            await goals_model.editGoalFamilyOnlyWith(req);
                            callback(responseCode.SUCCESS, t('goals_and_dreams_edited'), {});
                        } else {
                            callback(responseCode.OPERATION_FAILED, _err, {});
                        }
                    })

                } else if (req.privacy == 'Friends') {
                    goalRequestModel.deleteMany({
                        goal_id: _result._id,
                        status: { $ne: 'Accept' }
                    }, async (err, _result) => {
                        if (!err) {
                            req.goal_id = _result._id
                            await goals_model.editGoalFriendsWith(req);
                            callback(responseCode.SUCCESS, t('goals_and_dreams_edited'), {});
                        } else {
                            callback(responseCode.OPERATION_FAILED, _err, {});
                        }
                    })
                } else if (req.privacy == 'Groups') {
                    callback(responseCode.SUCCESS, t('goals_and_dreams_edited'), {});
                }

            }
        })
    },
    addGoalsAndDreams: (req, callback) => {
        let addData = {
            user_id: req.user_id,
            goal_title: req.goal_title,
            goal_type: req.goal_type,
            start_date: req.start_date,
            end_date: req.end_date,
            privacy: req.privacy,
            goal_category_id: req.goal_category_id,
            is_reminder: req.is_reminder,
            addToCalender: req.addToCalender,
            share_with_all: (req.share_with_all != undefined && req.share_with_all != '0') ? req.share_with_all : '0',
            goal_description: (req.goal_description != undefined && req.goal_description != '') ? req.goal_description : '',
            music: (req.music != undefined && req.music != '') ? req.music : '',
        }
        goalsAndDreamsModel.create(addData, async (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                console.log("_result", _result)
                if (req.privacy == 'OnlyMe') {
                    goalRequestModel.create({
                        user_id: req.user_id,
                        goal_id: _result._id,
                        request_user_id: req.user_id,
                        status: 'Accept'
                    }, async (err, _result) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('goals_and_dreams_added'), {});
                        } else {
                            callback(responseCode.OPERATION_FAILED, _err, {});
                        }
                    })
                } else if (req.privacy == 'FollowingWith') {
                    goalRequestModel.create({
                        user_id: req.user_id,
                        goal_id: _result._id,
                        request_user_id: req.user_id,
                        status: 'Accept'
                    }, async (err, _user) => {
                        if (!err) {
                            req.goal_id = _result._id
                            await goals_model.addGoalFollowingWith(req);
                            callback(responseCode.SUCCESS, t('goals_and_dreams_added'), {});
                        } else {
                            callback(responseCode.OPERATION_FAILED, _err, {});
                        }
                    })

                } else if (req.privacy == 'FamilyOnly') {
                    goalRequestModel.create({
                        user_id: req.user_id,
                        goal_id: _result._id,
                        request_user_id: req.user_id,
                        status: 'Accept'
                    }, async (err, _user) => {
                        if (!err) {
                            req.goal_id = _result._id
                            await goals_model.addGoalFamilyOnlyWith(req);
                            callback(responseCode.SUCCESS, t('goals_and_dreams_added'), {});
                        } else {
                            callback(responseCode.OPERATION_FAILED, _err, {});
                        }
                    })

                } else if (req.privacy == 'Friends') {
                    goalRequestModel.create({
                        user_id: req.user_id,
                        goal_id: _result._id,
                        request_user_id: req.user_id,
                        status: 'Accept'
                    }, async (err, _user) => {
                        if (!err) {
                            req.goal_id = _result._id
                            await goals_model.addGoalFriendsWith(req);
                            callback(responseCode.SUCCESS, t('goals_and_dreams_added'), {});
                        } else {
                            callback(responseCode.OPERATION_FAILED, _err, {});
                        }
                    })
                } else if (req.privacy == 'Groups') {
                    callback(responseCode.SUCCESS, t('goals_and_dreams_added'), {});
                }

            }
        })
    },
    editGoalFamilyOnlyWith: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                let familyResult = await familyModel.find({
                    user_id: req.user_id,
                    is_delete: '0',
                });
                if (familyResult.length > 0) {
                    goalRequestModel.deleteMany({ goal_id: req.goal_id, user_id: req.user_id, status: { $ne: 'Accept' } }, (err, result) => {
                        if (!err) {
                            asyncLoop(familyResult, async (item, next) => {
                                let findUser = await goalRequestModel.find({
                                    user_id: req.user_id,
                                    goal_id: req.goal_id,
                                    request_user_id: item.family_member_id,
                                    status: 'Accept'
                                })
                                if (findUser.length > 0) {
                                    next();
                                } else {
                                    goalRequestModel.create({
                                        user_id: req.user_id,
                                        goal_id: req.goal_id,
                                        request_user_id: item.family_member_id
                                    }, async (err, _result) => {
                                        if (!err) {
                                            let userData = await common.userDetails({ receiver_id: req.user_id });
                                            // send notification to the owner of the post
                                            let notifi_data = {
                                                receiver_id: item.family_member_id,
                                                sender_id: req.user_id,
                                                title: "Neshwa",
                                                message: userData.username + " invite you to see their goal & dream",
                                                tag: "goals_request",
                                                image_url: "",
                                                goal_id: req.goal_id,
                                                goal_request_id: _result._id,
                                            }
                                            common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                                if (err) {
                                                    console.log("err", err)
                                                    next();
                                                } else {
                                                    console.log("data", data)
                                                    next();
                                                }
                                            })
                                        } else {
                                            next(err);
                                            return;
                                        }
                                    })
                                }
                            }, (err) => {
                                if (err) {
                                    reject(err);
                                } else {
                                    resolve(true);
                                }
                            });

                        } else {
                            reject(err);
                        }
                    })
                } else {
                    resolve(true)
                }
            } catch (e) {
                reject("data_not_found");
            }
        });
    },
    addGoalFamilyOnlyWith: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                let familyResult = await familyModel.find({
                    user_id: req.user_id,
                    is_delete: '0',
                });
                if (familyResult.length > 0) {
                    goalRequestModel.deleteMany({ goal_id: req.goal_id, user_id: req.user_id, request_user_id: { $ne: req.user_id } }, (err, result) => {
                        if (!err) {
                            asyncLoop(familyResult, async (item, next) => {
                                let findUser = await goalRequestModel.find({
                                    user_id: req.user_id,
                                    goal_id: req.goal_id,
                                    request_user_id: item.following_id,
                                    status: 'Accept'
                                })
                                if (findUser.length > 0) {
                                    next();
                                } else {
                                    goalRequestModel.create({
                                        user_id: req.user_id,
                                        goal_id: req.goal_id,
                                        request_user_id: item.family_member_id
                                    }, async (err, _result) => {
                                        if (!err) {
                                            let userData = await common.userDetails({ receiver_id: req.user_id });
                                            // send notification to the owner of the post
                                            let notifi_data = {
                                                receiver_id: item.family_member_id,
                                                sender_id: req.user_id,
                                                title: "Neshwa",
                                                message: userData.username + " invite you to see their goal & dream",
                                                tag: "goals_request",
                                                image_url: "",
                                                goal_id: req.goal_id,
                                                goal_request_id: _result._id,
                                            }
                                            common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                                if (err) {
                                                    console.log("err", err)
                                                    next();
                                                } else {
                                                    console.log("data", data)
                                                    next();
                                                }
                                            })
                                        } else {
                                            next(err);
                                            return;
                                        }
                                    })
                                }
                            }, (err) => {
                                if (err) {
                                    reject(err);
                                } else {
                                    resolve(true);
                                }
                            });

                        } else {
                            reject(err);
                        }
                    })
                } else {
                    resolve(true)
                }
            } catch (e) {
                reject("data_not_found");
            }
        });
    },
    addGoalFriendsWith: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                if (req.share_with_all == '1') {
                    followersModel.aggregate([
                        {
                            $match: {
                                follower_id: req.user_id,
                                status: 'Accept'
                            }
                        },
                        {
                            $lookup: {
                                from: 'followers',
                                let: { authorID: '$following_id' },
                                pipeline: [
                                    {
                                        $match: {
                                            $expr: {
                                                $and: [
                                                    { $eq: ['$following_id', req.user_id] }, // Check if the current user follows the author
                                                    { $eq: ['$follower_id', '$$authorID'] },
                                                    { $eq: ['$status', 'Accept'] }
                                                ],
                                            },
                                        },
                                    },
                                ],
                                as: 'reverseFollow',
                            },
                        },
                        {
                            $lookup: {
                                from: "users",
                                localField: "following_id",
                                foreignField: "_id",
                                as: "followingUserInfo"
                            }
                        },
                        {
                            $unwind: "$followingUserInfo"
                        },
                        {
                            $match: {
                                'reverseFollow.status': 'Accept',
                            }
                        },
                        {
                            $project: {
                                username: '$followingUserInfo.username',
                                user_id: '$followingUserInfo._id',
                            }
                        },
                    ]).then(_results => {
                        if (_results.length > 0) {
                            goalRequestModel.deleteMany({ goal_id: req.goal_id, user_id: req.user_id, request_user_id: { $ne: req.user_id }, status: { $ne: 'Accept' } }, (err, result) => {
                                if (!err) {
                                    asyncLoop(_results, async (item, next) => {
                                        let findUser = await goalRequestModel.find({
                                            user_id: req.user_id,
                                            goal_id: req.goal_id,
                                            request_user_id: item.user_id,
                                            status: 'Accept'
                                        })
                                        if (findUser.length > 0) {
                                            next();
                                        } else {
                                            goalRequestModel.create({
                                                user_id: req.user_id,
                                                goal_id: req.goal_id,
                                                request_user_id: item.user_id
                                            }, async (err, _result) => {
                                                if (!err) {
                                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                                    // send notification to the owner of the post
                                                    let notifi_data = {
                                                        receiver_id: item.user_id,
                                                        sender_id: req.user_id,
                                                        title: "Neshwa",
                                                        message: userData.username + " invite you to see their goal & dream",
                                                        tag: "goals_request",
                                                        image_url: "",
                                                        goal_id: req.goal_id,
                                                        goal_request_id: _result._id,
                                                    }
                                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                                        if (err) {
                                                            console.log("err", err)
                                                            next();
                                                        } else {
                                                            console.log("data", data)
                                                            next();
                                                        }
                                                    })
                                                } else {
                                                    next(err);
                                                    return;
                                                }
                                            })
                                        }
                                    }, (err) => {
                                        if (err) {
                                            reject(err);
                                        } else {
                                            resolve(true);
                                        }
                                    });
                                } else {
                                    reject(err);
                                }
                            })
                        } else {
                            resolve(true)
                        }
                    })
                } else {
                    if (req.share_with_users.length > 0) {
                        goalRequestModel.deleteMany({ goal_id: req.goal_id, user_id: req.user_id, request_user_id: { $ne: req.user_id }, status: { $ne: 'Accept' } }, (err, result) => {
                            if (!err) {
                                asyncLoop(req.share_with_users, async (item, next) => {
                                    let findUser = await goalRequestModel.find({
                                        user_id: req.user_id,
                                        goal_id: req.goal_id,
                                        request_user_id: item,
                                        status: 'Accept'
                                    })
                                    if (findUser.length > 0) {
                                        next();
                                    } else {
                                        goalRequestModel.create({
                                            user_id: req.user_id,
                                            goal_id: req.goal_id,
                                            request_user_id: item
                                        }, async (err, _result) => {
                                            if (!err) {
                                                let userData = await common.userDetails({ receiver_id: req.user_id });
                                                // send notification to the owner of the post
                                                let notifi_data = {
                                                    receiver_id: _result.request_user_id,
                                                    sender_id: req.user_id,
                                                    title: "Neshwa",
                                                    message: userData.username + " invite you to see their goal & dream",
                                                    tag: "goals_request",
                                                    image_url: "",
                                                    goal_id: req.goal_id,
                                                    goal_request_id: _result._id,
                                                }
                                                common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                                    if (err) {
                                                        console.log("err", err)
                                                        next();
                                                    } else {
                                                        console.log("data", data)
                                                        next();
                                                    }
                                                })
                                            } else {
                                                next(err);
                                                return;
                                            }
                                        })
                                    }
                                }, (err) => {
                                    if (err) {
                                        reject(err);
                                    } else {
                                        resolve(true);
                                    }
                                });
                                // req.share_with_users.map((item) => {
                                //     goalRequestModel.create({
                                //         user_id: req.user_id,
                                //         goal_id: req.goal_id,
                                //         request_user_id: item
                                //     }, async (err, _result) => {
                                //         if (!err) {
                                //             let userData = await common.userDetails({ receiver_id: req.user_id });
                                //             // send notification to the owner of the post
                                //             let notifi_data = {
                                //                 receiver_id: _result.request_user_id,
                                //                 sender_id: req.user_id,
                                //                 title: "Neshwa",
                                //                 message: userData.username + " invite you to see their goal & dream.",
                                //                 tag: "goals_request",
                                //                 image_url: "",
                                //                 goal_id: req.goal_id,
                                //                 goal_request_id: _result._id,
                                //             }
                                //             common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                //                 if (err) {
                                //                     console.log("err", err)
                                //                     resolve(_result);
                                //                 } else {
                                //                     console.log("data", data)
                                //                     resolve(_result);
                                //                 }
                                //             })
                                //         } else {
                                //             reject(err);
                                //         }
                                //     })
                                // })
                            } else {
                                reject(err);
                            }
                        })
                    }

                }

            } catch (e) {
                reject("data_not_found");
            }
        });
    },
    editGoalFriendsWith: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                if (req.share_with_all == '1') {
                    followersModel.aggregate([
                        {
                            $match: {
                                follower_id: req.user_id,
                                status: 'Accept'
                            }
                        },
                        {
                            $lookup: {
                                from: 'followers',
                                let: { authorID: '$following_id' },
                                pipeline: [
                                    {
                                        $match: {
                                            $expr: {
                                                $and: [
                                                    { $eq: ['$following_id', req.user_id] }, // Check if the current user follows the author
                                                    { $eq: ['$follower_id', '$$authorID'] },
                                                    { $eq: ['$status', 'Accept'] }
                                                ],
                                            },
                                        },
                                    },
                                ],
                                as: 'reverseFollow',
                            },
                        },
                        {
                            $lookup: {
                                from: "users",
                                localField: "following_id",
                                foreignField: "_id",
                                as: "followingUserInfo"
                            }
                        },
                        {
                            $unwind: "$followingUserInfo"
                        },
                        {
                            $match: {
                                'reverseFollow.status': 'Accept',
                            }
                        },
                        {
                            $project: {
                                username: '$followingUserInfo.username',
                                user_id: '$followingUserInfo._id',
                            }
                        },
                    ]).then(_results => {
                        if (_results.length > 0) {
                            goalRequestModel.deleteMany({ goal_id: req.goal_id, user_id: req.user_id, request_user_id: { $ne: req.user_id }, status: 'Pending' }, (err, result) => {
                                if (!err) {
                                    asyncLoop(_results, async (item, next) => {
                                        let findUser = await goalRequestModel.find({
                                            user_id: req.user_id,
                                            goal_id: req.goal_id,
                                            request_user_id: item.user_id
                                        })
                                        if (findUser.length > 0) {
                                            next();
                                        } else {
                                            goalRequestModel.create({
                                                user_id: req.user_id,
                                                goal_id: req.goal_id,
                                                request_user_id: item.user_id
                                            }, async (err, _result) => {
                                                if (!err) {
                                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                                    // send notification to the owner of the post
                                                    let notifi_data = {
                                                        receiver_id: item.user_id,
                                                        sender_id: req.user_id,
                                                        title: "Neshwa",
                                                        message: userData.username + " invite you to see their goal & dream",
                                                        tag: "goals_request",
                                                        image_url: "",
                                                        goal_id: req.goal_id,
                                                        goal_request_id: _result._id,
                                                    }
                                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                                        if (err) {
                                                            console.log("err", err)
                                                            next();
                                                        } else {
                                                            console.log("data", data)
                                                            next();
                                                        }
                                                    })
                                                } else {
                                                    next(err);
                                                    return;
                                                }
                                            })
                                        }
                                    }, (err) => {
                                        if (err) {
                                            reject(err);
                                        } else {
                                            resolve(true);
                                        }
                                    });
                                } else {
                                    reject(err);
                                }
                            })
                        } else {
                            resolve(true)
                        }
                    })
                } else {
                    if (req.share_with_users.length > 0) {
                        goalRequestModel.deleteMany({ goal_id: req.goal_id, user_id: req.user_id, request_user_id: { $ne: req.user_id }, status: 'Pending' }, (err, result) => {
                            if (!err) {
                                asyncLoop(req.share_with_users, async (item, next) => {
                                    let findUser = await goalRequestModel.find({
                                        user_id: req.user_id,
                                        goal_id: req.goal_id,
                                        request_user_id: item
                                    })
                                    if (findUser.length > 0) {
                                        next();
                                    } else {
                                        goalRequestModel.create({
                                            user_id: req.user_id,
                                            goal_id: req.goal_id,
                                            request_user_id: item
                                        }, async (err, _result) => {
                                            if (!err) {
                                                let userData = await common.userDetails({ receiver_id: req.user_id });
                                                // send notification to the owner of the post
                                                let notifi_data = {
                                                    receiver_id: _result.request_user_id,
                                                    sender_id: req.user_id,
                                                    title: "Neshwa",
                                                    message: userData.username + " invite you to see their goal & dream",
                                                    tag: "goals_request",
                                                    image_url: "",
                                                    goal_id: req.goal_id,
                                                    goal_request_id: _result._id,
                                                }
                                                common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                                    if (err) {
                                                        console.log("err", err)
                                                        next();
                                                    } else {
                                                        console.log("data", data)
                                                        next();
                                                    }
                                                })
                                            } else {
                                                next(err);
                                                return;
                                            }
                                        })
                                    }
                                }, (err) => {
                                    if (err) {
                                        reject(err);
                                    } else {
                                        resolve(true);
                                    }
                                });
                                // req.share_with_users.map((item) => {
                                //     goalRequestModel.create({
                                //         user_id: req.user_id,
                                //         goal_id: req.goal_id,
                                //         request_user_id: item
                                //     }, async (err, _result) => {
                                //         if (!err) {
                                //             let userData = await common.userDetails({ receiver_id: req.user_id });
                                //             // send notification to the owner of the post
                                //             let notifi_data = {
                                //                 receiver_id: _result.request_user_id,
                                //                 sender_id: req.user_id,
                                //                 title: "Neshwa",
                                //                 message: userData.username + " invite you to see their goal & dream.",
                                //                 tag: "goals_request",
                                //                 image_url: "",
                                //                 goal_id: req.goal_id,
                                //                 goal_request_id: _result._id,
                                //             }
                                //             common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                //                 if (err) {
                                //                     console.log("err", err)
                                //                     resolve(_result);
                                //                 } else {
                                //                     console.log("data", data)
                                //                     resolve(_result);
                                //                 }
                                //             })
                                //         } else {
                                //             reject(err);
                                //         }
                                //     })
                                // })
                            } else {
                                reject(err);
                            }
                        })
                    }

                }

            } catch (e) {
                reject("data_not_found");
            }
        });
    },
    addGoalFollowingWith: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                if (req.share_with_all == '1') {
                    let followingWith = await followersModel.find({
                        follower_id: req.user_id,
                        is_delete: '0',
                        status: 'Accept'
                    }, { following_id: 1 });
                    if (followingWith.length > 0) {
                        asyncLoop(followingWith, (item, next) => {
                            goalRequestModel.create({
                                user_id: req.user_id,
                                goal_id: req.goal_id,
                                request_user_id: item.following_id
                            }, async (err, _result) => {
                                if (!err) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the post
                                    let notifi_data = {
                                        receiver_id: item.following_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " invite you to see their goal & dream",
                                        tag: "goals_request",
                                        image_url: "",
                                        goal_id: req.goal_id,
                                        goal_request_id: _result._id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                            next();
                                        } else {
                                            console.log("data", data)
                                            next();
                                        }
                                    })
                                } else {
                                    next(err);
                                    return;
                                }
                            })
                        }, () => {
                            resolve(true)
                        })
                    } else {
                        resolve(true)
                    }
                } else {
                    if (req.share_with_users.length > 0) {
                        asyncLoop(req.share_with_users, async (item, next) => {
                            let findUser = await goalRequestModel.find({
                                user_id: req.user_id,
                                goal_id: req.goal_id,
                                request_user_id: item.following_id,
                                status: { $ne: 'Accept' }
                            })
                            if (findUser.length > 0) {
                                next();
                            } else {
                                goalRequestModel.create({
                                    user_id: req.user_id,
                                    goal_id: req.goal_id,
                                    request_user_id: item
                                }, async (err, _result) => {
                                    if (!err) {
                                        let userData = await common.userDetails({ receiver_id: req.user_id });
                                        // send notification to the owner of the post
                                        let notifi_data = {
                                            receiver_id: item,
                                            sender_id: req.user_id,
                                            title: "Neshwa",
                                            message: userData.username + " invite you to see their goal & dream",
                                            tag: "goals_request",
                                            image_url: "",
                                            goal_id: req.goal_id,
                                            goal_request_id: _result._id,
                                        }
                                        common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                            if (err) {
                                                console.log("err", err)
                                                next();
                                            } else {
                                                console.log("data", data)
                                                next();
                                            }
                                        })
                                    } else {
                                        next(err);
                                        return;
                                    }
                                })
                            }
                        }, (err) => {
                            if (err) {
                                reject(err);
                            } else {
                                resolve(true);
                            }
                        });
                    }

                }

            } catch (e) {
                reject("data_not_found");
            }
        });
    },
    editGoalFollowingWith: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                if (req.share_with_all == '1') {
                    let followingWith = await followersModel.find({
                        follower_id: req.user_id,
                        is_delete: '0',
                        status: 'Accept'
                    }, { following_id: 1 });
                    if (followingWith.length > 0) {
                        goalRequestModel.deleteMany({ goal_id: req.goal_id, user_id: req.user_id, request_user_id: { $ne: req.user_id }, status: 'Pending' }, (err, result) => {
                            if (!err) {
                                asyncLoop(followingWith, async (item, next) => {
                                    let findUser = await goalRequestModel.find({
                                        user_id: req.user_id,
                                        goal_id: req.goal_id,
                                        request_user_id: item.following_id
                                    })
                                    if (findUser.length > 0) {
                                        next();
                                    } else {
                                        goalRequestModel.create({
                                            user_id: req.user_id,
                                            goal_id: req.goal_id,
                                            request_user_id: item.following_id
                                        }, async (err, _result) => {
                                            if (!err) {
                                                let userData = await common.userDetails({ receiver_id: req.user_id });
                                                // send notification to the owner of the post
                                                let notifi_data = {
                                                    receiver_id: item.following_id,
                                                    sender_id: req.user_id,
                                                    title: "Neshwa",
                                                    message: userData.username + " invite you to see their goal & dream",
                                                    tag: "goals_request",
                                                    image_url: "",
                                                    goal_id: req.goal_id,
                                                    goal_request_id: _result._id,
                                                }
                                                common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                                    if (err) {
                                                        console.log("err", err)
                                                        next();
                                                    } else {
                                                        console.log("data", data)
                                                        next();
                                                    }
                                                })
                                            } else {
                                                next(err);
                                                return;
                                            }
                                        })
                                    }
                                }, (err) => {
                                    if (err) {
                                        reject(err);
                                    } else {
                                        resolve(true);
                                    }
                                });
                            } else {
                                reject(err);
                            }
                        })
                    } else {
                        resolve(true)
                    }
                } else {
                    if (req.share_with_users.length > 0) {
                        goalRequestModel.deleteMany({ goal_id: req.goal_id, user_id: req.user_id, request_user_id: { $ne: req.user_id }, status: 'Pending' }, (err, result) => {
                            if (!err) {
                                asyncLoop(req.share_with_users, async (item, next) => {
                                    let findUser = await goalRequestModel.find({
                                        user_id: req.user_id,
                                        goal_id: req.goal_id,
                                        request_user_id: item.following_id
                                    })
                                    if (findUser.length > 0) {
                                        next();
                                    } else {
                                        goalRequestModel.create({
                                            user_id: req.user_id,
                                            goal_id: req.goal_id,
                                            request_user_id: item
                                        }, async (err, _result) => {
                                            if (!err) {
                                                let userData = await common.userDetails({ receiver_id: req.user_id });
                                                // send notification to the owner of the post
                                                let notifi_data = {
                                                    receiver_id: item,
                                                    sender_id: req.user_id,
                                                    title: "Neshwa",
                                                    message: userData.username + " invite you to see their goal & dream",
                                                    tag: "goals_request",
                                                    image_url: "",
                                                    goal_id: req.goal_id,
                                                    goal_request_id: _result._id,
                                                }
                                                common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                                    if (err) {
                                                        console.log("err", err)
                                                        next();
                                                    } else {
                                                        console.log("data", data)
                                                        next();
                                                    }
                                                })
                                            } else {
                                                next(err);
                                                return;
                                            }
                                        })
                                    }
                                }, (err) => {
                                    if (err) {
                                        reject(err);
                                    } else {
                                        resolve(true);
                                    }
                                });
                            } else {
                                reject(err);
                            }
                        })
                    }

                }

            } catch (e) {
                reject("data_not_found");
            }
        });
    },
    /* funcion for accept or rejct request */
    acceptGoalsInvitation: async (req, callback) => {
        try {
            console.log("acceptGoalsInvitation", req)
            let share_withData = await goalRequestModel.find({ _id: req.goal_request_id });
            if (share_withData.length > 0) {
                console.log("share_withData", share_withData[0])
                let updateStatus = await goalRequestModel.findOneAndUpdate({ _id: req.goal_request_id }, {
                    status: 'Accept'
                });
                let sendNotification = await goals_model.sendNotification({ user_id: req.user_id, receiver_id: share_withData[0].user_id, goal_request_id: req.goal_request_id, tag: 'accept_goal_request' })

                callback(responseCode.SUCCESS, t('share_with_request_accepted'), {})
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
            }
        } catch (error) {
            console.log("error1062", error)
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
        }
    },
    sendNotification: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                let userData = await common.userDetails({ receiver_id: req.user_id });
                // send notification to the owner of the post
                let notifi_data = {
                    receiver_id: req.receiver_id,
                    sender_id: req.user_id,
                    title: "Neshwa",
                    tag: req.tag,
                    image_url: ""
                }

                notifi_data.message = userData.username + " has accept your invitation request.";
                notifi_data.goal_request_id = req.goal_request_id
                common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                    if (err) {
                        console.log("err", err)
                        reject("something_went_wrong");
                    } else {
                        console.log("data", data)
                        resolve(true);
                    }
                })
            } catch (e) {
                console.log("e", e)
                reject("something_went_wrong");
            }
        });
    },
    rejectGoalsInvitation: async (req, callback) => {
        try {
            let requestId = mongoose.Types.ObjectId(req.goal_request_id);
            let share_withData = await goalRequestModel.find({ _id: requestId });
            if (share_withData.length > 0) {
                let updateStatus = await goalRequestModel.update({ _id: requestId }, { status: 'Reject' });
                callback(responseCode.SUCCESS, t('share_with_request_rejected'), {})
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
            }
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
        }
    },
    /**
     * @default Api method used for add progress notes 
     */
    addProgressNotes: (req, callback) => {
        try {
            progressNoteModel.create({
                goal_id: req.goal_id,
                progress_note: req.progress_note
            }, (_err, _result) => {
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, null);
                } else {
                    callback(responseCode.SUCCESS, t('progress_note_added'), _result);
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
        }
    },
    calculateElapsedPercentage: (startDateTime, endDateTime) => {
        const start = moment(startDateTime);
        const end = moment(endDateTime);
        const now = moment();

        if (now.isBefore(start)) {
            return 0;
        }

        if (now.isAfter(end)) {
            return 100;
        }

        const totalDuration = end.diff(start);
        const elapsedDuration = now.diff(start);

        const percentage = ((elapsedDuration / totalDuration) * 100).toFixed(2);
        return percentage;
    },

    sendPostNotificationAllFollowing: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                let followingUsers = goals_model.getFollowUsers(req);
                if (followingUsers.length > 0) {
                    asyncLoop(followingUsers, async (items, next) => {
                        let userData = await common.userDetails({ receiver_id: req.user_id });
                        // send notification to the owner of the post
                        let notifi_data = {
                            receiver_id: items,
                            sender_id: req.user_id,
                            title: "Neshwa",
                            message: userData.username + " has create a new post",
                            tag: "add_post",
                            image_url: "",
                            post_id: req.post_id,
                        }
                        common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                            if (err) {
                                console.log("err", err)
                                next()
                            } else {
                                console.log("data", data)
                                next()
                            }
                        })
                    }, () => {
                        resolve(true);
                    })
                } else {
                    resolve(true);
                }
            } catch (e) {
                reject("something_went_wrong");
            }
        });
    },

};

module.exports = goals_model