var express = require('express');
var router = express.Router();
const Validator = require('Validator');
const goals_model = require('./goals_model')
var common = require('../../../utilities/common')
var validate_header = require('../../../middleware/validate-header')
var uniqueMiddleware = require('../../../middleware/uniqueMiddleware')
const httpStatusCode = require('http-status-codes');
const { t } = require('localizify');
/* routes are starting here */

router.post('/goalCategoriesList', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    goals_model.goalCategoriesList(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(req, res, response_data);
    })
})

router.post('/addGoalCategory', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        name: "required",
        image: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        goals_model.addGoalCategory(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/goalDelete', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        goal_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        goals_model.goalDelete(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/categoryDetail', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        goal_category_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        goals_model.categoryDetail(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/editGoalsAndDreams', (req, res) => {
    var request = req.body;
    // console.log("request", request)
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        goal_id: "required",
        goal_title: "required",
        goal_type: "required",
        start_date: "required",
        end_date: "required",
        privacy: "required",
        goal_category_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        goals_model.editGoalsAndDreams(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/addGoalsAndDreams', (req, res) => {
    var request = req.body;
    // console.log("request", request)
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        goal_title: "required",
        goal_type: "required",
        start_date: "required",
        end_date: "required",
        privacy: "required",
        goal_category_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        // console.log("errors", response_data)

        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        // console.log("request", request)

        goals_model.addGoalsAndDreams(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/acceptGoalsInvitation', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        goal_request_id: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        goals_model.acceptGoalsInvitation(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/rejectGoalsInvitation', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        goal_request_id: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        goals_model.rejectGoalsInvitation(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/addProgressNotes', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        progress_note: "required",
        goal_id: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        goals_model.addProgressNotes(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/goalDetail', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        goal_id: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        goals_model.goalDetail(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getGoalCategoryUsers', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        goal_category_id: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        goals_model.getGoalCategoryUsers(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/goalDreamRequestList', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    goals_model.goalDreamRequestList(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(req, res, response_data);
    })
})

router.post('/goalsAndDreamsList', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    goals_model.goalsAndDreamsList(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(req, res, response_data);
    })
})

module.exports = router;