var express = require('express');
var router = express.Router();
const Validator = require('Validator');
const social_model = require('./social_model')
var common = require('../../../utilities/common')
var validate_header = require('../../../middleware/validate-header')
var uniqueMiddleware = require('../../../middleware/uniqueMiddleware')
const httpStatusCode = require('http-status-codes');
const { t } = require('localizify');
/* routes are starting here */

router.post('/addFolder', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        name: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.addFolder(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/editFolder', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        name: "required",
        folder_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.editFolder(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/deleteFolder', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        folder_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.deleteFolder(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/folderList', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    social_model.folderList(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(req, res, response_data);
    })
})

router.post('/getPost', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    console.log(request)
    social_model.getPost(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(req, res, response_data);
    })
})

router.post('/addPost', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    console.log("request", request)
    var rules = {
        type: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.addPost(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/addComment', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    console.log("request", request)
    var rules = {
        post_id: "required",
        message: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.addComment(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/deleteMyComment', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        comment_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.deleteMyComment(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getComment', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    console.log("request", request)
    var rules = {
        post_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.getComment(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/postDetail', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        post_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.postDetail(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/deleteMyPost', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        post_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.deleteMyPost(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getTagUserList', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        post_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.getTagUserList(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getFollowFollowingUserList', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    social_model.getFollowFollowingUserList(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(req, res, response_data);
    })
})

router.post('/getPostViewer', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        post_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.getPostViewer(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getLikeUsersList', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        post_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.getLikeUsersList(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/likePost', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        post_id: "required",
        like_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.likePost(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            // console.log("response_data", response_data)
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/acceptCollabroteRequest', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        collaborate_id: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        social_model.acceptCollabroteRequest(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/rejectCollabroteRequest', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        collaborate_id: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        social_model.rejectCollabroteRequest(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/unlikePost', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        post_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.unlikePost(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/likeComment', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        comment_id: "required",
        like_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.likeComment(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/unlikeComment', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        comment_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.unlikeComment(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/addCollection', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        name: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.addCollection(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/editCollection', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        name: "required",
        collection_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.editCollection(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/deleteCollection', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        collection_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.deleteCollection(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/collectionList', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    social_model.collectionList(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(req, res, response_data);
    })
})

router.post('/recentLocations', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    social_model.recentLocations(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(req, res, response_data);
    })
})

router.post('/getPostFromCollection', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        collection_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        social_model.getPostFromCollection(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})
router.post('/savePostInCollection', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        collection_id: "required",
        post_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.savePostInCollection(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/deletePostFromCollection', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        post_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.deletePostFromCollection(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})


router.post('/addStory', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        medias: "required|array",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        console.log("addStory=> ", request)
        social_model.addStory(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getStory', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    social_model.getStory(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(req, res, response_data);
    })
})


router.post('/storyDetail', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        story_user_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.storyDetail(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            // console.log("response_data", response_data)
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/storyView', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        story_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.storyView(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/likeStory', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        story_id: "required",
        like_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.likeStory(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            // console.log("response_data", response_data)
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/unlikeStory', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        story_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.unlikeStory(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/deleteStory', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        story_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.deleteStory(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getStoryViewer', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        story_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.getStoryViewer(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getStoryLikeUsersList', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        story_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.getStoryLikeUsersList(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})
router.post('/reportPost', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        post_id: "required",
        reason_id: "required"

    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.reportPost(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})
router.post('/reportStory', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        story_id: "required",
        reason_id: "required",
        description: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.reportStory(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})
router.post('/editStoryPrivacy', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        story_id: "required",
        privacy: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.editStoryPrivacy(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})
/* Routes for Reels */

router.post('/addReel', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        reels: "required|array",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        console.log("addReel=> ", request)
        social_model.addReel(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getReels', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    social_model.getReels(request, (code, message, response) => {
        var response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(req, res, response_data);
    })
})

router.post('/reelDetail', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        reel_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {
        social_model.reelDetail(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/reelView', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        reel_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.reelView(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/likeReel', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        reel_id: "required",
        like_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.likeReel(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            // console.log("response_data", response_data)
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/unlikeReel', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        reel_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.unlikeReel(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/deleteReel', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        reel_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.deleteReel(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getReelViewer', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        reel_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.getReelViewer(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getReelLikeUsersList', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        reel_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.getReelLikeUsersList(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/addReelComment', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    console.log("request", request)
    var rules = {
        reel_id: "required",
        message: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.addReelComment(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/deleteMyReelComment', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        comment_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.deleteMyReelComment(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/getReelComment', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    console.log("request", request)
    var rules = {
        reel_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.getReelComment(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/likeReelComment', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        comment_id: "required",
        like_id: "required"
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.likeReelComment(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})

router.post('/unlikeReelComment', (req, res) => {
    var request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    var rules = {
        comment_id: "required",
    };
    var message = {
        required: t('field_requird')
    }
    var v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        for (var key in errors) {
            var error = errors[key][0];
            break;
        }
        var response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(req, res, response_data, httpStatusCode.OK);
    } else {

        social_model.unlikeReelComment(request, (code, message, response) => {
            var response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(req, res, response_data);
        })
    }
})
module.exports = router;