const responseCode = require('../../../config/response-error-code');
const common = require('../../../utilities/common');
const validate_header = require('../../../middleware/validate-header');
const constant = require('../../../config/constant');
const moment = require('moment');
const asyncLoop = require('node-async-loop')
const userModel = require('../../../model/user');
const folderModel = require('../../../model/folders')
const postsModel = require('../../../model/posts')
const postsHashtag = require('../../../model/post_hashtags')
const postLike = require('../../../model/post_like')
const storyLike = require('../../../model/story_like')
const reelLike = require('../../../model/reel_like')
const storyReportModel = require('../../../model/story_report')
const postReportModel = require('../../../model/post_report')
const postMediaModel = require('../../../model/post_media')
const postCollaboratorModel = require('../../../model/post_collaborator_users')
const commentLikeModel = require('../../../model/comment_like')
const reelCommentLikeModel = require('../../../model/reel_comment_like')
const postComments = require('../../../model/post_comments')
const reelComments = require('../../../model/reel_comments')
const postTagUsers = require('../../../model/post_tag_users')
const postCollaboratorUsers = require('../../../model/post_collaborator_users')
const postCategory = require('../../../model/post_category')
const PostViewer = require('../../../model/post_viewers')
const StoryViewer = require('../../../model/story_viewers')
const reelViewer = require('../../../model/reel_viewers')
const collectionModel = require('../../../model/collections')
const reactionModel = require('../../../model/reactions')
const collectionPostModel = require('../../../model/collection_posts')
const folderPostModel = require('../../../model/folder_posts')
const followersModel = require('../../../model/followers')
const storyModel = require('../../../model/stories')
const reelModel = require('../../../model/reels')
const notiLibrary = require('../../../config/notification')
const mongoose = require('mongoose');
const { t } = require('localizify');
const _ = require('lodash');
let social_model = {
    /**
     * @description API method for folder listing
     */
    folderList: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        let response_data = [];
        let condition = { user_id: req.user_id, is_active: "1", is_delete: "0", is_default: req.is_default }
        if (req.search != undefined && req.search != '') {
            condition.name = { $regex: '.*' + req.search + '.*', $options: 'i' };
        }
        folderModel.find(condition).skip(start)
            .limit(limit)
            .exec((_err, _result) => {
                // Your code here
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, []);
                } else {
                    if (_result.length > 0) {
                        asyncLoop(_result, async (item, next) => {
                            let count = await folderPostModel.countDocuments({ user_id: req.user_id, folder_id: item._id });
                            if (count > 0) {
                                let last_post_in_folder = await folderPostModel.findOne({ user_id: req.user_id, folder_id: item._id }).sort({ createdAt: -1 });
                                let last_post = await postMediaModel.find({ post_id: last_post_in_folder.post_id });
                                if (last_post.length > 0) {
                                    let folder_image = (last_post[0].type == 'image') ? constant.s3Bucket_post_media + last_post[0].media_url : constant.s3Bucket_post_media + last_post[0].thumb_image;
                                    response_data.push({
                                        _id: item._id,
                                        user_id: item.user_id,
                                        name: item.name,
                                        is_active: item.is_active,
                                        is_delete: item.is_delete,
                                        createdAt: item.createdAt,
                                        updatedAt: item.updatedAt,
                                        folder_image: folder_image,
                                        postCount: count,
                                    })
                                    next();
                                } else {
                                    response_data.push({
                                        _id: item._id,
                                        user_id: item.user_id,
                                        name: item.name,
                                        is_active: item.is_active,
                                        is_delete: item.is_delete,
                                        createdAt: item.createdAt,
                                        updatedAt: item.updatedAt,
                                        folder_image: "",
                                        postCount: count,
                                    })
                                    next();
                                }
                            } else {
                                response_data.push({
                                    _id: item._id,
                                    user_id: item.user_id,
                                    name: item.name,
                                    is_active: item.is_active,
                                    is_delete: item.is_delete,
                                    createdAt: item.createdAt,
                                    updatedAt: item.updatedAt,
                                    folder_image: "",
                                    postCount: count,
                                })
                                next();
                            }

                        }, () => {
                            callback(responseCode.SUCCESS, t('data_found'), response_data);
                        })
                    } else {
                        callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                    }
                }
            })
    },
    /**
     * @description API method for creating folder
     */
    addFolder: (req, callback) => {
        let addData = {
            name: req.name,
            user_id: req.user_id,
            is_default: "0"
        }
        folderModel.create(addData, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('folder_added'), {});
            }
        })
    },
    /**
     * @description API method for edit folder
     */
    editFolder: (req, callback) => {
        let addData = {
            name: req.name
        }
        let folderID = mongoose.Types.ObjectId(req.folder_id);
        folderModel.updateOne({ _id: folderID }, addData, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('folder_edited'), {});
            }
        })
    },
    /**
     * @description API method for delete folder
     */
    deleteFolder: (req, callback) => {
        let folderID = mongoose.Types.ObjectId(req.folder_id);
        folderModel.updateOne({ _id: folderID }, { is_delete: "1" }, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('folder_deleted'), {});
            }
        })
    },
    addPost: async (req, callback) => {
        let data = {
            user_id: req.user_id
        };
        if (req.privacy != undefined && req.privacy != '') {
            data.privacy = req.privacy;
        }
        if (req.description != undefined && req.description != '') {
            data.description = req.description;
        }
        if (req.profile_id != undefined && req.profile_id != '') {
            data.profile_id = req.profile_id;
        }
        if (req.is_color_background != undefined && req.is_color_background) {
            data.is_color_background = req.is_color_background;
            data.color_id = req.color_id;
            data.color_name = req.color_name;
        } else {
            data.is_color_background = false;
        }
        if (req.location_latitude != undefined && req.location_latitude != '') {
            data.location_latitude = req.location_latitude;
            data.location_longitude = req.location_longitude;
            data.location_full_address = req.location_full_address;
            data.location = req.location;
        }
        postsModel.create(data, async (err, result) => {
            if (!err) {
                try {
                    let folder_data = {
                        user_id: req.user_id,
                        post_id: result._id,
                    }
                    /* add post in default folder  */
                    let addPostFolder = await social_model.addPostInDefaultFolder(folder_data);
                    if (req.folder_id != undefined && req.folder_id != '') {
                        folder_data.folder_id = req.folder_id;
                        let addPostAnotherFolder = await social_model.addPostInFolder(folder_data);

                    }
                    /* add media to post */
                    if (req.media != undefined && req.media.length > 0) {
                        let media_data = {
                            post_id: result._id,
                            user_id: req.user_id,
                            media: req.media
                        }
                        let add_media = await social_model.addMediaToPost(media_data);
                    }
                    /* add hashtag to text post */
                    if (req.hashtag_ids != undefined && req.hashtag_ids.length > 0) {
                        let hashtag_data = {
                            post_id: result._id,
                            user_id: req.user_id,
                            type: 'Post',
                            hashtag_ids: req.hashtag_ids
                        }
                        let add_hashtag = await social_model.addHashTags(hashtag_data);
                    }
                    /* add post tag users to text post */
                    if (req.post_tag_users != undefined && req.post_tag_users.length > 0) {
                        let posttag_user_data = {
                            post_id: result._id,
                            user_id: req.user_id,
                            type: 'Post',
                            post_tag_users: req.post_tag_users
                        }
                        let add_posttag_user = await social_model.addPostTagUser(posttag_user_data);
                        let sendNotifi = await social_model.sendPostNotificationAllTagged(posttag_user_data);
                    }
                    /* add post collaborator users to text post */
                    if (req.post_collaborator_users != undefined && req.post_collaborator_users.length > 0) {
                        let post_collab_user_data = {
                            post_id: result._id,
                            user_id: req.user_id,
                            type: 'Post',
                            post_collaborator_users: req.post_collaborator_users
                        }
                        let add_post_collab_user = await social_model.addPostCollaboratorsUser(post_collab_user_data);
                    }
                    /* add post categories to text post */
                    if (req.post_category_id != undefined && req.post_category_id.length > 0) {
                        let post_category_data = {
                            post_id: result._id,
                            user_id: req.user_id,
                            type: 'Post',
                            post_category_id: req.post_category_id
                        }
                        let add_posttag_user = await social_model.addPostCategory(post_category_data);

                    }
                    /* send notification to following users */
                    req.post_id = result._id;
                    let sendNotifi = await social_model.sendPostNotificationAllFollowing(req);
                    callback(responseCode.SUCCESS, t('post_added'), null);
                } catch (error) {
                    console.log("addHashTags Error : ", error);
                    callback(responseCode.OPERATION_FAILED, t('somthing_went_wrong'), null)
                }
            } else {
                console.log(err)
                callback(responseCode.OPERATION_FAILED, t('somthing_went_wrong'), null)
            }
        })
    },
    sendPostNotificationAllFollowing: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                let followingUsers = social_model.getFollowUsers(req);
                if (followingUsers.length > 0) {
                    asyncLoop(followingUsers, async (items, next) => {
                        let userData = await common.userDetails({ receiver_id: req.user_id });
                        // send notification to the owner of the post
                        let notifi_data = {
                            receiver_id: items,
                            sender_id: req.user_id,
                            title: "Neshwa",
                            message: userData.username + " has create a new post",
                            tag: "add_post",
                            image_url: "",
                            post_id: req.post_id,
                        }
                        common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                            if (err) {
                                console.log("err", err)
                                next()
                            } else {
                                console.log("data", data)
                                next()
                            }
                        })
                    }, () => {
                        resolve(true);
                    })
                } else {
                    resolve(true);
                }
            } catch (e) {
                reject("something_went_wrong");
            }
        });
    },
    sendPostNotificationAllCollaborator: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {

                if (req.post_collaborator_users.length > 0) {
                    asyncLoop(req.post_collaborator_users, async (items, next) => {
                        let userData = await common.userDetails({ receiver_id: req.user_id });
                        // send notification to the owner of the post
                        let notifi_data = {
                            receiver_id: items,
                            sender_id: req.user_id,
                            title: "Neshwa",
                            // message: userData.username + "You've been added as a collaborator on a post",
                            message: "You've been added as a collaborator on a post.",
                            tag: "add_post_collaborator_user",
                            image_url: "",
                            post_id: req.post_id,
                        }
                        common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                            if (err) {
                                console.log("err", err)
                                next()
                            } else {
                                console.log("data", data)
                                next()
                            }
                        })
                    }, () => {
                        resolve(true);
                    })
                } else {
                    resolve(true);
                }
            } catch (e) {
                reject("something_went_wrong");
            }
        });
    },
    sendPostNotificationAllTagged: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {

                if (req.post_tag_users.length > 0) {
                    asyncLoop(req.post_tag_users, async (items, next) => {
                        let userData = await common.userDetails({ receiver_id: req.user_id });
                        // send notification to the owner of the post
                        let notifi_data = {
                            receiver_id: items,
                            sender_id: req.user_id,
                            title: "Neshwa",
                            message: userData.username + " tagged you in post.",
                            tag: "add_post_tagged_user",
                            image_url: "",
                            post_id: req.post_id,
                        }
                        common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                            if (err) {
                                console.log("err", err)
                                next()
                            } else {
                                console.log("data", data)
                                next()
                            }
                        })
                    }, () => {
                        resolve(true);
                    })
                } else {
                    resolve(true);
                }
            } catch (e) {
                reject("something_went_wrong");
            }
        });
    },
    addMediaToPost: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {

                postMediaModel.deleteMany({ post_id: req.post_id, user_id: req.user_id }, (err, result) => {
                    if (!err) {
                        let addData = [];
                        req.media.map((item) => {
                            addData.push({ user_id: req.user_id, post_id: req.post_id, type: item.type, media_url: item.media_url, index_number: parseInt(item.index_number), thumb_image: ((item.thumb_image != '') ? item.thumb_image : '') });
                        })
                        postMediaModel.create(addData, (err, _post_media_result) => {
                            if (!err) {
                                resolve(_post_media_result);
                            } else {
                                reject(err);
                            }
                        })
                    } else {
                        reject(err);
                    }
                })

            } catch (error) {
                reject(error);
            }
        });
    },
    addPostInFolder: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                let folder_post = await folderPostModel.create({
                    user_id: req.user_id,
                    folder_id: req.folder_id,
                    post_id: req.post_id
                });
                resolve(folder_post);
            } catch (error) {
                reject(error);
            }
        });
    },
    addPostInDefaultFolder: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                folderModel.find({ name: '' + moment().format('YYYY') + '', user_id: req.user_id, is_default: "1" }, async (err, _folder) => {
                    if (err) {
                        reject(err);
                    } else if (_folder.length > 0) {
                        let folder_post = await folderPostModel.create({
                            user_id: req.user_id,
                            folder_id: _folder[0]._id,
                            post_id: req.post_id
                        });
                        resolve(folder_post);
                    } else {
                        let folder_data = await folderModel.create({
                            user_id: req.user_id,
                            name: "" + moment().format('YYYY'),
                            is_default: "1",
                        });
                        let folder_post = await folderPostModel.create({
                            user_id: req.user_id,
                            folder_id: folder_data._id,
                            post_id: req.post_id
                        });
                        resolve(folder_post);
                    }
                })
            } catch (error) {
                reject(error);
            }
        });
    },
    addHashTags: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                postsHashtag.updateOne({ post_id: req.post_id, user_id: req.user_id }, { is_delete: 1 }, (err, result) => {
                    if (!err) {
                        let addData = [];
                        req.hashtag_ids.map((item) => {
                            addData.push({ user_id: req.user_id, post_id: req.post_id, type: req.type, hashtag_id: item });
                        })
                        postsHashtag.create(addData, (err, _post_hashtags_result) => {
                            if (!err) {
                                resolve(_post_hashtags_result);
                            } else {
                                reject(err);
                            }
                        })
                    } else {
                        reject(err);
                    }
                })
            } catch (e) {
                reject("data_not_found");
            }
        });

    },
    addPostCollaboratorsUser: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                postCollaboratorUsers.deleteMany({ post_id: req.post_id, user_id: req.user_id }, (err, result) => {
                    if (!err) {
                        let addData = [];
                        req.post_collaborator_users.map((item) => {
                            postCollaboratorUsers.create({
                                user_id: req.user_id,
                                post_id: req.post_id,
                                collaborator_user_id: item
                            }, async (err, _post_Collaborator_result) => {
                                if (!err) {

                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the post
                                    let notifi_data = {
                                        receiver_id: item,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has requested you as a collaborator on a post.",
                                        tag: "add_post_collaborator_user",
                                        image_url: "",
                                        post_id: req.post_id,
                                        collaborate_id: _post_Collaborator_result._id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                            resolve(_post_Collaborator_result);
                                        } else {
                                            console.log("data", data)
                                            resolve(_post_Collaborator_result);
                                        }
                                    })
                                } else {
                                    reject(err);
                                }
                            })
                        })
                    } else {
                        reject(err);
                    }
                })
            } catch (e) {
                reject("data_not_found");
            }
        });
    },
    acceptCollabroteRequest: async (req, callback) => {
        try {
            console.log("acceptCollabroteRequest", req)
            let collaborateData = await postCollaboratorUsers.find({ _id: req.collaborate_id });
            if (collaborateData.length > 0) {
                console.log("collaborateData", collaborateData[0])
                let updateStatus = await postCollaboratorUsers.findOneAndUpdate({ _id: req.collaborate_id }, {
                    status: 'Accept'
                });
                let sendNotification = await social_model.sendNotification({ user_id: req.user_id, receiver_id: collaborateData[0].collaborator_user_id, collaborate_id: req.collaborate_id, tag: 'accept_collaborate_request' })

                callback(responseCode.SUCCESS, t('collaborate_request_accepted'), {})
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
            }
        } catch (error) {
            console.log("error1062", error)
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
        }
    },
    sendNotification: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                let userData = await common.userDetails({ receiver_id: req.user_id });
                // send notification to the owner of the post
                let notifi_data = {
                    receiver_id: req.receiver_id,
                    sender_id: req.user_id,
                    title: "Neshwa",
                    tag: req.tag,
                    image_url: ""
                }

                notifi_data.message = userData.username + " has accept your collaborate request.";
                notifi_data.collaborate_id = req.collaborate_id
                common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                    if (err) {
                        console.log("err", err)
                        reject("something_went_wrong");
                    } else {
                        console.log("data", data)
                        resolve(true);
                    }
                })
            } catch (e) {
                console.log("e", e)
                reject("something_went_wrong");
            }
        });
    },
    rejectCollabroteRequest: async (req, callback) => {
        try {
            let collaborateData = await postCollaboratorUsers.find({ _id: req.collaborate_id });
            if (collaborateData.length > 0) {
                let updateStatus = await postCollaboratorUsers.deleteOne({ _id: req.collaborate_id });
                callback(responseCode.SUCCESS, t('collaborate_request_rejected'), {})
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
            }
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
        }
    },
    addPostTagUser: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                postTagUsers.deleteMany({ post_id: req.post_id, user_id: req.user_id }, (err, result) => {
                    if (!err) {
                        let addData = [];
                        req.post_tag_users.map((item) => {
                            addData.push({ user_id: req.user_id, post_id: req.post_id, tag_user_id: item });
                        })
                        postTagUsers.create(addData, (err, _post_hashtags_result) => {
                            if (!err) {
                                resolve(_post_hashtags_result);
                            } else {
                                reject(err);
                            }
                        })
                    } else {
                        reject(err);
                    }
                })
            } catch (e) {
                reject("data_not_found");
            }
        });
    },
    addPostCategory: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                postCategory.deleteMany({ post_id: req.post_id, user_id: req.user_id }, (err, result) => {
                    if (!err) {
                        let addData = [];
                        req.post_category_id.map((item) => {
                            addData.push({ user_id: req.user_id, post_id: req.post_id, category_id: item });
                        })
                        postCategory.create(addData, (err, _post_category_result) => {
                            if (!err) {
                                resolve(_post_category_result);
                            } else {
                                reject(err);
                            }
                        })
                    } else {
                        reject(err);
                    }
                })
            } catch (e) {
                reject("data_not_found");
            }
        });
    },
    /* This function is used to get users I have followed  */
    getFollowUsers: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                followersModel.find({ following_id: req.user_id, is_delete: "0", status: "Accept" }, { _id: 0, follower_id: 1 }, (err, result) => {
                    if (!err) {
                        if (result.length > 0) {
                            const userIds = result.map(user => user.follower_id);
                            resolve(userIds)
                        } else {
                            resolve([])
                        }
                    } else {
                        reject(err);
                    }
                })
            } catch (e) {
                reject(e);
            }
        });
    },
    getFollowFollowingUsers: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                followersModel.find({
                    // following_id: req.user_id,
                    $or: [
                        { following_id: req.user_id },
                        { follower_id: req.user_id },
                    ],
                    is_delete: "0",
                    status: "Accept"
                }, { _id: 0, follower_id: 1, following_id: 1 }, (err, result) => {
                    if (!err) {
                        if (result.length > 0) {
                            const userIds = result.map(user => {
                                // console.log(user)
                                if (user.follower_id.toString() == req.user_id.toString()) {
                                    return user.following_id;
                                } else {
                                    return user.follower_id;
                                }
                            });
                            resolve(userIds)
                        } else {
                            resolve([])
                        }
                    } else {
                        reject(err);
                    }
                })
            } catch (e) {
                reject(e);
            }
        });
    },
    getTagUserList: (req, callback) => {
        try {
            let postId = mongoose.Types.ObjectId(req.post_id);
            postTagUsers.aggregate([
                {
                    $match: {
                        post_id: postId
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'tag_user_id',
                        foreignField: '_id',
                        as: 'profile'

                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'tag_user_id',
                        foreignField: '_id',
                        as: 'profile'
                    }
                },
                {
                    $unwind: '$profile' // Deconstructs the array of joined documents into separate documents
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] },
                                            // { $eq: ['$status', 'Accept'] },

                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followInfo'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }

                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followStatus'
                    }
                },
                {
                    $addFields: {
                        isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } },
                        followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                    }
                },
                {
                    $project: {
                        user_id: '$tag_user_id',
                        first_name: '$profile.first_name',
                        last_name: '$profile.last_name',
                        email: '$profile.email',
                        nickname: '$profile.nickname',
                        username: '$profile.username',
                        profile_type: '$profile.profile_type',
                        isFollowed: 1,
                        followStatus: '$followStatus.status',
                        profile_image:
                        {
                            $cond: {
                                if: { $ne: ['$profile.profile_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                }, else: ''
                            }
                        }
                    }
                },
            ]).then(_users => {
                // console.log("_users", _users)
                if (_users.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _users);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, []);

        }
    },
    /**
   * @Descriprion API method for get post 
   */
    getPost: async (req, callback) => {
        console.log("getPost", req)
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            // let following_users = await social_model.getFollowUsers(req);
            // console.log("following_users", following_users)
            let condition = {
                $match: {
                    is_active: "1",
                    is_delete: "0",

                }
            }
            console.log("condition", condition)
            postsModel.aggregate([
                condition,
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'

                    }
                },
                {
                    "$unwind": "$profile"
                },
                {
                    // Lookup to join with blocked_user collection to find users blocked by current user
                    $lookup: {
                        from: 'blocked_users',
                        let: { postUserId: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $or: [
                                            { $and: [{ $eq: ['$user_id', req.user_id] }, { $eq: ['$block_user_id', '$$postUserId'] }] },
                                            { $and: [{ $eq: ['$block_user_id', req.user_id] }, { $eq: ['$user_id', '$$postUserId'] }] },
                                        ],
                                    },
                                },
                            },
                        ],
                        as: 'blockedInfo',
                    },
                },
                {
                    $lookup: {
                        from: 'post_hatshtags',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postHashtags'
                    }
                },

                {
                    $lookup: {
                        from: 'hashtags',
                        localField: 'postHashtags.hashtag_id',
                        foreignField: '_id',
                        as: 'hashtags'
                    }
                },

                {
                    $lookup: {
                        from: 'post_medias',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postMedia'
                    }
                },
                {
                    $lookup: {
                        from: 'post_likes',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'Totallikes'
                    }
                },
                {
                    $lookup: {
                        from: 'post_likes',
                        let: { postId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$user_id', req.user_id] }, // Match saved posts by user ID
                                            { $eq: ['$post_id', '$$postId'] } // Match saved posts by post ID
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { _id: 1, like_id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $lookup: {
                        from: 'post_comments',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postComments'
                    }
                },
                {
                    $lookup: {
                        from: 'post_viewers',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postViewers'
                    }
                },
                {
                    $lookup: {
                        from: 'post_tag_users',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'tagUser'
                    }
                },
                {
                    $lookup: {
                        from: 'collection_posts',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'collectionPost'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] },
                                            { $eq: ['$status', 'Accept'] },
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followerPrivacy'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$following_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$follower_id', '$$authorID'] },
                                            { $eq: ['$status', 'Accept'] }
                                        ],
                                    },
                                },
                            },
                        ],
                        as: 'reverseFollow',
                    },
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] },
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followInfo'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'followStatus'
                    },
                },
                {
                    $lookup: {
                        from: 'post_collaborator_users',
                        let: { postId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$post_id', '$$postId'] }, // Check if the current user follows the author
                                            { $eq: ['$status', 'Accept'] },
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'collaboratorUsers'
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'collaboratorUsers.collaborator_user_id',
                        foreignField: '_id',
                        as: 'collaboratorUserProfile'

                    }
                },
                // {
                //     "$unwind": "$collaboratorUserProfile"
                // },
                {
                    $lookup: {
                        from: 'post_collaborator_users',
                        let: { authorID: '$profile._id', postId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$collaborator_user_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$user_id', '$$authorID'] },
                                            { $eq: ['$post_id', '$$postId'] },
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'collaboratorInfo'
                    }
                },
                {
                    $lookup: {
                        from: 'post_collaborator_users',
                        let: { authorID: '$profile._id', postId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$collaborator_user_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$user_id', '$$authorID'] },
                                            { $eq: ['$post_id', '$$postId'] },
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { status: 1, _id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'collaboratorStatus'
                    },
                },
                {
                    $lookup: {
                        from: 'families',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$family_member_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$user_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'family'
                    }
                },
                {
                    $lookup: {
                        from: 'collection_posts',
                        let: { postId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$user_id', req.user_id] }, // Match saved posts by user ID
                                            { $eq: ['$post_id', '$$postId'] } // Match saved posts by post ID
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { _id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'savedPosts'
                    }
                },
                {
                    $addFields: {
                        totalLikes: { $size: '$Totallikes' },
                        totalComments: { $size: '$postComments' },
                        totalViewers: { $size: '$postViewers' },
                        totalSave: { $size: '$collectionPost' },
                        collaboratorCount: { $size: '$collaboratorUsers' },
                        userLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$Totallikes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } },
                        followerPrivacy: { $cond: { if: { $gt: [{ $size: '$followerPrivacy' }, 0] }, then: true, else: false } },
                        followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                        collaboratorStatus: { $cond: { if: { $gt: [{ $size: '$collaboratorStatus' }, 0] }, then: { $arrayElemAt: ['$collaboratorStatus', 0] }, else: { status: "", _id: "" } } },
                        collaboratorProfile: { $cond: { if: { $gt: [{ $size: '$collaboratorUserProfile' }, 0] }, then: { $arrayElemAt: ['$collaboratorUserProfile', 0] }, else: { _id: "", username: "", email: "", profile_image: "", first_name: "", last_name: "" } } },
                        isCollaborator: { $cond: { if: { $gt: [{ $size: '$collaboratorInfo' }, 0] }, then: true, else: false } },
                        isFriend: {
                            $and: [
                                { $gt: [{ $size: '$followerPrivacy' }, 0] },
                                { $gt: [{ $size: '$reverseFollow' }, 0] }
                            ]
                        },
                        isFamily: { $cond: { if: { $gt: [{ $size: '$family' }, 0] }, then: true, else: false } },
                        hasSavedPost: { $cond: { if: { $gt: [{ $size: '$savedPosts' }, 0] }, then: true, else: false } },
                        postMedia: {
                            $sortArray: { input: '$postMedia', sortBy: { index_number: 1 } },
                        },
                        isTagged: { $cond: { if: { $gt: [{ $size: '$tagUser' }, 0] }, then: true, else: false } },
                        type: 'Post'
                    }
                },
                {
                    $match: {
                        'blockedInfo': { $eq: [] },
                        $or: [
                            { 'privacy': 'Public' },
                            { 'privacy': 'Following', followerPrivacy: true },
                            { 'privacy': 'FamilyOnly', isFamily: true },
                            { 'privacy': 'OnlyMe', 'profile._id': req.user_id },
                            { 'privacy': 'Friends', isFriend: true },
                            { 'user_id': req.user_id }
                        ]
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        _id: 1,
                        user_id: 1,
                        type: 1,
                        location: 1,
                        description: 1,
                        music: 1,
                        privacy: 1,
                        status: 1,
                        is_active: 1,
                        is_delete: 1,
                        viewCount: 1,
                        createdAt: 1,
                        location_latitude: 1,
                        color_name: 1,
                        color_id: 1,
                        is_color_background: 1,
                        location_longitude: 1,
                        location_full_address: 1,
                        profile: {
                            username: '$profile.username',
                            user_id: '$profile._id',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$profile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$profile.email'
                        },
                        collaboratorCount: 1,
                        collaboratorUserProfile: {
                            username: '$collaboratorProfile.username',
                            user_id: '$collaboratorProfile._id',
                            first_name: '$collaboratorProfile.first_name',
                            last_name: '$collaboratorProfile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$collaboratorProfile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$collaboratorProfile.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$collaboratorProfile.email'
                        },
                        // collaboratorUserProfile: {
                        //     $arrayElemAt: [{
                        //         $map: {
                        //             input: '$collaboratorUserProfile',
                        //             as: 'collaboratorUserProfiles',
                        //             in: {
                        //                 username: '$collaboratorUserProfiles.username',
                        //                 user_id: '$collaboratorUserProfiles._id',
                        //                 first_name: '$collaboratorUserProfiles.first_name',
                        //                 last_name: '$collaboratorUserProfiles.last_name',
                        //                 profile_image:
                        //                 {
                        //                     $cond: {
                        //                         if: { $ne: ['$collaboratorUserProfiles.profile_image', ''] }, then: {
                        //                             $concat: [constant.s3Bucket_profile, '$collaboratorUserProfiles.profile_image']
                        //                         }, else: ''
                        //                     }
                        //                 },
                        //                 email: '$collaboratorUserProfiles.email'
                        //             }
                        //         }
                        //     },
                        //         0]
                        // },
                        hashtags: {
                            $map: {
                                input: '$hashtags',
                                as: 'hash_tag',
                                in: {
                                    _id: '$$hash_tag._id', // Include the _id field from the joined 'posts' collection
                                    hashtag: '$$hash_tag.hashtag' // Include the 'hashtag' field from the joined 'posts' collection
                                }
                            }
                        },
                        media: {
                            $map: {
                                input: '$postMedia',
                                as: 'post_media',
                                in: {
                                    _id: '$$post_media._id', // Include the _id field from the joined 'posts' collection
                                    type: '$$post_media.type',
                                    media_url: {
                                        $concat: [constant.s3Bucket_post_media, {
                                            $ifNull: ['$$post_media.media_url', 'default-avatar.png']
                                        }]
                                    },
                                    thumb_image: {
                                        $cond: {
                                            if: { $ne: ['$$post_media.thumb_image', ''] }, then: {
                                                $concat: [constant.s3Bucket_post_media, '$$post_media.thumb_image']
                                            }, else: ''
                                        }
                                    },
                                }
                            }
                        },
                        totalLikes: 1,
                        userLiked: 1,
                        totalComments: 1,
                        totalViewers: 1,
                        totalSave: 1,
                        userLikeEmoji: {
                            $arrayElemAt: [{
                                $map: {
                                    input: '$reaction',
                                    as: 'reactions',
                                    in: {
                                        _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                        image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                    }
                                }
                            },
                                0]
                        },
                        isFollowed: 1,
                        isCollaborator: 1,
                        collaboratorStatus: '$collaboratorStatus.status',
                        collaborate_id: '$collaboratorStatus._id',
                        followStatus: '$followStatus.status',
                        // isFriend: 1,
                        isTagged: 1,
                        hasSavedPost: 1,
                        type: 1
                        // Project other fields from the Post model as needed
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_posts => {
                if (_posts.length > 0) {
                    // console.log("_posts", _posts)
                    social_model.getReelsForPost(req, (code, msg, data) => {
                        if (code != '2' && code != '0') {
                            let result = [];
                            let obj = {
                                type: 'Reel',
                                reels: data
                            }
                            const middleIndex = Math.floor(_posts.length / 2);

                            // Insert the reels object at the middle index
                            _posts.splice(middleIndex, 0, obj);
                            // _posts.push(obj);
                            callback(responseCode.SUCCESS, t('data_found'), _posts);
                        } else {
                            callback(responseCode.SUCCESS, t('data_found'), _posts);
                        }
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            })
        } catch (error) {
            console.log(error)
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    postDetail: async (req, callback) => {
        try {
            let viewer = await social_model.postView(req);
            const objectId = mongoose.Types.ObjectId(req.post_id);
            postsModel.aggregate([
                {
                    $match: {
                        _id: objectId,
                        is_active: "1",
                        is_delete: "0",
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'

                    }
                },

                {
                    $lookup: {
                        from: 'post_hatshtags',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postHashtags'
                    }
                },

                {
                    $lookup: {
                        from: 'hashtags',
                        localField: 'postHashtags.hashtag_id',
                        foreignField: '_id',
                        as: 'hashtags'
                    }
                },
                {
                    $lookup: {
                        from: 'post_medias',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postMedia'
                    }
                },
                {
                    $lookup: {
                        from: 'post_likes',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'Totallikes'
                    }
                },
                {
                    $lookup: {
                        from: 'post_tag_users',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'tagUser'
                    }
                },
                {
                    $lookup: {
                        from: 'post_likes',
                        let: { postId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$user_id', req.user_id] }, // Match saved posts by user ID
                                            { $eq: ['$post_id', '$$postId'] } // Match saved posts by post ID 7417631600
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { _id: 1, like_id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $lookup: {
                        from: 'post_comments',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postComments'
                    }
                },
                {
                    $lookup: {
                        from: 'post_viewers',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postViewers'
                    }
                },
                {
                    $lookup: {
                        from: 'collection_posts',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'collectionPost'
                    }
                },
                {
                    "$unwind": "$profile"
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followInfo'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'followStatus'
                    },

                },
                {
                    $lookup: {
                        from: 'post_collaborator_users',
                        let: { authorID: '$profile._id', postId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$collaborator_user_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$user_id', '$$authorID'] },
                                            { $eq: ['$post_id', '$$postId'] },
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'collaboratorInfo'
                    }
                },
                {
                    $lookup: {
                        from: 'post_collaborator_users',
                        let: { authorID: '$profile._id', postId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$collaborator_user_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$user_id', '$$authorID'] },
                                            { $eq: ['$post_id', '$$postId'] },
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { status: 1, _id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'collaboratorStatus'
                    },
                },
                {
                    $lookup: {
                        from: 'post_collaborator_users',
                        let: { postId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$post_id', '$$postId'] }, // Check if the current user follows the author
                                            { $eq: ['$status', 'Accept'] },
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'collaboratorUsers'
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'collaboratorUsers.collaborator_user_id',
                        foreignField: '_id',
                        as: 'collaboratorUserProfile'

                    }
                },
                {
                    $lookup: {
                        from: 'collection_posts',
                        let: { postId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$user_id', req.user_id] }, // Match saved posts by user ID
                                            { $eq: ['$post_id', '$$postId'] } // Match saved posts by post ID
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { _id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'savedPosts'
                    }
                },
                // {
                //     $lookup: {
                //         from: 'post_tag_users',
                //         let: { postId: '$_id' },
                //         pipeline: [
                //             {
                //                 $match: {
                //                     $expr: {
                //                         $and: [
                //                             { $eq: ['$tag_user_id', req.user_id] }, // Match saved posts by user ID
                //                             { $eq: ['$post_id', '$$postId'] } // Match saved posts by post ID
                //                         ]
                //                     }
                //                 }
                //             },
                //             {
                //                 $project: { _id: 1 } // Include only _id field to check if there's a match
                //             }
                //         ],
                //         as: 'tagUser'
                //     }
                // },
                {
                    $addFields: {
                        totalLikes: { $size: '$Totallikes' },
                        totalComments: { $size: '$postComments' },
                        totalViewers: { $size: '$postViewers' },
                        totalSave: { $size: '$collectionPost' },
                        collaboratorCount: { $size: '$collaboratorUsers' },
                        userLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$Totallikes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        userLikeEmoji: {
                            $cond: {
                                if: { $in: [req.user_id, '$likes.user_id'] },
                                then: { $concat: [constant.s3Bucket_emoji, 'reaction.image'] },
                                else: ''
                            }
                        },
                        isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } },
                        followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                        hasSavedPost: { $cond: { if: { $gt: [{ $size: '$savedPosts' }, 0] }, then: true, else: false } },
                        isTagged: { $cond: { if: { $gt: [{ $size: '$tagUser' }, 0] }, then: true, else: false } },
                        postMedia: {
                            $sortArray: { input: '$postMedia', sortBy: { index_number: 1 } },
                        },
                        collaboratorProfile: { $cond: { if: { $gt: [{ $size: '$collaboratorUserProfile' }, 0] }, then: { $arrayElemAt: ['$collaboratorUserProfile', 0] }, else: { _id: "", username: "", email: "", profile_image: "", first_name: "", last_name: "" } } },
                        collaboratorStatus: { $cond: { if: { $gt: [{ $size: '$collaboratorStatus' }, 0] }, then: { $arrayElemAt: ['$collaboratorStatus', 0] }, else: { status: "", _id: "" } } },
                        isCollaborator: { $cond: { if: { $gt: [{ $size: '$collaboratorInfo' }, 0] }, then: true, else: false } },
                        // Add more fields for other emoji types as needed
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        _id: 1,
                        user_id: 1,
                        type: 1,
                        // message: 1,
                        location: 1,
                        description: 1,
                        music: 1,
                        privacy: 1,
                        status: 1,
                        is_active: 1,
                        is_delete: 1,
                        createdAt: 1,
                        color_name: 1,
                        color_id: 1,
                        is_color_background: 1,
                        viewCount: 1,
                        location_latitude: 1,
                        location_longitude: 1,
                        location_full_address: 1,
                        collaboratorCount: 1,
                        profile: {
                            username: '$profile.username',
                            user_id: '$profile._id',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$profile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                    }, else: ''
                                }
                            },

                            email: '$profile.email'
                        },

                        collaboratorUserProfile: {
                            username: '$collaboratorProfile.username',
                            user_id: '$collaboratorProfile._id',
                            first_name: '$collaboratorProfile.first_name',
                            last_name: '$collaboratorProfile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$collaboratorProfile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$collaboratorProfile.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$collaboratorProfile.email'
                        },
                        // collaboratorUserProfile: {
                        //     $arrayElemAt: [{
                        //         $map: {
                        //             input: '$collaboratorUserProfile',
                        //             as: 'collaboratorUserProfiles',
                        //             in: {
                        //                 username: '$collaboratorUserProfiles.username',
                        //                 user_id: '$collaboratorUserProfiles._id',
                        //                 first_name: '$collaboratorUserProfiles.first_name',
                        //                 last_name: '$collaboratorUserProfiles.last_name',
                        //                 profile_image:
                        //                 {
                        //                     $cond: {
                        //                         if: { $ne: ['$collaboratorUserProfiles.profile_image', ''] }, then: {
                        //                             $concat: [constant.s3Bucket_profile, '$collaboratorUserProfiles.profile_image']
                        //                         }, else: ''
                        //                     }
                        //                 },
                        //                 email: '$collaboratorUserProfiles.email'
                        //             }
                        //         }
                        //     },
                        //         0]
                        // },
                        hashtags: {
                            $map: {
                                input: '$hashtags',
                                as: 'hash_tag',
                                in: {
                                    _id: '$$hash_tag._id',
                                    hashtag: '$$hash_tag.hashtag'
                                }
                            }
                        },
                        media: {
                            $map: {
                                input: '$postMedia',
                                as: 'post_media',
                                in: {
                                    _id: '$$post_media._id',
                                    type: '$$post_media.type',
                                    media_url: {
                                        $concat: [constant.s3Bucket_post_media, {
                                            $ifNull: ['$$post_media.media_url', 'default-avatar.png']
                                        }]
                                    },
                                    thumb_image: {
                                        $cond: {
                                            if: { $ne: ['$$post_media.thumb_image', ''] }, then: {
                                                $concat: [constant.s3Bucket_post_media, '$$post_media.thumb_image']
                                            }, else: ''
                                        }
                                    },
                                }
                            }
                        },
                        totalLikes: 1,
                        userLiked: 1,
                        totalViewers: 1,
                        totalSave: 1,
                        totalComments: 1,
                        userLikeEmoji: {
                            $arrayElemAt: [{
                                $map: {
                                    input: '$reaction',
                                    as: 'reactions',
                                    in: {
                                        _id: '$$reactions._id',
                                        image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] }
                                    }
                                }
                            },
                                0]
                        },
                        hasSavedPost: 1,
                        isTagged: 1,
                        followStatus: '$followStatus.status',
                        isFollowed: 1,
                        isCollaborator: 1,
                        collaboratorStatus: '$collaboratorStatus.status',
                        collaborate_id: '$collaboratorStatus._id',
                        // Project other fields from the Post model as needed
                    }
                },

            ]).then(_posts => {
                if (_posts.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _posts[0]);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    deleteMyPost: async (req, callback) => {
        try {
            const postId = mongoose.Types.ObjectId(req.post_id);
            let postData = await postsModel.find({ _id: postId, user_id: req.user_id });
            if (postData.length > 0) {
                let post = await postsModel.deleteOne({ _id: postId, user_id: req.user_id });
                let postLikes = await postLike.deleteMany({ post_id: postId });
                let postView = await PostViewer.deleteMany({ post_id: postId });
                let postComment = await postComments.deleteMany({ post_id: postId });
                let postCollection = await collectionPostModel.deleteMany({ post_id: postId, user_id: req.user_id });
                callback(responseCode.SUCCESS, t('post_deleted'), {})
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
            }
        } catch (error) {
            // console.log("error", error)
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
        }
    },
    /**
     * @description This function will be used to get all recent locations
     * 
    */
    recentLocations: (req, callback) => {
        try {
            postsModel.find({ user_id: req.user_id, is_delete: "0", location_latitude: { $ne: "" } }, {
                _id: 0, location: 1, location_full_address: 1, location_latitude: 1, location_longitude: 1
            }).sort({ createdAt: -1 }).skip(0).limit(10).exec((err, results) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, [])
                } else if (results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, [])

        }
    },
    /**
    * API method for post view 
    */
    postView: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                let post_viewer = await PostViewer.find({ viewer_id: req.user_id, post_id: req.post_id });
                if (post_viewer.length == 0) {
                    let post_viewer_data = {
                        viewer_id: req.user_id,
                        post_id: req.post_id
                    }
                    let inserted_data = await PostViewer.create(post_viewer_data);
                    let post_data = await postsModel.findByIdAndUpdate(req.post_id, {
                        $inc: { viewCount: 1 }, // Increment the view count by 1
                    }, { new: true });
                    resolve(post_data)
                } else {
                    resolve([])

                }
            } catch (error) {
                reject(error)
            }
        })
    },
    getPostViewer: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        const objectId = mongoose.Types.ObjectId(req.post_id);
        PostViewer.aggregate([
            {
                $match: {
                    post_id: objectId
                }
            },
            {
                $lookup: {
                    from: 'post_likes',
                    let: { post_id: '$post_id', user_id: '$viewer_id' },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $or: [
                                        { $eq: ['$post_id', '$$post_id'] },
                                        { $eq: ['$user_id', '$$user_id'] }
                                    ]
                                }
                            }
                        }
                    ],
                    as: 'likes'
                }
            },
            {
                $lookup: {
                    from: 'reactions',
                    localField: 'likes.like_id',
                    foreignField: '_id',
                    as: 'reaction'
                }
            },
            {
                $lookup: {
                    from: 'users',
                    localField: 'viewer_id',
                    foreignField: '_id',
                    as: 'profile'

                }
            },
            {
                $unwind: '$profile'
            },
            {
                $lookup: {
                    from: 'followers',
                    let: { authorID: '$profile._id' },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $and: [
                                        { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                        { $eq: ['$following_id', '$$authorID'] }
                                    ]
                                }
                            }
                        }
                    ],
                    as: 'followInfo'
                }
            },
            {
                $lookup: {
                    from: 'followers',
                    let: { authorID: '$profile._id' },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $and: [
                                        { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                        { $eq: ['$following_id', '$$authorID'] }
                                    ]
                                }
                            }
                        },
                        {
                            $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                        }
                    ],
                    as: 'followStatus'
                },
            },
            {
                $addFields: {
                    userLiked: {
                        $cond: {
                            if: { $in: ['$profile._id', '$likes.user_id'] },
                            then: true,
                            else: false
                        }
                    },
                    userLikeEmoji: {
                        $cond: {
                            if: { $in: ['$profile._id', '$likes.user_id'] },
                            then: { $concat: [constant.s3Bucket_emoji, 'reaction.image'] },
                            else: []
                        }
                    },
                    followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                    isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } }
                    // Add more fields for other emoji types as needed
                }
            },
            {
                $sort: { createdAt: -1 }
            },
            {
                $project: {
                    _id: 1,
                    post_id: 1,
                    profile: {
                        user_id: '$profile._id',
                        username: '$profile.username',
                        user_id: '$profile._id',
                        first_name: '$profile.first_name',
                        last_name: '$profile.last_name',
                        profile_image:
                        {
                            $concat: [constant.s3Bucket_profile, {
                                $ifNull: ['$profile.profile_image', 'default-avatar.png']
                            }]
                        },
                        email: '$profile.email'
                    },
                    userLikeEmoji: {
                        $arrayElemAt: [{
                            $map: {
                                input: '$reaction',
                                as: 'reactions',
                                in: {
                                    _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                    image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                }
                            }
                        }, 0]
                    },
                    followStatus: '$followStatus.status',
                    isFollowed: 1,
                    userLiked: 1

                }
            },
            { $skip: start },
            { $limit: limit }
        ], (err, post_viewer) => {
            if (err) {
                callback(responseCode.OPERATION_FAILED, err, [])
            } else {
                if (post_viewer.length > 0) {
                    PostViewer.countDocuments({ post_id: objectId }, (err, count) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, [])
                        } else {
                            callback(responseCode.SUCCESS, t('data_found'), { totalViewers: count, viewersData: post_viewer })
                        }
                    });
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), { totalViewers: 0, viewersData: [] })
                }
            }
        })
    },
    /** 
     * @description This function is used to retrieve the list of users who have like  a specific post by its id.
    */
    getLikeUsersList: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            const objectId = mongoose.Types.ObjectId(req.post_id);
            postLike.aggregate([
                {
                    $match: {
                        post_id: objectId
                    }
                },
                {
                    $lookup: {
                        from: "users",
                        localField: "user_id",
                        foreignField: "_id",
                        as: "profile"
                    }
                },
                {
                    $lookup: {
                        from: "reactions",
                        localField: "like_id",
                        foreignField: "_id",
                        as: "reaction"
                    }
                },
                {
                    $unwind: '$profile'
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followInfo'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'followStatus'
                    },
                },
                {
                    $addFields: {
                        userLiked: true,
                        followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                        isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } }
                        // Add more fields for other emoji types as needed
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        _id: 1,
                        post_id: 1,
                        user_id: 1,
                        profile: {
                            user_id: '$profile._id',
                            username: '$profile.username',
                            user_id: '$profile._id',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$profile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$profile.email'
                        },
                        userLikeEmoji: {
                            $arrayElemAt: [{
                                $map: {
                                    input: '$reaction',
                                    as: 'reactions',
                                    in: {
                                        _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                        image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                    }
                                }
                            },
                                0]
                        },
                        isFollowed: 1,
                        followStatus: '$followStatus.status',
                        userLiked: 1,

                    }
                },
                { $skip: start },
                { $limit: limit }
            ], async (err, likedUsers) => {
                if (err) {
                    console.log("err", err);
                    callback(responseCode.OPERATION_FAILED, err, []);
                } else if (likedUsers.length > 0) {
                    const count = await social_model.getCountOfReactions(req);
                    // console.log("count", count);
                    callback(responseCode.SUCCESS, t('data_found'), { allEmojiWithCount: count, userLikeList: likedUsers });
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            })
        } catch (error) {
            console.log("error", error);
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    getCountOfReactions: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                const objectId = mongoose.Types.ObjectId(req.post_id);
                reactionModel.aggregate([
                    {
                        $project: {
                            _id: 1,
                            image: {
                                $concat: [constant.s3Bucket_emoji, '$image']
                            },
                            name: 1,
                        }
                    }
                ]).exec((err, result) => {
                    if (err) {
                        reject(error)
                    } else {
                        if (result.length > 0) {
                            asyncLoop(result, async (item, next) => {
                                let count = await postLike.countDocuments({ post_id: objectId, like_id: item._id });
                                item.count = count;
                                next();
                            }, () => {
                                resolve(result);
                            })
                        } else {
                            resolve([]);

                        }
                    }
                });
            } catch (error) {
                reject(error)
            }
        })
    },
    /**
    * API method for like/unlike Post 
    */
    likePost: (req, callback) => {
        try {
            const objectId = mongoose.Types.ObjectId(req.post_id);
            postLike.find({ post_id: objectId, user_id: req.user_id }, (err, result) => {
                console.log(result)
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    postLike.updateOne({ _id: result[0]._id }, { like_id: req.like_id }, (err, result) => {
                        if (!err) {
                            postsModel.find({ _id: objectId }, async (err, post_data) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else if (post_data.length > 0) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the post
                                    let notifi_data = {
                                        receiver_id: post_data[0].user_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has liked your post",
                                        tag: "like_post",
                                        image_url: "",
                                        like_id: result._id,
                                        post_id: req.post_id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                        } else {
                                            console.log("data", data)
                                        }
                                    })
                                    let collabUsers = await postCollaboratorModel.find({ post_id: objectId, status: "Accept" }, { _id: 1, collaborator_user_id: 1 });
                                    if (collabUsers.length > 0) {
                                        asyncLoop(collabUsers, async (item, next) => {
                                            let userData = await common.userDetails({ receiver_id: req.user_id });
                                            // send notification to the owner of the post
                                            let notifi_data = {
                                                receiver_id: item.collaborator_user_id,
                                                sender_id: req.user_id,
                                                title: "Neshwa",
                                                message: userData.username + " has liked your post",
                                                tag: "like_post",
                                                image_url: "",
                                                like_id: result._id,
                                                post_id: req.post_id,
                                            }
                                            common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                                if (err) {
                                                    console.log("err", err)
                                                    next();
                                                } else {
                                                    console.log("data", data)
                                                    next();
                                                }
                                            })
                                        }, () => {
                                            social_model.postDetail({ post_id: req.post_id, user_id: req.user_id }, (code, message, data) => {
                                                if (code != "0") {
                                                    data.userLiked = true;
                                                    callback(responseCode.SUCCESS, t('post_liked'), data)
                                                } else {
                                                    callback(responseCode.OPERATION_FAILED, message, {});
                                                }
                                            })
                                        })
                                    } else {
                                        social_model.postDetail({ post_id: req.post_id, user_id: req.user_id }, (code, message, data) => {
                                            if (code != "0") {
                                                data.userLiked = true;
                                                callback(responseCode.SUCCESS, t('post_liked'), data)
                                            } else {
                                                callback(responseCode.OPERATION_FAILED, message, {});
                                            }
                                        })
                                    }

                                } else {
                                    console.log("Post Something went wrong")
                                    let count = await postLike.countDocuments({ post_id: objectId });
                                    social_model.postDetail({ post_id: req.post_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('post_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                }
                            })

                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    postLike.create({ user_id: req.user_id, post_id: req.post_id, like_id: req.like_id }, (err, result) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            const objectId = mongoose.Types.ObjectId(req.post_id);
                            postsModel.find({ _id: objectId }, async (err, post_data) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else if (post_data.length > 0) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the post
                                    let notifi_data = {
                                        receiver_id: post_data[0].user_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has liked your post",
                                        tag: "like_post",
                                        image_url: "",
                                        like_id: result._id,
                                        post_id: req.post_id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                        } else {
                                            console.log("data", data)
                                        }
                                    })
                                    let count = await postLike.countDocuments({ post_id: objectId });
                                    social_model.postDetail({ post_id: req.post_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('post_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                } else {
                                    console.log("Post Something went wrong")
                                    social_model.postDetail({ post_id: req.post_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('post_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                }
                            })
                        }
                    })
                }
            })
        } catch (e) {
            console.log("Error in likeUnlikePost : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    unlikePost: (req, callback) => {
        try {
            postLike.find({ post_id: req.post_id, user_id: req.user_id }, (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    postLike.deleteOne({ _id: result[0]._id }, async (err, result) => {
                        if (!err) {
                            const objectId = mongoose.Types.ObjectId(req.post_id);
                            let count = await postLike.countDocuments({ post_id: objectId });
                            // console.log("count", count)
                            social_model.postDetail({ post_id: req.post_id, user_id: req.user_id }, (code, message, data) => {
                                if (code != "0") {
                                    callback(responseCode.SUCCESS, t('post_unliked'), data);
                                } else {
                                    callback(responseCode.OPERATION_FAILED, message, {});
                                }
                            })
                            // callback(responseCode.SUCCESS, t('post_unliked'), { totalLikes: count })
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('something_went_wrong'), {});
                }
            })
        } catch (e) {
            console.log("Error in UnlikePost : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    addComment: (req, callback) => {
        try {
            let comment_data = {
                user_id: req.user_id,
                post_id: req.post_id,
                message: req.message
            }
            if (req.parentComment != undefined && req.parentComment != '') {
                comment_data.parentComment = req.parentComment;
            }
            postComments.create(comment_data, async (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    if (req.parentComment != undefined && req.parentComment != '') {
                        var objectId = mongoose.Types.ObjectId(req.parentComment);
                        var msg = " has replied on your Comment";
                    } else {
                        var objectId = mongoose.Types.ObjectId(req.post_id);
                        var msg = " has comment on your Post";
                    }
                    postComments.find({ _id: objectId }, async (err, comment_data) => {
                        // console.log("comment_data", comment_data)
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else if (comment_data.length > 0) {
                            let userData = await common.userDetails({ receiver_id: comment_data[0].user_id });
                            // send notification to the owner of the post
                            let notifi_data = {
                                receiver_id: comment_data[0].user_id,
                                sender_id: req.user_id,
                                title: "Neshwa",
                                message: userData.username + msg,
                                tag: "comment_on_post",
                                image_url: "",
                                comment_id: result._id,
                                post_id: req.post_id,
                            }
                            common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                if (err) {
                                    console.log("err", err)
                                } else {
                                    console.log("data", data)
                                }
                            })
                            callback(responseCode.SUCCESS, t('comment_created'), {});
                        } else {
                            console.log("Post Something went wrong")
                            callback(responseCode.SUCCESS, t('comment_created'), {});
                        }
                    })
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    deleteMyComment: async (req, callback) => {
        try {
            let comment_id = mongoose.Types.ObjectId(req.comment_id);
            let post_id = mongoose.Types.ObjectId(req.post_id);
            /* remove all likes then delete comment */
            let deleteLike = await commentLikeModel.deleteMany({ comment_id: comment_id });
            let deleteComment = await postComments.deleteMany({ _id: comment_id });
            let parentComment = await postComments.deleteMany({ parentComment: comment_id });
            let totalComment = await postComments.countDocuments({ post_id: post_id });
            callback(responseCode.SUCCESS, t('comment_deleted'), { totalComments: totalComment });
        } catch (err) {
            callback(responseCode.OPERATION_FAILED, err, {})
        }
    },
    getComment: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            const objectId = mongoose.Types.ObjectId(req.post_id);
            postComments.aggregate([
                { $match: { post_id: objectId, parentComment: null } },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'
                    }
                },
                {
                    "$unwind": "$profile"
                },
                {
                    $lookup: {
                        from: 'comment_likes',
                        localField: '_id',
                        foreignField: 'comment_id',
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $addFields: {
                        totalLikes: { $size: '$likes' },
                        userLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$likes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        // userLikeEmoji: {
                        //     $cond: {
                        //         if: { $in: [req.user_id, '$likes.user_id'] },
                        //         then: '$reaction.image',
                        //         else: ''
                        //     }
                        // }

                        // Add more fields for other emoji types as needed
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        profile: {
                            username: '$profile.username',
                            user_id: '$profile._id',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$profile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$profile.email'
                        },
                        message: 1,
                        post_id: 1,
                        _id: 1,
                        parentComment: 1,
                        createdAt: 1,
                        updatedAt: 1,
                        totalLikes: 1,
                        userLiked: 1,
                        // userLikeEmoji: 1,
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(result => {
                if (result.length > 0) {
                    asyncLoop(result, (item, next) => {
                        postComments.aggregate([
                            {
                                $match: {
                                    post_id: objectId,
                                    parentComment: item._id
                                }
                            },
                            {
                                $lookup: {
                                    from: 'users',
                                    localField: 'user_id',
                                    foreignField: '_id',
                                    as: 'profile'

                                }
                            },
                            {
                                "$unwind": "$profile"
                            },
                            {
                                $lookup: {
                                    from: 'comment_likes',
                                    localField: '_id',
                                    foreignField: 'comment_id',
                                    as: 'likes'
                                }
                            },
                            {
                                $addFields: {
                                    totalLikes: { $size: '$likes' },
                                    userLiked: {
                                        $cond: {
                                            if: { $in: [req.user_id, '$likes.user_id'] },
                                            then: true,
                                            else: false
                                        }
                                    },
                                    // Add more fields for other emoji types as needed
                                }
                            },
                            {
                                $sort: { createdAt: 1 }
                            },
                            {
                                $project: {
                                    profile: {
                                        username: '$profile.username',
                                        user_id: '$profile._id',
                                        first_name: '$profile.first_name',
                                        last_name: '$profile.last_name',
                                        profile_image:
                                        {
                                            $concat: [constant.s3Bucket_profile, {
                                                $ifNull: ['$profile.profile_image', 'default-avatar.png']
                                            }]
                                        },
                                        email: '$profile.email'
                                    },
                                    message: 1,
                                    post_id: 1,
                                    _id: 1,
                                    createdAt: 1,
                                    comment_id: '$parentComment',
                                    updatedAt: 1,
                                    totalLikes: 1,
                                    userLiked: 1,
                                }
                            }
                        ]).then(replies => {
                            item.replies = replies
                            next();
                        })
                    }, () => {
                        callback(responseCode.SUCCESS, t('data_found'), result)
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, [])
        }
    },
    likeComment: (req, callback) => {
        try {
            commentLikeModel.find({ comment_id: req.comment_id, user_id: req.user_id }, (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    commentLikeModel.updateOne({ _id: result[0]._id }, { like_id: req.like_id }, (err, result) => {
                        if (!err) {
                            const objectId = mongoose.Types.ObjectId(req.comment_id);
                            postComments.find({ _id: objectId }, async (err, comment_data) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else if (comment_data.length > 0) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the post
                                    let notifi_data = {
                                        receiver_id: comment_data[0].user_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has liked your comment",
                                        tag: "like_comment",
                                        image_url: "",
                                        like_id: result._id,
                                        comment_id: req.comment_id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                        } else {
                                            console.log("data", data)
                                        }
                                    })
                                    let count = await commentLikeModel.countDocuments({ comment_id: objectId });
                                    callback(responseCode.SUCCESS, t('comment_liked'), { totalLikes: count });
                                } else {
                                    console.log("Comment Something went wrong")
                                    let count = await commentLikeModel.countDocuments({ comment_id: objectId });
                                    callback(responseCode.SUCCESS, t('comment_liked'), { totalLikes: count });
                                }
                            })
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    commentLikeModel.create({ user_id: req.user_id, comment_id: req.comment_id, like_id: req.like_id }, (err, result) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            const objectId = mongoose.Types.ObjectId(req.comment_id);
                            postComments.find({ _id: objectId }, async (err, comment_data) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else if (comment_data.length > 0) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the post
                                    let notifi_data = {
                                        receiver_id: comment_data[0].user_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has liked your comment",
                                        tag: "like_comment",
                                        image_url: "",
                                        like_id: result._id,
                                        comment_id: req.comment_id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                        } else {
                                            console.log("data", data)
                                        }
                                    })
                                    let count = await commentLikeModel.countDocuments({ comment_id: objectId });
                                    callback(responseCode.SUCCESS, t('comment_liked'), { totalLikes: count })
                                } else {
                                    console.log("Comment Something went wrong")
                                    let count = await commentLikeModel.countDocuments({ comment_id: objectId });
                                    callback(responseCode.SUCCESS, t('comment_liked'), { totalLikes: count })
                                }
                            })
                        }
                    })
                }
            })
        } catch (e) {
            console.log("Error in likeUnlikePost : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    unlikeComment: (req, callback) => {
        try {
            commentLikeModel.find({ comment_id: req.comment_id, user_id: req.user_id }, (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    commentLikeModel.deleteOne({ _id: result[0]._id }, (err, result) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('comment_unliked'), {})
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('something_went_wrong'), {});
                }
            })
        } catch (e) {
            console.log("Error in likeUnlikePost : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    /**
        * @description API method for collection listing
        */
    collectionList: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        let responseData = [];
        collectionModel.find({ user_id: req.user_id, is_active: "1", is_delete: "0" }).sort({ createdAt: -1 }).skip(start)
            .limit(limit)
            .exec((_err, _result) => {
                // Your code here
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, []);
                } else {
                    if (_result.length > 0) {
                        asyncLoop(_result, (item, next) => {
                            collectionPostModel.aggregate([
                                {
                                    $match: { user_id: req.user_id, collection_id: item._id }
                                },
                                {
                                    $lookup: {
                                        from: 'posts',
                                        localField: 'post_id',
                                        foreignField: '_id',
                                        as: 'postData'
                                    }
                                },
                                {
                                    $lookup: {
                                        from: 'users',
                                        localField: 'postData.user_id',
                                        foreignField: '_id',
                                        as: 'profile'
                                    }
                                },
                                {
                                    "$unwind": "$profile"
                                },
                                {
                                    "$unwind": "$postData"
                                },
                                {
                                    $sort: { createdAt: -1 }
                                },
                                {
                                    $project: {
                                        profile: {
                                            username: '$profile.username',
                                            user_id: '$profile._id',
                                            first_name: '$profile.first_name',
                                            last_name: '$profile.last_name',
                                            profile_image:
                                            {
                                                $concat: [constant.s3Bucket_profile, {
                                                    $ifNull: ['$profile.profile_image', 'default-avatar.png']
                                                }]
                                            },
                                            email: '$profile.email'
                                        },
                                        post_collection_id: '$_id',
                                        post_id: '$postData._id',
                                        type: '$postData.type',
                                        message: '$postData.message',
                                        location: '$postData.location',
                                        description: '$postData.description',
                                        location_latitude: '$postData.location_latitude',
                                        location_longitude: '$postData.location_longitude',
                                        location_full_address: '$postData.location_full_address'
                                    }
                                },
                                { $skip: 0 },
                                { $limit: 1 }
                            ], async (_err, postData) => {
                                if (_err) {
                                    next(_err);
                                    return;
                                } else if (postData.length > 0) {
                                    let count = await collectionPostModel.countDocuments({ user_id: req.user_id, collection_id: item._id });
                                    responseData.push({
                                        _id: item._id,
                                        name: item.name,
                                        updatedAt: item.updatedAt,
                                        createdAt: item.createdAt,
                                        postCount: count,
                                        posts: postData[0]
                                    })
                                    next();
                                } else {
                                    responseData.push({
                                        _id: item._id,
                                        name: item.name,
                                        updatedAt: item.updatedAt,
                                        createdAt: item.createdAt,
                                        postCount: 0,
                                        posts: {}
                                    })
                                    next();
                                }
                            })
                        }, (err) => {
                            if (!err) {
                                callback(responseCode.SUCCESS, t('data_found'), responseData);
                            } else {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            }
                        })
                    } else {
                        callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                    }
                }
            })
    },
    /**
     * @description API method for creating collection
     */
    addCollection: (req, callback) => {
        let addData = {
            name: req.name,
            user_id: req.user_id
        }
        collectionModel.create(addData, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('collection_added'), { _id: _result._id });
            }
        })
    },
    /**
     * @description API method for edit collection
     */
    editCollection: (req, callback) => {
        let addData = {
            name: req.name
        }
        collectionModel.updateOne({ _id: req.collection_id }, addData, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('collection_edited'), {});
            }
        })
    },
    /**
     * @description API method for delete collection
     */
    deleteCollection: (req, callback) => {
        const objectId = mongoose.Types.ObjectId(req.collection_id);
        collectionPostModel.deleteOne({ collection_id: objectId }, (_err, deletedData) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                collectionModel.updateOne({ _id: objectId }, { is_delete: "1" }, (_err, _result) => {
                    if (_err) {
                        callback(responseCode.OPERATION_FAILED, _err, {});
                    } else {
                        callback(responseCode.SUCCESS, t('collection_deleted'), {});
                    }
                })
            }
        })
    },
    /**
      * @description API method for Save Post in Collection
      */
    savePostInCollection: (req, callback) => {
        let post_data = {
            collection_id: req.collection_id,
            user_id: req.user_id,
            post_id: req.post_id
        }
        collectionPostModel.create(post_data, async (_err, _insertedData) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                const objectId = mongoose.Types.ObjectId(req.post_id);
                let count = await collectionPostModel.countDocuments({ post_id: objectId });
                social_model.postDetail({ post_id: req.post_id, user_id: req.user_id }, (code, message, data) => {
                    if (code != "0") {
                        // callback(responseCode.SUCCESS, t('post_saved_in_collection'), { hasSavedPost: true, post_id: req.post_id, count: count, postDetail: data });
                        callback(responseCode.SUCCESS, t('post_saved_in_collection'), data);
                    } else {
                        callback(responseCode.OPERATION_FAILED, message, {});
                    }
                })
            }
        })
    },
    /**
      * @description API method for Delete Post From Collection
      */
    deletePostFromCollection: (req, callback) => {
        const objectId = mongoose.Types.ObjectId(req.post_id);
        let post_data = {
            post_id: objectId,
            user_id: req.user_id
        }
        collectionPostModel.deleteOne(post_data, async (_err, deletedData) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                let count = await collectionPostModel.countDocuments({ post_id: objectId });
                social_model.postDetail({ post_id: req.post_id, user_id: req.user_id }, (code, message, data) => {
                    if (code != "0") {
                        // callback(responseCode.SUCCESS, t('post_deleted_from_collection'), { hasSavedPost: false, post_id: req.post_id, count: count, postDetail: data });
                        callback(responseCode.SUCCESS, t('post_deleted_from_collection'), data);
                    } else {
                        callback(responseCode.OPERATION_FAILED, message, {});
                    }
                })

            }
        })
    },
    /**
      * @description API method for get Post From Collection
      */
    getPostFromCollection: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        const objectId = mongoose.Types.ObjectId(req.collection_id);
        collectionPostModel.aggregate([
            {
                $match: { user_id: req.user_id, collection_id: objectId }
            },
            {
                $lookup: {
                    from: 'posts',
                    localField: 'post_id',
                    foreignField: '_id',
                    as: 'postData'
                }
            },
            {
                $lookup: {
                    from: 'users',
                    localField: 'postData.user_id',
                    foreignField: '_id',
                    as: 'profile'
                }
            },
            {
                "$unwind": "$profile"
            },
            {
                "$unwind": "$postData"
            },
            {
                $sort: { createdAt: -1 }
            },
            {
                $project: {
                    profile: {
                        username: '$profile.username',
                        user_id: '$profile._id',
                        first_name: '$profile.first_name',
                        last_name: '$profile.last_name',
                        profile_image:
                        {
                            $concat: [constant.s3Bucket_profile, {
                                $ifNull: ['$profile.profile_image', 'default-avatar.png']
                            }]
                        },
                        email: '$profile.email'
                    },
                    post_collection_id: '$_id',
                    post_id: '$postData._id',
                    type: '$postData.type',
                    message: '$postData.message',
                    location: '$postData.location',
                    description: '$postData.description',
                    location_latitude: '$postData.location_latitude',
                    location_longitude: '$postData.location_longitude',
                    location_full_address: '$postData.location_full_address'
                }
            },
            { $skip: start },
            { $limit: limit }
        ], (_err, postData) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, []);
            } else if (postData.length > 0) {
                callback(responseCode.SUCCESS, t('data_found'), postData)
            } else {
                callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
            }
        })
    },
    /**
    * API method for create story 
    */
    addStory: (req, callback) => {
        if (req.medias.length > 0) {
            asyncLoop(req.medias, (item, next) => {
                // let expireAt = new Date(Date.now() + 24 * 60 * 60 * 1000); // 24 hours from now
                let expireAt = moment().add('24 hours').format('YYYY-MM-DD HH:mm:ss'); // 24 hours from now
                let addData = {
                    user_id: req.user_id,
                    media_type: item.media_type,
                    media: item.media,
                    music: (item.music != undefined && item.music != '') ? item.music : '',
                    thumb_image: (item.thumb_image != undefined && item.thumb_image != '') ? item.thumb_image : '',
                    index_number: (item.index_number != undefined && item.index_number != '') ? item.index_number : '0',
                    privacy: item.privacy,
                    duration: item.duration,
                    expiresAt: expireAt,
                }
                if (item.location_latitude != undefined && item.location_latitude != '') {
                    data.location_latitude = item.location_latitude;
                    data.location_longitude = item.location_longitude;
                    data.location_full_address = item.location_full_address;
                    data.location = item.location;
                }
                storyModel.create(addData, (err, result) => {
                    if (!err) {
                        next();
                    } else {
                        next(err);
                        return;
                    }
                });
            }, (err) => {
                if (!err) {
                    callback(responseCode.SUCCESS, t('story_added_success'), {});
                } else {
                    console.log("Error while adding data to stories : ", err);
                    callback(responseCode.OPERATION_FAILED, err, {});
                }
            })
        } else {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {});
        }
    },
    /**
    * API method for get story 
    */
    getStory: async (req, callback) => {
        try {
            let followingUsers = await social_model.getFollowFollowingUsers(req);
            followingUsers.push(req.user_id);
            // console.log("followingUsers", followingUsers)
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            const storiesGroupedByUsers = await userModel.aggregate([
                {
                    $lookup: {
                        from: 'stories',
                        localField: '_id',
                        foreignField: 'user_id',
                        as: 'stories'
                    }
                },
                // {
                //     $match: {
                //         _id: { $in: followingUsers },
                //         'stories.is_expired': "0",
                //         'stories.is_delete': "0",
                {
                    // Lookup to join with blocked_user collection to find users blocked by current user
                    $lookup: {
                        from: 'blocked_users',
                        let: { postUserId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $or: [
                                            { $and: [{ $eq: ['$user_id', req.user_id] }, { $eq: ['$block_user_id', '$$postUserId'] }] },
                                            { $and: [{ $eq: ['$block_user_id', req.user_id] }, { $eq: ['$user_id', '$$postUserId'] }] },
                                        ],
                                    },
                                },
                            },
                        ],
                        as: 'blockedInfo',
                    },
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] },
                                            { $eq: ['$status', 'Accept'] },
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followerPrivacy'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followInfo'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'followStatus'
                    },
                },
                {
                    $lookup: {
                        from: 'families',
                        let: { authorID: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$family_member_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$user_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'family'
                    }
                },
                {
                    $match: {
                        'blockedInfo': { $eq: [] },
                    }
                },
                {
                    $addFields: {
                        followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                        isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } },
                        followerPrivacy: { $cond: { if: { $gt: [{ $size: '$followerPrivacy' }, 0] }, then: true, else: false } },
                        isFamily: { $cond: { if: { $gt: [{ $size: '$family' }, 0] }, then: true, else: false } },
                        isCurrentUser: { $eq: ['$_id', req.user_id] }
                    }
                },

                {
                    $sort: {
                        isCurrentUser: -1, // Sort to put the current user first
                        'stories.createdAt': 1 // Sort stories by createdAt in acending order
                    }
                },
                {
                    $project: {
                        user_id: '$_id',
                        username: 1,
                        first_name: 1,
                        last_name: 1,
                        followStatus: '$followStatus.status',
                        isFollowed: 1,
                        profile_image: '$profile_image',
                        email: '$email',
                        stories: {
                            $filter: {
                                input: '$stories',
                                as: 'story',
                                cond: {
                                    $and: [
                                        { $eq: ['$$story.is_delete', "0"] },
                                        { $eq: ['$$story.is_expired', "0"] },

                                        {
                                            $or: [
                                                { $eq: ['$$story.privacy', 'Public'] },
                                                { $eq: ['$$story.user_id', req.user_id] },
                                                {
                                                    $and: [
                                                        { $eq: ['$$story.privacy', 'Following'] },
                                                        { $eq: ['$followerPrivacy', true] }
                                                    ],

                                                },
                                                {
                                                    $and: [
                                                        { $eq: ['$$story.privacy', 'FamilyOnly'] },
                                                        { $eq: ['$isFamily', true] },
                                                        { $eq: ['$$story.user_id', req.user_id] },
                                                    ],
                                                },
                                                {
                                                    $and: [
                                                        { $eq: ['$$story.privacy', 'OnlyMe'] },
                                                        { $eq: ['$$story.user_id', req.user_id] },
                                                    ],
                                                },
                                            ],
                                        },
                                    ],
                                }, // Filter out deleted stories
                            }
                        }
                    }
                },
                {
                    $match: {
                        // _id: { $in: followingUsers },
                        'stories.is_expired': "0",
                        'stories.is_delete': "0",
                        'stories.0': { $exists: true }, // only include users with at least one story
                        'stories.privacy': {
                            $in: ['Public', 'Following', 'FamilyOnly', 'OnlyMe'],
                        },
                    }
                },
                {
                    $project: {
                        user_id: '$_id',
                        username: 1,
                        first_name: 1,
                        last_name: 1,
                        followStatus: '$followStatus',
                        isFollowed: 1,
                        profile_image:
                        {
                            $cond: {
                                if: { $ne: ['$profile_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_profile, '$profile_image']
                                }, else: ''
                            }
                        },
                        email: '$email',
                        stories: {
                            $map: {
                                input: '$stories',
                                as: 'story',
                                in: {
                                    _id: '$$story._id',
                                    media_type: '$$story.media_type',
                                    media: {
                                        $cond: {
                                            if: { $ne: ['$$story.media', ''] }, then: {
                                                $concat: [constant.s3Bucket_story_media, '$$story.media']
                                            }, else: ''
                                        }
                                    },
                                    thumb_image: {
                                        $cond: {
                                            if: { $ne: ['$$story.thumb_image', ''] }, then: {
                                                $concat: [constant.s3Bucket_story_media, '$$story.thumb_image']
                                            }, else: ''
                                        }
                                    },
                                    privacy: '$$story.privacy',
                                    duration: '$$story.duration',
                                    expiresAt: '$$story.expiresAt',
                                    createdAt: '$$story.createdAt'
                                }
                            }
                        },

                    },

                },
                { $skip: start },
                { $limit: limit }
            ]);
            if (storiesGroupedByUsers.length > 0) {
                const usersWithStories = storiesGroupedByUsers.filter(user => user.stories.length > 0);

                asyncLoop(usersWithStories, async (item, next) => {
                    let storyIds = [];
                    asyncLoop(item.stories, async (item2, next2) => {
                        storyIds.push(item2._id);
                        let likeUser = await social_model.getStoryLikeUsers({ story_id: item2._id, user_id: req.user_id });
                        let isLiked = await storyLike.find({ user_id: req.user_id, story_id: item2._id });
                        let currentUserView = await StoryViewer.countDocuments({ viewer_id: req.user_id, story_id: item2._id });
                        let totalViewerCount = await StoryViewer.countDocuments({ story_id: item2._id });
                        let totalLikes = await storyLike.countDocuments({ story_id: item2._id });
                        let userLikeEmoji = await StoryViewer.countDocuments({ story_id: item2._id });
                        item2.isViewed = currentUserView != 0 ? true : false;
                        item2.totalViewers = totalViewerCount;
                        item2.likeUser = likeUser;
                        item2.totalLikes = totalLikes;
                        item2.isLiked = isLiked.length > 0 ? true : false;
                        if (isLiked.length > 0) {
                            let userLikeEmoji = await reactionModel.findOne({ _id: isLiked[0].like_id, story_id: item2._id }, {
                                _id: 1, image: { $concat: [constant.s3Bucket_emoji, '$image'] }
                            });
                            item2.userLikeEmoji = userLikeEmoji;

                        } else {
                            item2.userLikeEmoji = {}
                        }
                        next2();
                    }, async () => {
                        let totalStory = item.stories.length;
                        let totalViewedStory = await StoryViewer.countDocuments({ story_id: { $in: storyIds }, viewer_id: req.user_id, story_user_id: item._id });
                        item.totalStory = totalStory;
                        item.totalViewedStory = totalViewedStory;
                        item.totalUnviewedStory = totalStory - totalViewedStory;
                        next();
                    })
                }, () => {
                    callback(responseCode.SUCCESS, t('data_found'), usersWithStories)
                })
            } else {
                callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
            }
            // console.log(storiesGroupedByUsers);
        } catch (error) {
            console.error(error);
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    storyDetail: async (req, callback) => {
        try {
            let stories = await storyModel.aggregate([
                {
                    $match: {
                        user_id: req.story_user_id
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'

                    }
                },
                {
                    $unwind: '$profile'
                },
                {
                    $lookup: {
                        from: 'story_viewers',
                        localField: '_id',
                        foreignField: 'story_id',
                        as: 'storyViewers'
                    }
                },
                {
                    $lookup: {
                        from: 'story_likes',
                        localField: '_id',
                        foreignField: 'story_id',
                        as: 'Totallikes'
                    }
                },
                {
                    $lookup: {
                        from: 'story_likes',
                        let: { storyId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$user_id', req.user_id] },
                                            { $eq: ['$story_id', '$$storyId'] }
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { _id: 1, like_id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $addFields: {
                        totalLikes: { $size: '$Totallikes' },
                        // totalComments: { $size: '$storyComments' },
                        totalViewers: { $size: '$storyViewers' },
                        isViewed: {
                            $cond: {
                                if: { $in: [req.user_id, '$storyViewers.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        userLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$Totallikes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        userLikeEmoji: {
                            $cond: {
                                if: { $in: [req.user_id, '$likes.user_id'] },
                                then: { $concat: [constant.s3Bucket_emoji, 'reaction.image'] },
                                else: ''
                            }
                        }
                    }
                },
                {
                    $sort: { createdAt: 1 }
                },
                {
                    $project: {
                        _id: 1,
                        user_id: 1,
                        media_type: 1,
                        media:
                        {
                            $cond: {
                                if: { $ne: ['$media', ''] }, then: {
                                    $concat: [constant.s3Bucket_story_media, '$media']
                                }, else: ''
                            }
                        },
                        thumb_image:
                        {
                            $cond: {
                                if: { $ne: ['$thumb_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_story_media, '$thumb_image']
                                }, else: ''
                            }
                        },
                        location: 1,
                        music: 1,
                        privacy: 1,
                        is_delete: 1,
                        createdAt: 1,
                        viewCount: 1,
                        location_latitude: 1,
                        location_longitude: 1,
                        location_full_address: 1,
                        profile: {
                            username: '$profile.username',
                            user_id: '$profile._id',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$profile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                    }, else: ''
                                }
                            },

                            email: '$profile.email'
                        },
                        totalLikes: 1,
                        userLiked: 1,
                        isViewed: 1,
                        totalViewers: 1,
                        // totalComments: 1,
                        userLikeEmoji: {
                            $arrayElemAt: [{
                                $map: {
                                    input: '$reaction',
                                    as: 'reactions',
                                    in: {
                                        _id: '$$reactions._id', // Include the _id field from the joined 'reaction' collection
                                        image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'reaction' collection
                                    }
                                }
                            },
                                0]
                        }
                    }
                },
            ]).then(_story => {
                // console.log("_story", _story)
                if (_story.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _story);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            })
        } catch (err) {
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    storySpecificDetail: async (req, callback) => {
        try {
            // console.log(req)
            const objectId = mongoose.Types.ObjectId(req.story_id);

            let stories = await storyModel.aggregate([
                {
                    $match: {
                        _id: objectId
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'

                    }
                },
                {
                    $unwind: '$profile'
                },
                {
                    $lookup: {
                        from: 'story_viewers',
                        localField: '_id',
                        foreignField: 'story_id',
                        as: 'storyViewers'
                    }
                },
                {
                    $lookup: {
                        from: 'story_likes',
                        localField: '_id',
                        foreignField: 'story_id',
                        as: 'Totallikes'
                    }
                },
                {
                    $lookup: {
                        from: 'story_likes',
                        let: { storyId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$user_id', req.user_id] },
                                            { $eq: ['$story_id', '$$storyId'] }
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { _id: 1, like_id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $addFields: {
                        totalLikes: { $size: '$Totallikes' },
                        // totalComments: { $size: '$storyComments' },
                        totalViewers: { $size: '$storyViewers' },
                        isViewed: {
                            $cond: {
                                if: { $in: [req.user_id, '$storyViewers.viewer_id'] },
                                then: true,
                                else: false
                            }
                        },
                        isLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$Totallikes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        userLikeEmoji: {
                            $cond: {
                                if: { $in: [req.user_id, '$likes.user_id'] },
                                then: { $concat: [constant.s3Bucket_emoji, 'reaction.image'] },
                                else: ''
                            }
                        }
                    }
                },
                {
                    $sort: { createdAt: 1 }
                },
                {
                    $project: {
                        _id: 1,
                        user_id: 1,
                        media_type: 1,
                        media:
                        {
                            $cond: {
                                if: { $ne: ['$media', ''] }, then: {
                                    $concat: [constant.s3Bucket_story_media, '$media']
                                }, else: ''
                            }
                        },
                        thumb_image:
                        {
                            $cond: {
                                if: { $ne: ['$thumb_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_story_media, '$thumb_image']
                                }, else: ''
                            }
                        },
                        location: 1,
                        music: 1,
                        privacy: 1,
                        is_delete: 1,
                        duration: 1,
                        expiresAt: 1,
                        createdAt: 1,
                        location_latitude: 1,
                        location_longitude: 1,
                        location_full_address: 1,
                        totalLikes: 1,
                        isLiked: 1,
                        isViewed: 1,
                        totalViewers: 1,
                        userLikeEmoji: {
                            $arrayElemAt: [{
                                $map: {
                                    input: '$reaction',
                                    as: 'reactions',
                                    in: {
                                        _id: '$$reactions._id', // Include the _id field from the joined 'reaction' collection
                                        image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'reaction' collection
                                    }
                                }
                            },
                                0]
                        }
                    }
                },
            ]).then(_story => {
                // console.log("_story", _story)
                if (_story.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _story[0]);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            })
        } catch (err) {
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    /**
    * API method for like/unlike Post 
    */
    likeStory: (req, callback) => {
        try {
            const objectId = mongoose.Types.ObjectId(req.story_id);
            storyLike.find({ story_id: objectId, user_id: req.user_id }, (err, result) => {
                console.log(result)
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    storyLike.updateOne({ _id: result[0]._id }, { like_id: req.like_id }, (err, result) => {
                        if (!err) {
                            storyModel.find({ _id: objectId }, async (err, story_data) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else if (story_data.length > 0) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the post
                                    let notifi_data = {
                                        receiver_id: story_data[0].user_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has liked your story",
                                        tag: "like_story",
                                        image_url: "",
                                        like_id: req.like_id,
                                        story_id: req.story_id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                        } else {
                                            console.log("data", data)
                                        }
                                    })
                                    social_model.storySpecificDetail({ story_id: req.story_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            // data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('post_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                } else {
                                    console.log("Story Something went wrong")
                                    let count = await postLike.countDocuments({ story_id: objectId });
                                    social_model.storySpecificDetail({ story_id: req.story_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            // data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('post_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                }
                            })

                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    storyLike.create({ user_id: req.user_id, story_id: req.story_id, like_id: req.like_id }, (err, result) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            const objectId = mongoose.Types.ObjectId(req.story_id);
                            storyModel.find({ _id: objectId }, async (err, story_data) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else if (story_data.length > 0) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the story
                                    let notifi_data = {
                                        receiver_id: story_data[0].user_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has liked your story",
                                        tag: "like_story",
                                        image_url: "",
                                        like_id: req.like_id,
                                        story_id: req.story_id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                        } else {
                                            console.log("data", data)
                                        }
                                    })
                                    let count = await storyLike.countDocuments({ story_id: objectId });
                                    social_model.storySpecificDetail({ story_id: req.story_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            // data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('story_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                } else {
                                    console.log("Story Something went wrong")
                                    social_model.storySpecificDetail({ story_id: req.story_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            // data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('story_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                }
                            })
                        }
                    })
                }
            })
        } catch (e) {
            console.log("Error in likeUnlikeStory : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    unlikeStory: (req, callback) => {
        try {
            storyLike.find({ story_id: req.story_id, user_id: req.user_id }, (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the story
                    storyLike.deleteOne({ _id: result[0]._id }, async (err, result) => {
                        if (!err) {
                            const objectId = mongoose.Types.ObjectId(req.story_id);
                            let count = await storyLike.countDocuments({ story_id: objectId });
                            // console.log("count", count)
                            social_model.storySpecificDetail({ story_id: req.story_id, user_id: req.user_id }, (code, message, data) => {
                                if (code != "0") {
                                    callback(responseCode.SUCCESS, t('story_unliked'), data);
                                } else {
                                    callback(responseCode.OPERATION_FAILED, message, {});
                                }
                            })
                            // callback(responseCode.SUCCESS, t('story_unliked'), { totalLikes: count })
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('something_went_wrong'), {});
                }
            })
        } catch (e) {
            console.log("Error in UnlikeStory : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    /**
   * API method for story view 
   */
    storyView: async (req, callback) => {
        try {
            const storyId = mongoose.Types.ObjectId(req.story_id);

            let story = await storyModel.findOne({ _id: storyId });
            let story_viewer = await StoryViewer.find({ viewer_id: req.user_id, story_id: storyId });
            if (story_viewer.length == 0) {
                let story_viewer_data = {
                    story_user_id: story.user_id,
                    viewer_id: req.user_id,
                    story_id: req.story_id
                }
                let inserted_data = await StoryViewer.create(story_viewer_data);
                let story_data = await storyModel.findByIdAndUpdate(req.story_id, {
                    $inc: { viewCount: 1 }, // Increment the view count by 1
                }, { new: true });
                callback(responseCode.SUCCESS, t('story has been viewed successfull'), {})
            } else {
                callback(responseCode.SUCCESS, t('story has been viewed successfull'), {})
            }
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
        }
    },
    deleteStory: async (req, callback) => {
        try {
            const storyId = mongoose.Types.ObjectId(req.story_id);
            let story_data = await storyModel.findByIdAndUpdate(storyId, {
                is_delete: "1"
            });
            callback(responseCode.SUCCESS, t('story has been deleted successfull'), {})

        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
        }
    },
    reportPost: async (req, callback) => {
        try {
            let insert_data = {
                user_id: req.user_id,
                reason_id: req.reason_id,
                post_id: req.post_id,
                description: (req.description != undefined && req.description != '') ? req.description : '',
            }
            let post_data = await postReportModel.create(insert_data);
            callback(responseCode.SUCCESS, t('Your report has been submitted successfully'), {})

        } catch (error) {
            console.log("error", error)
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
        }
    },
    reportStory: async (req, callback) => {
        try {
            let insert_data = {
                user_id: req.user_id,
                reason_id: req.reason_id,
                story_id: req.story_id,
                description: req.description,
            }
            let story_data = await storyReportModel.create(insert_data);
            callback(responseCode.SUCCESS, t('Your report has been submitted successfully'), {})

        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
        }
    },
    editStoryPrivacy: async (req, callback) => {
        try {
            const storyId = mongoose.Types.ObjectId(req.story_id);
            let story_data = await storyModel.findByIdAndUpdate(storyId, {
                privacy: req.privacy
            });
            callback(responseCode.SUCCESS, t('story privacy has been updated successfull'), {})

        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
        }
    },
    getStoryViewer: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        const objectId = mongoose.Types.ObjectId(req.story_id);
        StoryViewer.aggregate([
            {
                $match: {
                    story_id: objectId
                }
            },
            {
                $lookup: {
                    from: 'story_likes',
                    let: { story_id: '$story_id', user_id: '$viewer_id' },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $or: [
                                        { $eq: ['$story_id', '$$story_id'] },
                                        { $eq: ['$user_id', '$$user_id'] }
                                    ]
                                }
                            }
                        }
                    ],
                    as: 'likes'
                }
            },
            {
                $lookup: {
                    from: 'reactions',
                    localField: 'likes.like_id',
                    foreignField: '_id',
                    as: 'reaction'
                }
            },
            {
                $lookup: {
                    from: 'users',
                    localField: 'viewer_id',
                    foreignField: '_id',
                    as: 'profile'

                }
            },
            {
                $unwind: '$profile'
            },
            {
                $lookup: {
                    from: 'followers',
                    let: { authorID: '$profile._id' },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $and: [
                                        { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                        { $eq: ['$following_id', '$$authorID'] }
                                    ]
                                }
                            }
                        }
                    ],
                    as: 'followInfo'
                }
            },
            {
                $lookup: {
                    from: 'followers',
                    let: { authorID: '$profile._id' },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $and: [
                                        { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                        { $eq: ['$following_id', '$$authorID'] }
                                    ]
                                }
                            }
                        },
                        {
                            $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                        }
                    ],
                    as: 'followStatus'
                },
            },
            {
                $addFields: {
                    userLiked: {
                        $cond: {
                            if: { $in: ['$profile._id', '$likes.user_id'] },
                            then: true,
                            else: false
                        }
                    },
                    userLikeEmoji: {
                        $cond: {
                            if: { $in: ['$profile._id', '$likes.user_id'] },
                            then: { $concat: [constant.s3Bucket_emoji, 'reaction.image'] },
                            else: []
                        }
                    },
                    followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                    isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } }

                    // Add more fields for other emoji types as needed
                }
            },
            {
                $sort: { createdAt: -1 }
            },
            {
                $project: {
                    _id: 1,
                    story_id: 1,
                    profile: {
                        user_id: '$profile._id',
                        username: '$profile.username',
                        user_id: '$profile._id',
                        first_name: '$profile.first_name',
                        last_name: '$profile.last_name',
                        profile_image:
                        {
                            $concat: [constant.s3Bucket_profile, {
                                $ifNull: ['$profile.profile_image', 'default-avatar.png']
                            }]
                        },
                        email: '$profile.email'
                    },
                    userLikeEmoji: {
                        $arrayElemAt: [{
                            $map: {
                                input: '$reaction',
                                as: 'reactions',
                                in: {
                                    _id: '$$reactions._id', // Include the _id field from the joined 'storys' collection
                                    image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'storys' collection
                                }
                            }
                        }, 0]
                    },
                    followStatus: '$followStatus.status',
                    isFollowed: 1,
                    userLiked: 1

                }
            },
            { $skip: start },
            { $limit: limit }
        ], (err, story_viewer) => {
            if (err) {
                callback(responseCode.OPERATION_FAILED, err, [])
            } else {
                if (story_viewer.length > 0) {
                    StoryViewer.countDocuments({ story_id: objectId }, (err, count) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, [])
                        } else {
                            callback(responseCode.SUCCESS, t('data_found'), { totalViewers: count, viewersData: story_viewer })
                        }
                    });
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), { totalViewers: 0, viewersData: [] })
                }
            }
        })
    },
    /** 
    * @description This function is used to retrieve the list of users who have like  a specific story by its id.
   */
    getStoryLikeUsersList: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = (req.limit != undefined && req.limit > 0) ? req.limit : constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            const objectId = mongoose.Types.ObjectId(req.story_id);
            storyLike.aggregate([
                {
                    $match: {
                        story_id: objectId
                    }
                },
                {
                    $lookup: {
                        from: "users",
                        localField: "user_id",
                        foreignField: "_id",
                        as: "profile"
                    }
                },
                {
                    $lookup: {
                        from: "reactions",
                        localField: "like_id",
                        foreignField: "_id",
                        as: "reaction"
                    }
                },
                {
                    $unwind: '$profile'
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followInfo'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'followStatus'
                    },
                },
                {
                    $addFields: {
                        userLiked: true,
                        followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                        isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } }
                        // Add more fields for other emoji types as needed
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        _id: 1,
                        story_id: 1,
                        user_id: 1,
                        profile: {
                            user_id: '$profile._id',
                            username: '$profile.username',
                            user_id: '$profile._id',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$profile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$profile.email'
                        },
                        userLikeEmoji: {
                            $arrayElemAt: [{
                                $map: {
                                    input: '$reaction',
                                    as: 'reactions',
                                    in: {
                                        _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                        image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                    }
                                }
                            },
                                0]
                        },
                        followStatus: '$followStatus.status',
                        isFollowed: 1,
                        userLiked: 1,

                    }
                },
                { $skip: start },
                { $limit: limit }
            ], async (err, likedUsers) => {
                if (err) {
                    console.log("err", err);
                    callback(responseCode.OPERATION_FAILED, err, []);
                } else if (likedUsers.length > 0) {
                    const count = await social_model.getCountOfReactionsForStory(req);
                    // console.log("count", count);
                    callback(responseCode.SUCCESS, t('data_found'), { allEmojiWithCount: count, userLikeList: likedUsers });
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            })
        } catch (error) {
            console.log("error", error);
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    getStoryLikeUsers: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                const objectId = req.story_id
                // console.log("getStoryLikeUsers", req)
                storyLike.aggregate([
                    {
                        $match: {
                            story_id: objectId
                        }
                    },
                    {
                        $lookup: {
                            from: "users",
                            localField: "user_id",
                            foreignField: "_id",
                            as: "profile"
                        }
                    },
                    {
                        $lookup: {
                            from: "reactions",
                            localField: "like_id",
                            foreignField: "_id",
                            as: "reaction"
                        }
                    },
                    {
                        $unwind: '$profile'
                    },
                    {
                        $lookup: {
                            from: 'followers',
                            let: { authorID: '$profile._id' },
                            pipeline: [
                                {
                                    $match: {
                                        $expr: {
                                            $and: [
                                                { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                                { $eq: ['$following_id', '$$authorID'] }
                                            ]
                                        }
                                    }
                                }
                            ],
                            as: 'followInfo'
                        }
                    },
                    {
                        $lookup: {
                            from: 'followers',
                            let: { authorID: '$profile._id' },
                            pipeline: [
                                {
                                    $match: {
                                        $expr: {
                                            $and: [
                                                { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                                { $eq: ['$following_id', '$$authorID'] }
                                            ]
                                        }
                                    }
                                },
                                {
                                    $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                                }
                            ],
                            as: 'followStatus'
                        },
                    },
                    {
                        $addFields: {
                            userLiked: true,
                            followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                            isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } }
                            // Add more fields for other emoji types as needed
                        }
                    },
                    {
                        $sort: { createdAt: -1 }
                    },
                    {
                        $project: {
                            _id: 1,
                            story_id: 1,
                            user_id: 1,
                            profile: {
                                user_id: '$profile._id',
                                username: '$profile.username',
                                user_id: '$profile._id',
                                first_name: '$profile.first_name',
                                last_name: '$profile.last_name',
                                profile_image:
                                {
                                    $cond: {
                                        if: { $ne: ['$profile.profile_image', ''] }, then: {
                                            $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                        }, else: ''
                                    }
                                },
                                email: '$profile.email'
                            },
                            userLikeEmoji: {
                                $arrayElemAt: [{
                                    $map: {
                                        input: '$reaction',
                                        as: 'reactions',
                                        in: {
                                            _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                            image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                        }
                                    }
                                },
                                    0]
                            },
                            followStatus: '$followStatus.status',
                            isFollowed: 1,
                            userLiked: 1,

                        }
                    },
                    { $skip: 0 },
                    { $limit: 3 }
                ], async (err, likedUsers) => {
                    // console.log("getStoryLikeUsers likedUsers", likedUsers)
                    if (err) {
                        console.log("err", err);
                        reject(err)
                    } else if (likedUsers.length > 0) {
                        resolve(likedUsers)
                    } else {
                        resolve([])
                    }
                })
            } catch (error) {
                reject(error)
            }
        })
    },
    getCountOfReactionsForStory: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                const objectId = mongoose.Types.ObjectId(req.story_id);
                reactionModel.aggregate([
                    {
                        $project: {
                            _id: 1,
                            image: {
                                $concat: [constant.s3Bucket_emoji, '$image']
                            },
                            name: 1,
                        }
                    }
                ]).exec((err, result) => {
                    if (err) {
                        reject(error)
                    } else {
                        if (result.length > 0) {
                            asyncLoop(result, async (item, next) => {
                                let count = await storyLike.countDocuments({ story_id: objectId, like_id: item._id });
                                item.count = count;
                                next();
                            }, () => {
                                resolve(result);
                            })
                        } else {
                            resolve([]);

                        }
                    }
                });
            } catch (error) {
                reject(error)
            }
        })
    },
    /* Reels Functions  */
    addReel: (req, callback) => {
        try {
            if (req.reels.length > 0) {
                asyncLoop(req.reels, (item, next) => {
                    let addData = {
                        user_id: req.user_id,
                        media_type: item.media_type,
                        media: item.media,
                        music: (item.music != undefined && item.music != '') ? item.music : '',
                        thumb_image: (item.thumb_image != undefined && item.thumb_image != '') ? item.thumb_image : '',
                        description: (item.description != undefined && item.description != '') ? item.description : '',
                        index_number: (item.index_number != undefined && item.index_number != '') ? item.index_number : '0',
                        privacy: item.privacy,
                    }
                    if (item.location_latitude != undefined && item.location_longitude != undefined && item.location_full_address != undefined && item.location != undefined) {
                        addData.location_latitude = item.location_latitude;
                        addData.location_longitude = item.location_longitude;
                        addData.location_full_address = item.location_full_address;
                        addData.location = item.location;
                    }
                    reelModel.create(addData, (err, result) => {
                        if (!err) {
                            next();
                        } else {
                            next(err);
                            return;
                        }
                    });
                }, (err) => {
                    if (!err) {
                        callback(responseCode.SUCCESS, t('reel_added_success'), {});
                    } else {
                        console.log("Error while adding data to reels : ", err);
                        callback(responseCode.OPERATION_FAILED, err, {});
                    }
                })
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {});
            }
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {});
        }
    },
    /**
  * @Descriprion API method for get reels 
  */
    getReelsForPost: async (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            // let following_users = await social_model.getFollowUsers(req);
            // console.log("following_users", following_users)
            let condition = {
                $match: {
                    is_delete: "0",
                }
            }
            // console.log("condition", condition)
            reelModel.aggregate([
                condition,
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'

                    }
                },
                {
                    "$unwind": "$profile"
                },
                {
                    // Lookup to join with blocked_user collection to find users blocked by current user
                    $lookup: {
                        from: 'blocked_users',
                        let: { postUserId: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $or: [
                                            { $and: [{ $eq: ['$user_id', req.user_id] }, { $eq: ['$block_user_id', '$$postUserId'] }] },
                                            { $and: [{ $eq: ['$block_user_id', req.user_id] }, { $eq: ['$user_id', '$$postUserId'] }] },
                                        ],
                                    },
                                },
                            },
                        ],
                        as: 'blockedInfo',
                    },
                },
                {
                    $lookup: {
                        from: 'reel_likes',
                        localField: '_id',
                        foreignField: 'reel_id',
                        as: 'Totallikes'
                    }
                },
                {
                    $lookup: {
                        from: 'reel_likes',
                        let: { reelId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$user_id', req.user_id] }, // Match saved reel by user ID
                                            { $eq: ['$reel_id', '$$reelId'] } // Match saved reel by reel ID
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { _id: 1, like_id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $lookup: {
                        from: 'reel_comments',
                        localField: '_id',
                        foreignField: 'reel_id',
                        as: 'reelComments'
                    }
                },
                {
                    $lookup: {
                        from: 'reel_viewers',
                        localField: '_id',
                        foreignField: 'reel_id',
                        as: 'reelViewers'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] },
                                            { $eq: ['$status', 'Accept'] },
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followerPrivacy'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followInfo'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'followStatus'
                    },
                },
                {
                    $lookup: {
                        from: 'families',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$family_member_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$user_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'family'
                    }
                },
                {
                    $addFields: {
                        totalLikes: { $size: '$Totallikes' },
                        totalComments: { $size: '$reelComments' },
                        totalViewers: { $size: '$reelViewers' },
                        userLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$Totallikes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } },
                        followerPrivacy: { $cond: { if: { $gt: [{ $size: '$followerPrivacy' }, 0] }, then: true, else: false } },
                        followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                        isFamily: { $cond: { if: { $gt: [{ $size: '$family' }, 0] }, then: true, else: false } },
                    }
                },
                {
                    $match: {
                        'blockedInfo': { $eq: [] },
                        $or: [
                            { 'privacy': 'Public' },
                            { 'user_id': req.user_id },
                            { 'privacy': 'Following', followerPrivacy: true },
                            { 'privacy': 'FamilyOnly', isFamily: true },
                            { 'privacy': 'OnlyMe', 'profile._id': req.user_id }
                        ]
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        _id: 1,
                        user_id: 1,
                        media_type: 1,
                        location: 1,
                        description: 1,
                        music: 1,
                        privacy: 1,
                        is_delete: 1,
                        viewCount: 1,
                        createdAt: 1,
                        location_latitude: 1,
                        location_longitude: 1,
                        location_full_address: 1,
                        profile: {
                            username: '$profile.username',
                            user_id: '$profile._id',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$profile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$profile.email'
                        },
                        media:
                        {
                            $cond: {
                                if: { $ne: ['$media', ''] }, then: {
                                    $concat: [constant.s3Bucket_reel_media, '$media']
                                }, else: ''
                            }
                        },
                        thumb_image: {
                            $cond: {
                                if: { $ne: ['$thumb_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_reel_media, '$thumb_image']
                                }, else: ''
                            }
                        },
                        totalLikes: 1,
                        userLiked: 1,
                        totalComments: 1,
                        totalViewers: 1,
                        userLikeEmoji: {
                            $arrayElemAt: [{
                                $map: {
                                    input: '$reaction',
                                    as: 'reactions',
                                    in: {
                                        _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                        image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                    }
                                }
                            },
                                0]
                        },
                        followStatus: '$followStatus.status',
                        isFollowed: 1,
                        // Project other fields from the Post model as needed
                    }
                },
                { $skip: 0 },
                { $limit: 5 }
            ]).then(_reels => {
                if (_reels.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _reels);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            })
        } catch (error) {
            console.log(error)
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    getReels: async (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            // let following_users = await social_model.getFollowUsers(req);
            // console.log("following_users", following_users)
            let condition = {
                $match: {
                    is_delete: "0",
                }
            }
            console.log("condition", condition)
            reelModel.aggregate([
                condition,
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'

                    }
                },
                {
                    "$unwind": "$profile"
                },
                {
                    $lookup: {
                        from: 'reel_likes',
                        localField: '_id',
                        foreignField: 'reel_id',
                        as: 'Totallikes'
                    }
                },
                {
                    $lookup: {
                        from: 'reel_likes',
                        let: { reelId: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$user_id', req.user_id] }, // Match saved reel by user ID
                                            { $eq: ['$reel_id', '$$reelId'] } // Match saved reel by reel ID
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { _id: 1, like_id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $lookup: {
                        from: 'reel_comments',
                        localField: '_id',
                        foreignField: 'reel_id',
                        as: 'reelComments'
                    }
                },
                {
                    $lookup: {
                        from: 'reel_viewers',
                        localField: '_id',
                        foreignField: 'reel_id',
                        as: 'reelViewers'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] },
                                            { $eq: ['$status', 'Accept'] },
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followerPrivacy'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followInfo'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'followStatus'
                    },
                },
                {
                    $lookup: {
                        from: 'families',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$family_member_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$user_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'family'
                    }
                },
                {
                    $addFields: {
                        totalLikes: { $size: '$Totallikes' },
                        totalComments: { $size: '$reelComments' },
                        totalViewers: { $size: '$reelViewers' },
                        userLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$Totallikes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } },
                        followerPrivacy: { $cond: { if: { $gt: [{ $size: '$followerPrivacy' }, 0] }, then: true, else: false } },
                        followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                        isFamily: { $cond: { if: { $gt: [{ $size: '$family' }, 0] }, then: true, else: false } },
                    }
                },
                {
                    $match: {
                        $or: [
                            { 'user_id': req.user_id },
                            { 'privacy': 'Public' },
                            { 'privacy': 'Following', followerPrivacy: true },
                            { 'privacy': 'FamilyOnly', isFamily: true },
                            { 'privacy': 'OnlyMe', 'profile._id': req.user_id }
                        ]
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        _id: 1,
                        user_id: 1,
                        media_type: 1,
                        location: 1,
                        description: 1,
                        music: 1,
                        privacy: 1,
                        is_delete: 1,
                        viewCount: 1,
                        createdAt: 1,
                        location_latitude: 1,
                        location_longitude: 1,
                        location_full_address: 1,
                        profile: {
                            username: '$profile.username',
                            user_id: '$profile._id',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$profile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$profile.email'
                        },
                        media:
                        {
                            $cond: {
                                if: { $ne: ['$media', ''] }, then: {
                                    $concat: [constant.s3Bucket_reel_media, '$media']
                                }, else: ''
                            }
                        },
                        thumb_image: {
                            $cond: {
                                if: { $ne: ['$thumb_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_reel_media, '$thumb_image']
                                }, else: ''
                            }
                        },
                        totalLikes: 1,
                        userLiked: 1,
                        totalComments: 1,
                        totalViewers: 1,
                        userLikeEmoji: {
                            $arrayElemAt: [{
                                $map: {
                                    input: '$reaction',
                                    as: 'reactions',
                                    in: {
                                        _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                        image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                    }
                                }
                            },
                                0]
                        },
                        followStatus: '$followStatus.status',
                        isFollowed: 1,
                        // Project other fields from the Post model as needed
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_reels => {
                if (_reels.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _reels);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            })
        } catch (error) {
            console.log(error)
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    reelDetail: async (req, callback) => {
        try {
            social_model.reelView(req, (code, message, data) => {

                const objectId = mongoose.Types.ObjectId(req.reel_id);
                // console.log("objectId", objectId)
                reelModel.aggregate([
                    {
                        $match: {
                            _id: objectId
                        }
                    },
                    {
                        $lookup: {
                            from: 'users',
                            localField: 'user_id',
                            foreignField: '_id',
                            as: 'profile'
                        }
                    },
                    {
                        "$unwind": "$profile"
                    },
                    {
                        $lookup: {
                            from: 'reel_likes',
                            localField: '_id',
                            foreignField: 'reel_id',
                            as: 'Totallikes'
                        }
                    },
                    {
                        $lookup: {
                            from: 'reel_likes',
                            let: { reelId: '$_id' },
                            pipeline: [
                                {
                                    $match: {
                                        $expr: {
                                            $and: [
                                                { $eq: ['$user_id', req.user_id] }, // Match saved reel by user ID
                                                { $eq: ['$reel_id', '$$reelId'] } // Match saved reel by reel ID
                                            ]
                                        }
                                    }
                                },
                                {
                                    $project: { _id: 1, like_id: 1 } // Include only _id field to check if there's a match
                                }
                            ],
                            as: 'likes'
                        }
                    },
                    {
                        $lookup: {
                            from: 'reactions',
                            localField: 'likes.like_id',
                            foreignField: '_id',
                            as: 'reaction'
                        }
                    },
                    {
                        $lookup: {
                            from: 'reel_comments',
                            localField: '_id',
                            foreignField: 'reel_id',
                            as: 'reelComments'
                        }
                    },
                    {
                        $lookup: {
                            from: 'reel_viewers',
                            localField: '_id',
                            foreignField: 'reel_id',
                            as: 'reelViewers'
                        }
                    },
                    {
                        $lookup: {
                            from: 'followers',
                            let: { authorID: '$profile._id' },
                            pipeline: [
                                {
                                    $match: {
                                        $expr: {
                                            $and: [
                                                { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                                { $eq: ['$following_id', '$$authorID'] }
                                            ]
                                        }
                                    }
                                }
                            ],
                            as: 'followInfo'
                        }
                    },
                    {
                        $lookup: {
                            from: 'followers',
                            let: { authorID: '$profile._id' },
                            pipeline: [
                                {
                                    $match: {
                                        $expr: {
                                            $and: [
                                                { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                                { $eq: ['$following_id', '$$authorID'] }
                                            ]
                                        }
                                    }
                                },
                                {
                                    $project: { status: 1, _id: 0 } // Include only _id field to check if there's a match
                                }
                            ],
                            as: 'followStatus'
                        },
                    },
                    {
                        $lookup: {
                            from: 'families',
                            let: { authorID: '$profile._id' },
                            pipeline: [
                                {
                                    $match: {
                                        $expr: {
                                            $and: [
                                                { $eq: ['$family_member_id', req.user_id] }, // Check if the current user follows the author
                                                { $eq: ['$user_id', '$$authorID'] }
                                            ]
                                        }
                                    }
                                }
                            ],
                            as: 'family'
                        }
                    },
                    {
                        $addFields: {
                            totalLikes: { $size: '$Totallikes' },
                            totalComments: { $size: '$reelComments' },
                            totalViewers: { $size: '$reelViewers' },
                            userLiked: {
                                $cond: {
                                    if: { $in: [req.user_id, '$Totallikes.user_id'] },
                                    then: true,
                                    else: false
                                }
                            },
                            isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } },
                            followStatus: { $cond: { if: { $gt: [{ $size: '$followStatus' }, 0] }, then: { $arrayElemAt: ['$followStatus', 0] }, else: { status: "" } } },
                            isFamily: { $cond: { if: { $gt: [{ $size: '$family' }, 0] }, then: true, else: false } },
                        }
                    },
                    // {
                    //     $match: {
                    //         $or: [
                    //             { 'privacy': 'Public' },
                    //             // { 'privacy': 'Following', isFollowed: true },
                    //             { 'privacy': 'FamilyOnly', isFamily: true },
                    //             { 'privacy': 'OnlyMe', 'profile._id': req.user_id }
                    //         ]
                    //     }
                    // },
                    {
                        $sort: { createdAt: -1 }
                    },
                    {
                        $project: {
                            _id: 1,
                            user_id: 1,
                            media_type: 1,
                            location: 1,
                            description: 1,
                            music: 1,
                            privacy: 1,
                            is_delete: 1,
                            viewCount: 1,
                            createdAt: 1,
                            location_latitude: 1,
                            location_longitude: 1,
                            location_full_address: 1,
                            profile: {
                                username: '$profile.username',
                                user_id: '$profile._id',
                                first_name: '$profile.first_name',
                                last_name: '$profile.last_name',
                                profile_image:
                                {
                                    $cond: {
                                        if: { $ne: ['$profile.profile_image', ''] }, then: {
                                            $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                        }, else: ''
                                    }
                                },
                                email: '$profile.email'
                            },
                            media:
                            {
                                $cond: {
                                    if: { $ne: ['$media', ''] }, then: {
                                        $concat: [constant.s3Bucket_reel_media, '$media']
                                    }, else: ''
                                }
                            },
                            thumb_image: {
                                $cond: {
                                    if: { $ne: ['$thumb_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_reel_media, '$thumb_image']
                                    }, else: ''
                                }
                            },
                            totalLikes: 1,
                            userLiked: 1,
                            totalComments: 1,
                            totalViewers: 1,
                            userLikeEmoji: {
                                $arrayElemAt: [{
                                    $map: {
                                        input: '$reaction',
                                        as: 'reactions',
                                        in: {
                                            _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                            image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                        }
                                    }
                                },
                                    0]
                            },
                            followStatus: '$followStatus.status',
                            isFollowed: 1,
                            // Project other fields from the Post model as needed
                        }
                    },

                ]).then(_reels => {
                    // console.log("_reels", _reels)
                    if (_reels.length > 0) {
                        callback(responseCode.SUCCESS, t('data_found'), _reels[0]);
                    } else {
                        callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
                    }
                })
            });
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    /**
  * API method for Reel view 
  */
    reelView: async (req, callback) => {
        try {
            const ReelId = mongoose.Types.ObjectId(req.reel_id);
            let Reel = await reelModel.findOne({ _id: ReelId });
            let Reel_viewer = await reelViewer.find({ viewer_id: req.user_id, reel_id: ReelId });
            if (Reel_viewer.length == 0) {
                let Reel_viewer_data = {
                    Reel_user_id: Reel.user_id,
                    viewer_id: req.user_id,
                    reel_id: req.reel_id
                }
                let inserted_data = await reelViewer.create(Reel_viewer_data);
                let Reel_data = await reelModel.findByIdAndUpdate(req.reel_id, {
                    $inc: { viewCount: 1 }, // Increment the view count by 1
                }, { new: true });
                callback(responseCode.SUCCESS, t('reel_viewed'), {})
            } else {
                callback(responseCode.SUCCESS, t('reel_viewed'), {})
            }
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
        }
    },
    /**
   * API method for like/unlike Reel 
   */
    likeReel: (req, callback) => {
        try {
            const objectId = mongoose.Types.ObjectId(req.reel_id);
            reelLike.find({ reel_id: objectId, user_id: req.user_id }, (err, result) => {
                console.log(result)
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    reelLike.updateOne({ _id: result[0]._id }, { like_id: req.like_id }, (err, result) => {
                        if (!err) {
                            reelModel.find({ _id: objectId }, async (err, reel_data) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else if (reel_data.length > 0) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the post
                                    let notifi_data = {
                                        receiver_id: reel_data[0].user_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has liked your reel",
                                        tag: "like_reel",
                                        image_url: "",
                                        like_id: req.like_id,
                                        reel_id: req.reel_id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                        } else {
                                            console.log("data", data)
                                        }
                                    })
                                    social_model.reelDetail({ reel_id: req.reel_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            // data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('reel_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                } else {
                                    console.log("Reel Something went wrong")
                                    let count = await postLike.countDocuments({ reel_id: objectId });
                                    social_model.reelDetail({ reel_id: req.reel_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            // data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('reel_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                }
                            })

                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    reelLike.create({ user_id: req.user_id, reel_id: req.reel_id, like_id: req.like_id }, (err, result) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            const objectId = mongoose.Types.ObjectId(req.reel_id);
                            reelModel.find({ _id: objectId }, async (err, reel_data) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else if (reel_data.length > 0) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the reel
                                    let notifi_data = {
                                        receiver_id: reel_data[0].user_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has liked your reel",
                                        tag: "like_reel",
                                        image_url: "",
                                        like_id: req.like_id,
                                        reel_id: req.reel_id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                        } else {
                                            console.log("data", data)
                                        }
                                    })
                                    let count = await reelLike.countDocuments({ reel_id: objectId });
                                    social_model.reelDetail({ reel_id: req.reel_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            // data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('reel_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                } else {
                                    console.log("Reel Something went wrong")
                                    social_model.reelDetail({ reel_id: req.reel_id, user_id: req.user_id }, (code, message, data) => {
                                        if (code != "0") {
                                            // data.userLiked = true;
                                            callback(responseCode.SUCCESS, t('reel_liked'), data)
                                        } else {
                                            callback(responseCode.OPERATION_FAILED, message, {});
                                        }
                                    })
                                }
                            })
                        }
                    })
                }
            })
        } catch (e) {
            console.log("Error in likeUnlikeReel : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    unlikeReel: (req, callback) => {
        try {
            reelLike.find({ reel_id: req.reel_id, user_id: req.user_id }, (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the reel
                    reelLike.deleteOne({ _id: result[0]._id }, async (err, result) => {
                        if (!err) {
                            const objectId = mongoose.Types.ObjectId(req.reel_id);
                            let count = await reelLike.countDocuments({ reel_id: objectId });
                            // console.log("count", count)
                            social_model.reelDetail({ reel_id: req.reel_id, user_id: req.user_id }, (code, message, data) => {
                                if (code != "0") {
                                    callback(responseCode.SUCCESS, t('reel_unliked'), data);
                                } else {
                                    callback(responseCode.OPERATION_FAILED, message, {});
                                }
                            })
                            // callback(responseCode.SUCCESS, t('reel_unliked'), { totalLikes: count })
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('something_went_wrong'), {});
                }
            })
        } catch (e) {
            console.log("Error in UnlikeReel : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    deleteReel: async (req, callback) => {
        try {
            const reelId = mongoose.Types.ObjectId(req.reel_id);
            let reelData = await reelModel.find({ _id: reelId, user_id: req.user_id });
            if (reelData.length > 0) {
                let reel = await reelModel.deleteOne({ _id: reelId });
                let reelLike = await reelLike.deleteMany({ reel_id: reelId });
                let reelView = await reelViewer.deleteMany({ reel_id: reelId });
                let reelComment = await reelComments.deleteMany({ reel_id: reelId });
                callback(responseCode.SUCCESS, t('reel_deleted'), {})
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
            }
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
        }
    },
    addReelComment: (req, callback) => {
        try {
            let comment_data = {
                user_id: req.user_id,
                reel_id: req.reel_id,
                message: req.message
            }
            if (req.parentComment != undefined && req.parentComment != '') {
                comment_data.parentComment = req.parentComment;
            }
            reelComments.create(comment_data, async (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    if (req.parentComment != undefined && req.parentComment != '') {
                        var objectId = mongoose.Types.ObjectId(req.parentComment);
                        var msg = " has replied on your Comment";
                    } else {
                        var objectId = mongoose.Types.ObjectId(req.reel_id);
                        var msg = " has comment on your Post";
                    }
                    reelComments.find({ _id: objectId }, async (err, comment_data) => {
                        // console.log("comment_data", comment_data)
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else if (comment_data.length > 0) {
                            let userData = await common.userDetails({ receiver_id: comment_data[0].user_id });
                            // send notification to the owner of the reel
                            let notifi_data = {
                                receiver_id: comment_data[0].user_id,
                                sender_id: req.user_id,
                                title: "Neshwa",
                                message: userData.username + msg,
                                tag: "comment_on_reel",
                                image_url: "",
                                comment_id: result._id,
                                reel_id: req.reel_id,
                            }
                            common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                if (err) {
                                    console.log("err", err)
                                } else {
                                    console.log("data", data)
                                }
                            })
                            callback(responseCode.SUCCESS, t('comment_created'), {});
                        } else {
                            console.log("Post Something went wrong")
                            callback(responseCode.SUCCESS, t('comment_created'), {});
                        }
                    })
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    deleteMyReelComment: async (req, callback) => {
        try {
            let comment_id = mongoose.Types.ObjectId(req.comment_id);
            let reel_id = mongoose.Types.ObjectId(req.reel_id);
            /* remove all likes then delete comment */
            let deleteLike = await commentLikeModel.deleteMany({ comment_id: comment_id });
            let deleteComment = await reelComments.deleteMany({ _id: comment_id });
            let parentComment = await reelComments.deleteMany({ parentComment: comment_id });
            let totalComment = await reelComments.countDocuments({ reel_id: reel_id });
            callback(responseCode.SUCCESS, t('comment_deleted'), { totalComments: totalComment });
        } catch (err) {
            callback(responseCode.OPERATION_FAILED, err, {})
        }
    },
    getReelComment: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            const objectId = mongoose.Types.ObjectId(req.reel_id);
            reelComments.aggregate([
                { $match: { reel_id: objectId, parentComment: null } },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'
                    }
                },
                {
                    "$unwind": "$profile"
                },
                {
                    $lookup: {
                        from: 'reel_comment_likes',
                        localField: '_id',
                        foreignField: 'comment_id',
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $addFields: {
                        totalLikes: { $size: '$likes' },
                        userLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$likes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        // userLikeEmoji: {
                        //     $cond: {
                        //         if: { $in: [req.user_id, '$likes.user_id'] },
                        //         then: '$reaction.image',
                        //         else: ''
                        //     }
                        // }

                        // Add more fields for other emoji types as needed
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        profile: {
                            username: '$profile.username',
                            user_id: '$profile._id',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$profile.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$profile.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$profile.email'
                        },
                        message: 1,
                        reel_id: 1,
                        _id: 1,
                        parentComment: 1,
                        createdAt: 1,
                        updatedAt: 1,
                        totalLikes: 1,
                        userLiked: 1,
                        // userLikeEmoji: 1,
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(result => {
                if (result.length > 0) {
                    asyncLoop(result, (item, next) => {
                        reelComments.aggregate([
                            {
                                $match: {
                                    reel_id: objectId,
                                    parentComment: item._id
                                }
                            },
                            {
                                $lookup: {
                                    from: 'users',
                                    localField: 'user_id',
                                    foreignField: '_id',
                                    as: 'profile'

                                }
                            },
                            {
                                "$unwind": "$profile"
                            },
                            {
                                $lookup: {
                                    from: 'reel_comment_likes',
                                    localField: '_id',
                                    foreignField: 'comment_id',
                                    as: 'likes'
                                }
                            },
                            {
                                $addFields: {
                                    totalLikes: { $size: '$likes' },
                                    userLiked: {
                                        $cond: {
                                            if: { $in: [req.user_id, '$likes.user_id'] },
                                            then: true,
                                            else: false
                                        }
                                    },
                                    // Add more fields for other emoji types as needed
                                }
                            },
                            {
                                $sort: { createdAt: 1 }
                            },
                            {
                                $project: {
                                    profile: {
                                        username: '$profile.username',
                                        user_id: '$profile._id',
                                        first_name: '$profile.first_name',
                                        last_name: '$profile.last_name',
                                        profile_image:
                                        {
                                            $concat: [constant.s3Bucket_profile, {
                                                $ifNull: ['$profile.profile_image', 'default-avatar.png']
                                            }]
                                        },
                                        email: '$profile.email'
                                    },
                                    message: 1,
                                    reel_id: 1,
                                    _id: 1,
                                    createdAt: 1,
                                    comment_id: '$parentComment',
                                    updatedAt: 1,
                                    totalLikes: 1,
                                    userLiked: 1,
                                }
                            }
                        ]).then(replies => {
                            item.replies = replies
                            next();
                        })
                    }, () => {
                        callback(responseCode.SUCCESS, t('data_found'), result)
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, [])
        }
    },
    likeReelComment: (req, callback) => {
        try {
            reelCommentLikeModel.find({ comment_id: req.comment_id, user_id: req.user_id }, (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    reelCommentLikeModel.updateOne({ _id: result[0]._id }, { like_id: req.like_id }, (err, result) => {
                        if (!err) {
                            const objectId = mongoose.Types.ObjectId(req.comment_id);
                            reelComments.find({ _id: objectId }, async (err, comment_data) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else if (comment_data.length > 0) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the post
                                    let notifi_data = {
                                        receiver_id: comment_data[0].user_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has liked your comment",
                                        tag: "reel_like_comment",
                                        image_url: "",
                                        like_id: result._id,
                                        comment_id: req.comment_id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                        } else {
                                            console.log("data", data)
                                        }
                                    })
                                    let count = await reelCommentLikeModel.countDocuments({ comment_id: objectId });
                                    callback(responseCode.SUCCESS, t('comment_liked'), { totalLikes: count });
                                } else {
                                    console.log("Comment Something went wrong")
                                    let count = await reelCommentLikeModel.countDocuments({ comment_id: objectId });
                                    callback(responseCode.SUCCESS, t('comment_liked'), { totalLikes: count });
                                }
                            })
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    reelCommentLikeModel.create({ user_id: req.user_id, comment_id: req.comment_id, like_id: req.like_id }, (err, result) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            const objectId = mongoose.Types.ObjectId(req.comment_id);
                            reelComments.find({ _id: objectId }, async (err, comment_data) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else if (comment_data.length > 0) {
                                    let userData = await common.userDetails({ receiver_id: req.user_id });
                                    // send notification to the owner of the post
                                    let notifi_data = {
                                        receiver_id: comment_data[0].user_id,
                                        sender_id: req.user_id,
                                        title: "Neshwa",
                                        message: userData.username + " has liked your comment",
                                        tag: "reel_like_comment",
                                        image_url: "",
                                        like_id: result._id,
                                        comment_id: req.comment_id,
                                    }
                                    common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                                        if (err) {
                                            console.log("err", err)
                                        } else {
                                            console.log("data", data)
                                        }
                                    })
                                    let count = await reelCommentLikeModel.countDocuments({ comment_id: objectId });
                                    callback(responseCode.SUCCESS, t('comment_liked'), { totalLikes: count })
                                } else {
                                    console.log("Comment Something went wrong")
                                    let count = await reelCommentLikeModel.countDocuments({ comment_id: objectId });
                                    callback(responseCode.SUCCESS, t('comment_liked'), { totalLikes: count })
                                }
                            })
                        }
                    })
                }
            })
        } catch (e) {
            console.log("Error in likeUnlikePost : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    unlikeReelComment: (req, callback) => {
        try {
            reelCommentLikeModel.find({ comment_id: req.comment_id, user_id: req.user_id }, (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    reelCommentLikeModel.deleteOne({ _id: result[0]._id }, (err, result) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('comment_unliked'), {})
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('something_went_wrong'), {});
                }
            })
        } catch (e) {
            console.log("Error in likeUnlikePost : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
};

module.exports = social_model