const responseCode = require('../../../config/response-error-code');
const con = require('../../../config/database');
const common = require('../../../utilities/common');
const validate_header = require('../../../middleware/validate-header');
const moment = require('moment');
const constant = require('../../../config/constant');
const asyncLoop = require('node-async-loop')
const userModel = require('../../../model/user');
const contactModel = require('../../../model/contact');
const followersModel = require('../../../model/followers');
const familyModel = require('../../../model/family');
const userInerestsModel = require('../../../model/user_interests');
const userQuesAnsModel = require('../../../model/user_question_answers');
const userDeviceModel = require('../../../model/user_device_info');
const blockedUserModel = require('../../../model/blocked_users');
const notificationModel = require('../../../model/notification');
const mongoose = require('mongoose');
const { t } = require('localizify');
let user_model = {
    sendNotification: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                let userData = await common.userDetails({ receiver_id: req.user_id });
                console.log("userData 23 =>", userData)
                // send notification to the owner of the post
                let notifi_data = {
                    receiver_id: req.receiver_id,
                    sender_id: req.user_id,
                    title: "Neshwa",
                    tag: req.tag,
                    image_url: ""
                }
                if (req.tag == 'follow_request') {
                    notifi_data.message = userData.username + " has sent to you follow request.";
                    notifi_data.follow_id = req.follow_id
                } else if (req.tag == 'follower') {
                    notifi_data.message = userData.username + " has followed you.";
                    notifi_data.follow_id = req.follow_id
                } else if (req.tag == 'accept_follow_request') {
                    notifi_data.message = userData.username + " has accept your follow request you.";
                    notifi_data.follow_id = req.follow_id
                }
                console.log("notifi_data 42 =>", notifi_data)

                common.sendNotificationForAllDevices(notifi_data, (err, data) => {
                    if (err) {
                        console.log("err", err)
                        reject("something_went_wrong");
                    } else {
                        console.log("data", data)
                        resolve(true);
                    }
                })
            } catch (e) {
                console.log("e", e)
                reject("something_went_wrong");
            }
        });
    },
    api_user_list: (callback) => {
        userModel.aggregate([
            {
                $lookup: {
                    from: 'user_device_infos',     //collection name
                    localField: '_id',
                    foreignField: 'user_id',
                    as: 'userDeviceData'     //alias
                }
            },
            {
                $unwind: '$userDeviceData' // Deconstructs the array of joined documents into separate documents
            },
            {
                $project: {
                    username: 1,
                    nickname: 1,
                    first_name: 1,
                    last_name: 1,
                    dob: 1,
                    email: 1,
                    country_code: 1,
                    phone: 1,
                    gender: 1,
                    profile_image: { $concat: [constant.s3Bucket_profile, "$profile_image"] },
                    blood_type: 1,
                    pin: 1,
                    password: 1,
                    profile_type: 1,
                    signup_step: 1,
                    status: 1,
                    is_verify: 1,
                    token: '$userDeviceData.token', // Select specific field (name) from the joined user data
                    device_type: '$userDeviceData.device_type', // Select specific field (name) from the joined user data
                    device_token: '$userDeviceData.device_token' // Select specific field (name) from the joined user data
                }
            }
        ]).then(result => {
            if (result.length > 0) {
                // console.log("result", result)
                asyncLoop(result, (item, next) => {
                    item.password = (item.password != null && item.password != '') ? validate_header.dcryption(item.password) : ''
                    item.pin = (item.pin != null && item.pin != '') ? validate_header.dcryption(item.pin) : ''
                    // console.log("object", item)
                    next();
                }, () => {
                    callback(result);
                })

            } else {
                console.log("Error:", err);
                callback([]);

            }
        })

    },
    signup: (req, callback) => {
        let password = validate_header.encryption(req.password);
        let insert_data = {
            email: req.email,
            username: req.username,
            password: password,
            otp: '1111'
            // otp: Math.floor(100000 + Math.random() * 900000)
        }
        if (req.country_code != undefined && req.country_code != '') {
            insert_data.country_code = req.country_code
        }
        if (req.phone != undefined && req.phone != '') {
            insert_data.phone = req.phone
        }
        insert_data.referral_code = validate_header.randomTokenString(8);
        if (req.referral_code != undefined && req.referral_code != '') {
            userModel.findOne({ referral_code: req.referral_code }, function (err, userData) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    if (_data.length > 0) {
                        insert_data.referred_by = userData._id;
                    } else {
                        insert_data.referred_by = '';
                    }
                    userModel.create(insert_data, function (err, userObj) {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            let insert_device_data = {
                                user_id: userObj._id,
                                device_type: req.device_type,
                                device_token: req.device_token
                            }
                            userDeviceModel.create(insert_device_data, (err, userDeviceObj) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    common.getUserData({ user_id: userObj._id, device_type: req.device_type }, (err, user) => {
                                        if (err) {
                                            callback(responseCode.OPERATION_FAILED, err, {});
                                        } else {
                                            callback(responseCode.OTP_NOT_VERYFIED, t('verification_code_sent'), user);
                                        }
                                    })
                                }
                            })
                        }
                    })
                }
            })
        } else {
            userModel.create(insert_data, function (err, userObj) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    let insert_device_data = {
                        user_id: userObj._id,
                        device_type: req.device_type,
                        device_token: req.device_token
                    }
                    userDeviceModel.create(insert_device_data, (err, userDeviceObj) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            common.getUserData({ user_id: userObj._id, device_type: req.device_type }, (err, user) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    callback(responseCode.OTP_NOT_VERYFIED, t('verification_code_sent'), user);
                                }
                            })
                        }
                    })
                }
            })
        }

    },
    verifyOTP: (req, callback) => {
        if (req.email != undefined && req.email != '') {
            userModel.findOne({ email: req.email }, (err, user) => {
                if (err) {
                    callback(responseCode.NO_DATA_FOUND, t('user_not_found'), {});
                } else if (user.status == 'Active') {
                    if (user.is_verify == '0') {
                        if (user.otp == req.otp) {
                            let updateData = {
                                otp: '',
                                is_verify: '1',
                                status: 'Active',
                            };
                            if (user.signup_step != 'DONE') {
                                updateData.signup_step = 'VERIFY';
                            }
                            userModel.updateOne({ _id: user._id }, updateData, function (err, data) {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    let updateData = {
                                        token: validate_header.randomTokenString(40),
                                        device_type: req.device_type,
                                        device_token: req.device_token
                                    };
                                    userDeviceModel.updateOne({ user_id: user._id, device_type: req.device_type }, updateData, function (err, data) {
                                        if (err) {
                                            callback(responseCode.OPERATION_FAILED, err, []);
                                        } else {
                                            common.getUserData({ user_id: user._id, device_type: req.device_type }, (err, user) => {
                                                if (err) {
                                                    callback(responseCode.OPERATION_FAILED, err, {});
                                                } else {
                                                    callback(responseCode.SUCCESS, t('verification_success'), user);
                                                }
                                            })
                                        }
                                    })
                                }
                            });
                        } else {
                            callback(responseCode.OPERATION_FAILED, t('invalid_verification_code'), {});
                        }
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('already_verified'), {});
                    }

                } else {
                    return callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {});
                }
            });
        } else if (req.phone != undefined && req.phone != '') {
            userModel.findOne({ phone: req.phone, country_code: req.country_code }, function (err, user) {
                if (!user[0]) {
                    return callback(responseCode.NO_DATA_FOUND, t('user_not_found'), {});
                } else if (user.status == 'Active') {
                    if (user.is_verify == '0') {
                        if (user.otp == req.otp) {
                            let updateData = {
                                otp: '',
                                is_verify: '1',
                                status: 'Active',
                            };
                            if (user.signup_step != 'DONE') {
                                updateData.signup_step = 'VERIFY';
                            }
                            userModel.updateOne({ _id: user._id }, updateData, function (err, data) {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    updateData = {
                                        token: validate_header.randomTokenString(40),
                                        device_type: req.device_type,
                                        device_token: req.device_token
                                    };
                                    userDeviceModel.updateOne({ user_id: user._id, device_type: req.device_type }, updateData, function (err, data) {
                                        if (err) {
                                            callback(responseCode.OPERATION_FAILED, err, []);
                                        } else {
                                            common.getUserData({ user_id: user._id, device_type: req.device_type }, (err, user) => {
                                                if (err) {
                                                    callback(responseCode.OPERATION_FAILED, err, {});
                                                } else {
                                                    callback(responseCode.SUCCESS, t('verification_success'), user);
                                                }
                                            })
                                        }
                                    })
                                }
                            });
                        } else {
                            callback(responseCode.OPERATION_FAILED, t('invalid_verification_code'), {});
                        }
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('already_verified'), {});
                    }

                } else {
                    callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {});
                }

            });
        } else {
            callback(responseCode.OPERATION_FAILED, t('provide_any_one_details'), {});
        }
    },
    /*
     * API method for set pin authentication  
     */
    setPin: (req, callback) => {
        userModel.findById(req.user_id, function (err, user) {
            if (err) {
                callback(responseCode.NO_DATA_FOUND, t('user_not_found'), {});
            } else {
                if (user.is_pin_set == '0') {
                    let password = validate_header.encryption(req.pin);
                    let params = {
                        is_pin_set: 1,
                        pin: password,
                        signup_step: 'DONE'
                    };
                    userModel.updateOne({ _id: user._id }, params, function (err, data) {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            common.getUserData({ user_id: user._id, device_type: req.device_type }, (err, user) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    callback(responseCode.SUCCESS, t('sinup_success'), user);
                                }
                            })
                        }
                    });
                } else {
                    callback(responseCode.OPERATION_FAILED, t('already_pin_set'));
                }
            }
        });
    },
    /* 
    * API method for signin and login with email or phone number or username and paswword
    */
    signIn: (req, callback) => {
        if (req.username != undefined && req.username != '') {
            userModel.find({
                $or: [
                    { 'username': req.username },
                    { 'email': req.username }
                ],
                is_delete: '0'
            }, (err, login_data) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (login_data.length < 1) {
                    callback(responseCode.NO_DATA_FOUND, t('invalid_credentials'), {});
                } else {
                    let userData = login_data[0];
                    if (userData.status == 'Active') {
                        /*  Check whether PIN has been set or not. If yes then check the entered PIN otherwise go to next step.*/
                        /*  
                        * Check whether PIN has been set or not.
                        * If yes then check the entered PIN is correct or not.
                        * If no then simply return the user profile without asking for PIN.
                        */
                        if (req.password == validate_header.dcryption(userData.password)) {
                            let updateData = {
                                device_type: req.device_type,
                                device_token: req.device_token
                            };
                            if (userData.signup_step == 'VERIFY' || userData.signup_step == 'DONE') {
                                updateData.token = validate_header.randomTokenString(40);
                            }
                            // console.log("updateData", updateData)
                            // console.log("userData", userData)
                            userDeviceModel.updateOne({ user_id: userData._id }, updateData, function (err, data) {
                                // console.log("err", err)
                                // console.log("data", data)
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    common.getUserData({ user_id: userData._id, device_type: req.device_type }, (err, user) => {
                                        if (err) {
                                            callback(responseCode.OPERATION_FAILED, err, {});
                                        } else {
                                            callback(responseCode.SUCCESS, t('login_success'), user);
                                        }
                                    })
                                }
                            });
                        } else {
                            callback(responseCode.OPERATION_FAILED, t('invalid_password'), {});
                        }
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {})
                    }
                }
            })
        } else {
            userModel.find({
                phone: req.phone,
                country_code: req.country_code,
                is_delete: '0'
            }, (err, login_data) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (login_data.length < 1) {
                    callback(responseCode.NO_DATA_FOUND, t('invalid_credentials'), {});
                } else {
                    let userData = login_data[0];
                    if (userData.status == 'Active') {
                        /*  Check whether PIN has been set or not. If yes then check the entered PIN otherwise go to next step.*/
                        /*  
                        * Check whether PIN has been set or not.
                        * If yes then check the entered PIN is correct or not.
                        * If no then simply return the user profile without asking for PIN.
                        */
                        if (req.password == validate_header.dcryption(userData.password)) {
                            let updateData = {
                                device_type: req.device_type,
                                device_token: req.device_token
                            };
                            if (userData.signup_step == 'VERIFY' || userData.signup_step == 'DONE') {
                                updateData.token = validate_header.randomTokenString(40);
                            }
                            userDeviceModel.updateOne({ user_id: userData._id }, updateData, function (err, data) {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    common.getUserData({ user_id: userData._id, device_type: req.device_type }, (err, user) => {
                                        if (err) {
                                            callback(responseCode.OPERATION_FAILED, err, {});
                                        } else {
                                            callback(responseCode.SUCCESS, t('login_success'), user);
                                        }
                                    })
                                }
                            });
                        } else {
                            callback(responseCode.OPERATION_FAILED, t('invalid_password'), {});
                        }
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {})
                    }
                }
            })
        }
    },
    /* 
    * API method for signin and login with  pin
    */
    authenticateWithPin: (req, callback) => {
        userModel.find({
            _id: req.user_id,
            is_delete: '0'
        }, (err, login_data) => {
            if (err) {
                callback(responseCode.OPERATION_FAILED, err, {});
            } else if (login_data.length < 1) {
                callback(responseCode.NO_DATA_FOUND, t('invalid_credentials'), {});
            } else {
                let userData = login_data[0];
                if (userData.signup_step == 'DONE') {
                    if (userData.status == 'Active') {
                        /*  Check whether PIN has been set or not. 
                        If yes then check the entered PIN otherwise go to next step.*/
                        /*  
                        * Check whether PIN has been set or not.
                        * If yes then check the entered PIN is correct or not.
                        * If no then simply return the user profile without asking for PIN.
                        */
                        if (req.pin == validate_header.dcryption(userData.pin)) {
                            common.getUserData({ user_id: userData._id, device_type: req.device_type }, (err, user) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    callback(responseCode.SUCCESS, t('login_success'), user);
                                }
                            })
                        } else {
                            callback(responseCode.OPERATION_FAILED, t('invalid_pin'));
                        }
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {})
                    }
                } else {
                    callback(responseCode.OPERATION_FAILED, t('signup_step_incomplete'), {})
                }
            }
        })
    },
    /**
     * API method for enable biometric authentication in mobile app.
     * @param {Object} req - request object
     * @param {Function} callback - callback function
     */
    enableBiometrics: (req, callback) => {
        let update_data = {
            is_biomatric_enable: req.biomatric_enable
        };
        userModel.updateOne({ _id: req.user_id }, update_data, (err, data) => {
            if (err) {
                callback(responseCode.OPERATION_FAILED, err, {});
            } else {
                common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, {});
                    } else {
                        callback(responseCode.SUCCESS, t('enable_biometric_success'), user);
                    }
                })
            }
        });
    },
    /**
     * API method for forgot password
    */
    forgotPassword: (req, callback) => {
        if (req.email != undefined && req.email != '') {
            // Validate email
            userModel.find({ email: req.email, is_delete: '0' }, (err, rows) => {
                if (rows.length > 0) {
                    if (rows[0].status == 'Active') {
                        // var otp = Math.floor(1000 + Math.random() * 9000);
                        // var message = "Your OTP to reset your password is " + otp;
                        let updateData = {
                            otp: '1111',
                            is_verify: 0,
                        };
                        userModel.updateOne({ _id: rows[0]._id }, updateData, (err, data) => {
                            if (err) {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            } else {
                                /* send otp on email */
                                callback(responseCode.SUCCESS, t('verification_code_sent'), { email: rows[0].email, phone: rows[0].phone, country_code: rows[0].country_code });
                            }
                        })
                        // smsService.sendSMS(rows[0].country_code + "" + rows[0].phone, message).then((resp) => {
                        //     callback(responseCode.SUCCESS, 'OTP has been sent successfully', { user_id: rows[0].id, country_code: rows[0].country_code})
                        // }).catch((error) => {
                        //     callback(responseCode.OTP_NOT_SENT, error, null);
                        // });
                    } else {
                        return callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {});
                    }
                } else {
                    callback(responseCode.OPERATION_FAILED, t('user_not_found'));
                }
            });
        } else if (req.phone != undefined && req.phone != '') {
            // Validate phone number
            userModel.find({ phone: req.phone, country_code: req.country_code, is_delete: '0' }, (err, rows) => {
                if (rows.length > 0) {
                    if (rows[0].status == 'Active') {
                        // let otp = Math.floor(1000 + Math.random() * 9000);
                        // let message = "Your OTP to reset your password is " + otp;
                        let updateData = {
                            otp: '1111',
                            is_verify: 0,
                        };
                        userModel.updateOne({ _id: rows[0]._id }, updateData, (err, data) => {
                            if (err) {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            } else {
                                /* send otp on email */
                                callback(responseCode.SUCCESS, t('verification_code_sent'), { email: rows[0].email, phone: rows[0].phone, country_code: rows[0].country_code });
                            }
                        })
                        // smsService.sendSMS(rows[0].country_code + "" + rows[0].phone, message).then((resp) => {
                        //     callback(responseCode.SUCCESS, 'OTP has been sent successfully', { user_id: rows[0].id, country_code: rows[0].country_code})
                        // }).catch((error) => {
                        //     callback(responseCode.OTP_NOT_SENT, error, null);
                        // });
                    } else {
                        return callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {});
                    }
                } else {
                    callback(responseCode.OPERATION_FAILED, t('user_not_found'));
                }
            });
        } else {
            callback(responseCode.OPERATION_FAILED, t('provide_any_one_details'), {})
        }
    },
    /**
     * API method for logout from
      */
    logout: (req, callback) => {
        console.log("logout req- ", req)
        let update_data = {
            token: ''
        }
        userDeviceModel.updateMany({ user_id: req.user_id, device_type: req.device_type }, update_data, (err, result) => {
            if (err) {
                callback(responseCode.OPERATION_FAILED, err, null)
            } else {
                callback(responseCode.SUCCESS, t('logout_success'), {});
            }
        })
    },
    /*
    * API method for change password 
    */
    changePassword: (req, callback) => {
        if (req.email != undefined && req.email != '') {
            userModel.find({ email: req.email }, (err, user) => {
                if (!user) {
                    return callback(responseCode.NO_DATA_FOUND, t('user_not_found'), {});
                } else {
                    let password = validate_header.encryption(req.password);
                    let params = {
                        password: password
                    };
                    userModel.updateOne({ _id: user[0]._id }, params, (err, data) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('password_change_success'), {});

                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    });
                }
            });
        } else if (req.phone != undefined && req.phone != '') {
            userModel.find({ phone: req.phone, country_code: req.country_code }, (err, user) => {
                if (!user[0]) {
                    return callback(responseCode.NO_DATA_FOUND, t('user_not_found'), {});
                } else {
                    let password = validate_header.encryption(req.password);
                    let params = {
                        password: password
                    };
                    userModel.updateOne({ _id: user[0]._id }, params, (err, data) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('password_change_success'), {});

                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    });
                }
            });
        } else {
            return callback(responseCode.OPERATION_FAILED, 'Please provide either email or phone number', null);
        }

    },

    /*
    * API method for change pin 
    */
    changePin: (req, callback) => {
        userModel.find({ _id: req.user_id }, (err, user) => {
            if (!user[0] || err) {
                return callback(responseCode.NO_DATA_FOUND, t('user_not_found'), {});
            } else if (user[0].pin == validate_header.encryption(req.old_pin)) {
                let password = validate_header.encryption(req.pin);
                let params = {
                    pin: password
                };
                userModel.updateOne({ _id: req.user_id }, params, (err, data) => {
                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, {});
                    } else {
                        common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                            if (err) {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            } else {
                                callback(responseCode.SUCCESS, t('pin_change_success'), user);
                            }
                        })
                    }
                });
            } else {
                callback(responseCode.OPERATION_FAILED, t('incorrect_old_pin'));
            }
        });
    },
    /**
     * API method for resend otp
      */
    resendOTP: (req, callback) => {
        if (req.email != undefined && req.email != '') {
            let email = req.email;
            //checking whether the user is already exist or not
            userModel.find({ email: email }, (err, result) => {
                if (result && result.length > 0) {
                    // let otp = Math.floor(1000 + Math.random() * 9000);
                    // let message = 'Your One Time Password is :' + otp;
                    // sendEmail("Reset Your Password", "Hi," + message + "Best Regards,  Team");
                    // sendMessage("Reset Your Password", "Hi," + message + "Best Regards, Team");
                    let update_data = {
                        otp: '1111',
                        is_verify: 0
                    }
                    userModel.updateOne({ _id: result[0]._id }, update_data, (err, data) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            delete result[0].otp;
                            callback(responseCode.SUCCESS, t('otp_resend'), result[0]);
                        }
                    });
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('user_not_found'), {})
                }
            });
        } else if (req.phone != undefined && req.phone != '') {
            //checking whether the user is already exist or not
            userModel.find({ phone: req.phone, country_code: req.country_code, }, (err, result) => {
                if (result && result.length > 0) {
                    // let otp = Math.floor(1000 + Math.random() * 9000);
                    // let message = 'Your One Time Password is :' + otp;
                    // sendEmail("Reset Your Password", "Hi," + message + "Best Regards,  Team");
                    // sendMessage("Reset Your Password", "Hi," + message + "Best Regards, Team");
                    let update_data = {
                        otp: '1111',
                        is_verify: 0
                    }
                    userModel.updateOne({ _id: result[0]._id }, update_data, (err, data) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            delete result[0].otp;
                            callback(responseCode.SUCCESS, t('otp_resend'), result[0]);
                        }
                    });
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('user_not_found'), {})
                }
            });
        } else {
            callback(responseCode.OPERATION_FAILED, t('provide_any_one_details'), {})
        }
    },
    /**
     * API method for edit profile
     */
    editProfile: (req, callback) => {
        console.log("req", req)
        let update_data = {};
        if (req.type == 'personal_info') {
            if (req.first_name != undefined && req.first_name != '') {
                update_data.first_name = req.first_name;
            }
            if (req.last_name != undefined && req.last_name != '') {
                update_data.last_name = req.last_name;
            }
            if (req.nickname != undefined && req.nickname != '') {
                update_data.nickname = req.nickname;
            }
            if (req.profile_image != undefined && req.profile_image != '') {
                update_data.profile_image = req.profile_image;
            }
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_dob_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'dob') {
            if (req.dob != undefined && req.dob != '') {
                update_data.dob = req.dob;
            }
            // console.log(update_data)
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_dob_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'my_interest') {
            if (req.interests != undefined && req.interests.length > 0) {
                userInerestsModel.updateMany({ user_id: req.user_id }, { is_delete: '1' }, (err, result) => {
                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, null);
                    } else {
                        let insert_data = [];
                        req.interests.map((value) => {
                            insert_data.push({ user_id: req.user_id, interest_id: value });
                        });
                        userInerestsModel.create(insert_data, (err, result) => {
                            if (!err) {
                                common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                                    if (err) {
                                        callback(responseCode.OPERATION_FAILED, err, {});
                                    } else {
                                        callback(responseCode.SUCCESS, t('edit_interest_success'), user);
                                    }
                                })
                            } else {
                                console.log("Error while adding questions : ", err);
                                callback(responseCode.OPERATION_FAILED, err, {});
                            }
                        })
                    }
                })
            } else {
                callback(responseCode.OPERATION_FAILED, t('please_provide_interests'), {});
            }

        } else if (req.type == 'gender') {

            if (req.gender != undefined && req.gender != '') {
                update_data.gender = req.gender;
            }
            if (req.gender != undefined && req.gender != '' && req.gender == '2') {
                update_data.other_gender = req.other_gender;
            }
            if (req.gender != undefined && req.gender != '' && req.gender == '3') {
                update_data.add_other_gender = req.add_other_gender;
            }
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_gender_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'profile') {
            if (req.profile_type != undefined && req.profile_type != '') {
                update_data.profile_type = req.profile_type
            }
            if (req.secondary_email != undefined && req.secondary_email != '') {
                update_data.secondary_email = req.secondary_email
            }
            if (req.secondary_country_code != undefined && req.secondary_country_code != '') {
                update_data.secondary_country_code = req.secondary_country_code
            }
            if (req.secondary_phone != undefined && req.secondary_phone != '') {
                update_data.secondary_phone = req.secondary_phone
            }
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_profile_type_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'blood') {
            if (req.blood_type != undefined && req.blood_type != '') {
                update_data.blood_type = req.blood_type
            }
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_blood_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'personality') {
            if (req.personality_type != undefined && req.personality_type != '') {
                update_data.personality_type = req.personality_type
            }
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_profile_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'question') {
            if (req.questions != undefined && req.questions.length > 0) {
                userQuesAnsModel.updateMany({ user_id: req.user_id }, { is_delete: '1' }, (err, result) => {
                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, null);
                    } else {
                        let insert_data = [];
                        req.questions.map((value) => {
                            insert_data.push({ user_id: req.user_id, question_id: value.question, answer_id: value.answer });
                        });
                        userQuesAnsModel.create(insert_data, (err, result) => {
                            if (!err) {
                                common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                                    if (err) {
                                        callback(responseCode.OPERATION_FAILED, err, {});
                                    } else {
                                        callback(responseCode.SUCCESS, t('edit_question_success'), user);
                                    }
                                })
                            } else {
                                console.log("Error while adding questions : ", err);
                                callback(responseCode.OPERATION_FAILED, err, {});
                            }
                        })
                    }
                })
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {});
            }
        } else if (req.type == 'love_language') {
            // console.log("object", req)
            if (req.love_language != undefined && req.love_language.length > 0) {
                userModel.updateOne({ _id: req.user_id }, { love_language: req.love_language }, function (err, data) {

                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, {});
                    } else {
                        common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                            // console.log("err", err)
                            // console.log("data", data)
                            if (err) {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            } else {
                                callback(responseCode.SUCCESS, t('edit_love_language_success'), user);
                            }
                        })
                    }
                });
            }
            else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
            }
        }
    },
    /**
     * API method for contactUs
      */
    contactUs: (req, callback) => {
        let insert_data = {
            user_id: req.user_id,
            first_name: req.first_name,
            last_name: req.last_name,
            email: req.email,
            subject: req.subject,
            description: description
        }
        contactModel.create(insert_data, (err, result) => {
            if (!err) {
                callback(responseCode.SUCCESS, t('contactus_success'), {});
            } else {
                console.log("Error in Contact us : " + JSON.stringify(err));
                callback(responseCode.OPERATION_FAILED, err, null);
            }
        });
    },
    /**
     * API method for follow / unfollow
     */
    followUnfollow: (req, callback) => {
        const userID = mongoose.Types.ObjectId(req.user_id);
        const objectId = mongoose.Types.ObjectId(req.following_id);
        followersModel.find({ follower_id: userID, following_id: objectId }, (_err, _following_result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else if (_following_result.length > 0) {
                /* Do unfollow  operation */
                followersModel.deleteOne({ _id: _following_result[0]._id }, (_err, _result) => {
                    if (_err) {
                        callback(responseCode.OPERATION_FAILED, _err, {});
                    } else {
                        callback(responseCode.SUCCESS, t('unfollowed_success'), { isFollowed: false, followStatus: "" });
                    }
                })
            } else {
                /* Do follow  operation */
                let insData = {
                    follower_id: req.user_id,
                    following_id: req.following_id
                };
                followersModel.create(insData, (_err, _result) => {
                    if (_err) {
                        callback(responseCode.OPERATION_FAILED, _err, {});
                    } else {
                        userModel.findOne({ _id: _result.following_id }, {
                            username: 1,
                            user_id: '$_id',
                            first_name: 1,
                            last_name: 1,
                            profile_image:
                            {
                                $concat: [constant.s3Bucket_profile, {
                                    $ifNull: ['$profile_image', 'default-avatar.png']
                                }]
                            },
                            email: 1,
                            profile_type: 1
                        }, async (err, user) => {
                            if (err) {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            } else {
                                // console.log("user", user)
                                let resposne_data = {
                                    username: user.username,
                                    user_id: user.user_id,
                                    first_name: user.first_name,
                                    last_name: user.last_name,
                                    profile_image: user.profile_image,
                                    email: user.email,
                                    profile_type: user.profile_type,
                                }
                                if (user.profile_type == 'Private') {
                                    resposne_data.isFollowed = true;
                                    resposne_data.followStatus = 'Pending';
                                    let sendNotification = await user_model.sendNotification({ user_id: req.user_id, receiver_id: _result.following_id, follow_id: _result._id, tag: 'follow_request' })
                                    callback(responseCode.SUCCESS, t('followed_request_success'), resposne_data);
                                } else {
                                    /* if account is Public */
                                    let updateStatus = await followersModel.findOneAndUpdate({ _id: _result._id }, {
                                        status: 'Accept'
                                    });
                                    resposne_data.isFollowed = true;
                                    resposne_data.followStatus = 'Accept';
                                    let sendNotification = await user_model.sendNotification({ user_id: req.user_id, receiver_id: _result.following_id, follow_id: _result._id, tag: 'follower' })
                                    callback(responseCode.SUCCESS, t('followed_success'), resposne_data);
                                }
                            }
                        })
                    }
                })
            }
        })
    },
    /**
     * Api method used for accept follow request
      */
    acceptFollowRequest: async (req, callback) => {
        try {
            console.log("acceptFollowRequest", req)
            let followData = await followersModel.find({ _id: req.follow_id });
            if (followData.length > 0) {
                console.log("followData", followData[0])
                let updateStatus = await followersModel.findOneAndUpdate({ _id: req.follow_id }, {
                    status: 'Accept'
                });
                let sendNotification = await user_model.sendNotification({ user_id: req.user_id, receiver_id: followData[0].follower_id, follow_id: req.follow_id, tag: 'accept_follow_request' })

                callback(responseCode.SUCCESS, t('follow_request_accepted'), {})
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
            }
        } catch (error) {
            console.log("error1062", error)
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
        }
    },
    /**
     * Api method used for reject follow request
      */
    rejectFollowRequest: async (req, callback) => {
        try {
            let followData = await followersModel.find({ _id: req.follow_id });
            if (followData.length > 0) {
                let updateStatus = await followersModel.deleteOne({ _id: req.follow_id });
                callback(responseCode.SUCCESS, t('follow_request_rejected'), {})
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);

            }
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), null);
        }
    },
    /**
    * API method for follower list 
    */
    follower_request_list: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        let search = (req.search != undefined && req.search != '') ? req.search : '';

        // console.log("req.user_id", req.user_id)
        try {
            followersModel.aggregate([
                {
                    $match: {
                        following_id: req.user_id,
                        is_delete: '0',
                        status: 'Pending'
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'follower_id',
                        foreignField: '_id',
                        as: 'followUser'
                    }
                },
                {
                    $unwind: '$followUser' // Deconstructs the array of joined documents into separate documents
                },
                {
                    $match: {
                        $or: [
                            { 'followUser.username': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'followUser.first_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'followUser.last_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                        ]
                    }
                },
                {
                    $project: {
                        _id: 1,
                        follower_id: 1,
                        followStatus: '$status',
                        first_name: '$followUser.first_name',
                        last_name: '$followUser.last_name',
                        email: '$followUser.email',
                        nickname: '$followUser.nickname',
                        username: '$followUser.username',
                        profile_type: '$followUser.profile_type',
                        profile_image: { $concat: [constant.s3Bucket_profile, "$followUser.profile_image"] }
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    follower_list: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        let search = (req.search != undefined && req.search != '') ? req.search : '';

        // console.log("req.user_id", req.user_id)
        try {
            followersModel.aggregate([
                {
                    $match: {
                        following_id: req.user_id,
                        is_delete: '0',
                        status: 'Accept'
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'follower_id',
                        foreignField: '_id',
                        as: 'followUser'
                    }
                },
                {
                    $unwind: '$followUser' // Deconstructs the array of joined documents into separate documents
                },
                {
                    $match: {
                        $or: [
                            { 'followUser.username': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'followUser.first_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'followUser.last_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                        ]
                    }
                },
                {
                    $project: {
                        _id: 1,
                        follower_id: 1,
                        user_id: '$followUser._id',
                        followStatus: '$status',
                        first_name: '$followUser.first_name',
                        last_name: '$followUser.last_name',
                        email: '$followUser.email',
                        nickname: '$followUser.nickname',
                        username: '$followUser.username',
                        profile_type: '$followUser.profile_type',
                        profile_image: { $concat: [constant.s3Bucket_profile, "$followUser.profile_image"] }
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    /**
    * API method for following list 
    */
    following_list: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        let search = (req.search != undefined && req.search != '') ? req.search : '';
        try {
            followersModel.aggregate([
                {
                    $match: {
                        follower_id: req.user_id,
                        is_delete: '0',
                        status: 'Accept'
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'following_id',
                        foreignField: '_id',
                        as: 'followUser'
                    }
                },
                {
                    $unwind: '$followUser' // Deconstructs the array of joined documents into separate documents
                },

                {
                    $match: {
                        $or: [
                            { 'followUser.username': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'followUser.first_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'followUser.last_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                        ]
                    }
                },
                {
                    $project: {
                        _id: 1,
                        following_id: 1,
                        user_id: '$followUser._id',
                        followStatus: '$status',
                        first_name: '$followUser.first_name',
                        last_name: '$followUser.last_name',
                        email: '$followUser.email',
                        nickname: '$followUser.nickname',
                        username: '$followUser.username',
                        profile_type: '$followUser.profile_type',
                        profile_image: { $concat: [constant.s3Bucket_profile, "$followUser.profile_image"] }
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    /**
   * API method for get all users  
   */
    getUsers: (req, callback) => {
        try {
            userModel.aggregate([
                // {
                //     $lookup: {
                //         from: 'followers',
                //         localField: '_id',
                //         foreignField: 'following_id',
                //         as: 'followers'
                //     }
                // },
                {
                    $lookup: {
                        from: 'followers',
                        let: { userID: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$following_id', '$$userID'] }, //
                                            { $eq: ['$status', 'Accept'] } // Match saved 
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followers'
                    }
                },
                {
                    $match: {
                        _id: { $ne: req.user_id },
                        followers: { $size: 0 } // Filter users with no followers
                    }
                }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    getUsersForFamily: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            let search = '';
            if (req.search != undefined && req.search != '') {
                search = req.search
            }
            userModel.aggregate([
                {
                    $addFields: {
                        fullName: { $concat: ['$first_name', ' ', '$last_name'] }
                    }
                },
                {
                    $lookup: {
                        from: 'families',
                        let: { userID: '$_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$family_member_id', '$$userID'] }, //
                                            { $eq: ['$user_id', req.user_id] } // Match saved 
                                        ]
                                    }
                                }
                            },
                            {
                                $project: { _id: 1 } // Include only _id field to check if there's a match
                            }
                        ],
                        as: 'family'
                    }
                },
                {
                    $match: {
                        _id: { $ne: req.user_id },
                        $or: [
                            { 'username': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'first_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'last_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'fullName': { $regex: '.*' + search + '.*', $options: 'i' } },
                        ],
                        family: { $size: 0 }

                    }
                },
                {
                    $project: {
                        username: 1,
                        user_id: '$_id',
                        first_name: 1,
                        last_name: 1,
                        profile_image:
                        {
                            $cond: {
                                if: { $ne: ['$profile_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_profile, '$profile_image']
                                }, else: ''
                            }
                        },
                        email: 1
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },

    /**
    * API method for add family members 
    */
    addFamily: (req, callback) => {
        let family_data = req.family_data;
        if (family_data.length > 0) {
            let addData = [];
            family_data.map((item) => {
                addData.push({ user_id: req.user_id, family_member_id: item.family_member_id, relationship_id: item.relationship_id });
            })
            familyModel.create(addData, (_err, _result) => {
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, {});
                } else {
                    callback(responseCode.SUCCESS, t('family_member_added'), {});
                }
            })
        } else {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {});
        }
    },
    /**
   * API method for get family members.
   */
    getFamily: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            let search = (req.search != undefined && req.search != '') ? req.search : '';
            familyModel.aggregate([
                {
                    $match: {
                        user_id: req.user_id,
                        is_delete: '0',

                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'family_member_id',
                        foreignField: '_id',
                        as: 'User'
                    }
                },
                {
                    $unwind: '$User' // Deconstructs the array of joined documents into separate documents
                },
                {
                    $addFields: {
                        fullName: { $concat: ['$User.first_name', ' ', '$User.last_name'] }
                    }
                },
                {
                    $lookup: {
                        from: 'relationships',
                        localField: 'relationship_id',
                        foreignField: '_id',
                        as: 'relationship'
                    }
                },
                {
                    $unwind: '$relationship' // Deconstructs the array of joined documents into separate documents
                },

                {
                    $match: {
                        $or: [
                            { 'User.username': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'User.first_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'User.last_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'fullName': { $regex: '.*' + search + '.*', $options: 'i' } },
                        ]
                    }
                },
                {
                    $project: {
                        family_id: '$_id',
                        relationship_name: '$relationship.name',
                        relationship_id: '$relationship._id',
                        first_name: '$User.first_name',
                        last_name: '$User.last_name',
                        email: '$User.email',
                        nickname: '$User.nickname',
                        username: '$User.username',
                        profile_type: '$User.profile_type',
                        profile_image: { $concat: [constant.s3Bucket_profile, "$User.profile_image"] }
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    /**
 * API method for remove family members 
 */
    deleteFamily: (req, callback) => {
        familyModel.updateOne({ _id: req.family_id }, { is_delete: 1 }, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('family_member_deleted'), {});
            }
        })
    },
    /**
     * This function is used to block user
      */
    blockUser: (req, callback) => {
        const objectId = mongoose.Types.ObjectId(req.block_user_id);
        blockedUserModel.find({
            user_id: req.user_id,
            block_user_id: req.block_user_id
        }, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else if (_result.length > 0) {
                callback(responseCode.SUCCESS, t('user_already_blocked'), {});
            } else {
                blockedUserModel.create({
                    user_id: req.user_id,
                    block_user_id: req.block_user_id
                }, async (_err, result) => {
                    if (_err) {
                        callback(responseCode.OPERATION_FAILED, _err, {});
                    } else {
                        let del = await followersModel.deleteOne({ follower_id: req.user_id, following_id: objectId });
                        let del2 = await followersModel.deleteOne({ follower_id: objectId, following_id: req.user_id });
                        callback(responseCode.SUCCESS, t('user_blocked'), {});
                    }
                })
            }
        })

    },
    /**
     * This function is used to un-block user
      */
    unblockUser: (req, callback) => {
        blockedUserModel.deleteOne({
            user_id: req.user_id,
            block_user_id: req.block_user_id
        }, (_err, result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('user_unblocked'), {});
            }
        })
    },
    /*
    * API method for Friend List
    */
    friendList: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            let search = '';
            if (req.search != undefined && req.search != '') {
                search = req.search
            }
            followersModel.aggregate([
                {
                    $match: {
                        follower_id: req.user_id,
                        status: 'Accept'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$following_id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$following_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$follower_id', '$$authorID'] },
                                            { $eq: ['$status', 'Accept'] }
                                        ],
                                    },
                                },
                            },
                        ],
                        as: 'reverseFollow',
                    },
                },
                {
                    $lookup: {
                        from: "users",
                        localField: "following_id",
                        foreignField: "_id",
                        as: "followingUserInfo"
                    }
                },
                {
                    $unwind: "$followingUserInfo"
                },
                {
                    $addFields: {
                        fullName: { $concat: ['$followingUserInfo.first_name', ' ', '$followingUserInfo.last_name'] }
                    }
                },
                {
                    $match: {
                        'reverseFollow.status': 'Accept',
                        $or: [
                            { 'followingUserInfo.username': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'followingUserInfo.first_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'followingUserInfo.last_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'fullName': { $regex: '.*' + search + '.*', $options: 'i' } },
                        ],
                    }
                },
                {
                    $project: {
                        username: '$followingUserInfo.username',
                        user_id: '$followingUserInfo._id',
                        first_name: '$followingUserInfo.first_name',
                        last_name: '$followingUserInfo.last_name',
                        profile_image:
                        {
                            $cond: {
                                if: { $ne: ['$followingUserInfo.profile_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_profile, '$followingUserInfo.profile_image']
                                }, else: ''
                            }
                        },
                        email: '$followingUserInfo.email',
                    }
                },

                { $skip: start },
                { $limit: limit }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    notificationList: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            let notificationData = notificationModel.aggregate([
                {
                    $match: {
                        receiver_id: req.user_id
                    }
                },
                {
                    $lookup: {
                        from: "users",
                        localField: "sender_id",
                        foreignField: "_id",
                        as: "senderDetails"
                    }
                },
                {
                    $unwind: "$senderDetails"
                },
                {
                    $lookup: {
                        from: "users",
                        localField: "receiver_id",
                        foreignField: "_id",
                        as: "receiverDetails"
                    }
                },
                {
                    $unwind: "$receiverDetails"
                },
                {
                    $sort: {
                        createdAt: -1
                    }
                },
                {
                    $project: {
                        _id: 1,
                        sender_id: 1,
                        receiver_id: 1,
                        receiver_type: 1,
                        sender_type: 1,
                        notification_tag: 1,
                        image: 1,
                        title: 1,
                        message: 1,
                        follow_id: 1,
                        post_id: 1,
                        comment_id: 1,
                        like_id: 1,
                        reel_id: 1,
                        collaborate_id: 1,
                        goal_id: 1,
                        goal_request_id: 1,
                        is_read: 1,
                        createdAt: 1,
                        senderProfile: {
                            username: '$senderDetails.username',
                            user_id: '$senderDetails._id',
                            first_name: '$senderDetails.first_name',
                            last_name: '$senderDetails.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$senderDetails.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$senderDetails.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$senderDetails.email'
                        },
                        receiverProfile: {
                            username: '$receiverDetails.username',
                            user_id: '$receiverDetails._id',
                            first_name: '$receiverDetails.first_name',
                            last_name: '$receiverDetails.last_name',
                            profile_image:
                            {
                                $cond: {
                                    if: { $ne: ['$receiverDetails.profile_image', ''] }, then: {
                                        $concat: [constant.s3Bucket_profile, '$receiverDetails.profile_image']
                                    }, else: ''
                                }
                            },
                            email: '$receiverDetails.email'
                        },
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            console.log("error in notification list => ", error)
            callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
        }
    },


    /*
    * API method for create group
    */
    blockUserList: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            let search = '';
            if (req.search != undefined && req.search != '') {
                search = req.search
            }
            blockedUserModel.aggregate([
                {
                    $lookup: {
                        from: "users",
                        localField: "block_user_id",
                        foreignField: "_id",
                        as: "userInfo"
                    }

                },
                {
                    $unwind: "$userInfo"
                },
                {
                    $addFields: {
                        fullName: { $concat: ['$userInfo.first_name', ' ', '$userInfo.last_name'] }
                    }
                },
                {
                    $match: {
                        user_id: req.user_id,
                        $or: [
                            { 'userInfo.username': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'userInfo.first_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'userInfo.last_name': { $regex: '.*' + search + '.*', $options: 'i' } },
                            { 'fullName': { $regex: '.*' + search + '.*', $options: 'i' } },
                        ],
                    }
                },
                {
                    $project: {
                        username: '$userInfo.username',
                        user_id: '$block_user_id',
                        first_name: '$userInfo.first_name',
                        last_name: '$userInfo.last_name',
                        profile_image:
                        {
                            $cond: {
                                if: { $ne: ['$userInfo.profile_image', ''] }, then: {
                                    $concat: [constant.s3Bucket_profile, '$userInfo.profile_image']
                                }, else: ''
                            }
                        },
                        email: '$userInfo.email',
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    /*
    * API method for create group
    */
    addGroup: (req, callback) => {
        let sql = `insert into tbl_chat_room set ?`;
        let data = {
            type: 'Group',
            isGroupChat: 1,
            name: req.name,
            image: req.image,
            created_by: req.user_id,
            user1: req.user_id,
        };
        con.query(sql, (err, result) => {
            if (!err) {
                //add member to chat room
                let arr = [];
                arr.push(req.user_id);
                user_model.addMemberToChatRoom({
                    users: arr,
                    group_id: result.insertId,
                    is_admin: 1,
                }, (code, msg, data) => {
                    if (code == 1) {
                        callback(responseCode.CREATED, t('group_create_successfully'), {});
                    } else {
                        callback(code, msg, {});
                    }
                });
            } else {
                callback(responseCode.OPERATION_FAILED, err, {});
            }
        });
    },
    addMemberToChatRoom: (req, callback) => {
        let insertData = []
        asyncLoop(req.users, (item, next) => {
            insertData.push([req.group_id, item.user_id, (req.is_admin != undefined ? 1 : 0)]);
        }, () => {
            con.query(`INSERT INTO tbl_chat_users set (chat_room_id,user_id,is_admin) VALUES ?`, [insertData], (_err, _result) => {
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, {});
                } else {
                    callback(responseCode.SUCCESS, t('member_added'), {});
                }
            })
        })
    },
    removeMemberFromGroup: (req, callback) => {
        let sql = `DELETE FROM tbl_chat_users WHERE chat_room_id =? AND user_id =?`;
        con.query(sql, [req.group_id, req.user_id], (_err, res) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('member_deleted'), {});
            }
        })
    },
};

module.exports = user_model