const responseCode = require('../../../config/response-error-code');
const con = require('../../../config/database');
const common = require('../../../utilities/common');
const validate_header = require('../../../middleware/validate-header');
const constant = require('../../../config/constant');
const moment = require('moment');
const asyncLoop = require('node-async-loop')
const user_model = require('../user/user_model');
const folderModel = require('../../../model/folders')
const postsModel = require('../../../model/posts')
const postsHashtag = require('../../../model/post_hashtags')
const postLike = require('../../../model/post_like')
const commentLikeModel = require('../../../model/comment_like')
const postComments = require('../../../model/post_comments')
const postTagUsers = require('../../../model/post_tag_users')
const postCategory = require('../../../model/post_category')
const PostViewer = require('../../../model/post_viewers')
const collectionModel = require('../../../model/collections')
const collectionPostModel = require('../../../model/collection_posts')
const mongoose = require('mongoose');
const { t } = require('localizify');
let social_model = {
    /**
     * @description API method for folder listing
     */
    folderList: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        folderModel.find({ user_id: req.user_id, is_active: "1", is_delete: "0" }).skip(start)
            .limit(limit)
            .exec((_err, _result) => {
                // Your code here
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, []);
                } else {
                    if (_result.length > 0) {
                        callback(responseCode.SUCCESS, t('data_found'), _result);
                    } else {
                        callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                    }
                }
            })
    },
    /**
     * @description API method for creating folder
     */
    addFolder: (req, callback) => {
        let addData = {
            name: req.name,
            user_id: req.user_id
        }
        let sql = `INSERT INTO tbl_folders SET ?`;
        folderModel.create(addData, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('folder_added'), {});
            }
        })
    },
    /**
     * @description API method for edit folder
     */
    editFolder: (req, callback) => {
        let addData = {
            name: req.name
        }
        let folderID = mongoose.Types.ObjectId(req.folder_id);
        folderModel.updateOne({ _id: folderID }, addData, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('folder_edited'), {});
            }
        })
    },
    /**
     * @description API method for delete folder
     */
    deleteFolder: (req, callback) => {
        let folderID = mongoose.Types.ObjectId(req.folder_id);
        folderModel.updateOne({ _id: folderID }, { is_delete: "1" }, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('folder_deleted'), {});
            }
        })
    },
    addPost: async (req, callback) => {
        let data = {
            user_id: req.user_id,
            type: req.type
        };
        if (req.privacy != undefined && req.privacy != '') {
            data.privacy = req.privacy;
        }
        if (req.description != undefined && req.description != '') {
            data.description = req.description;
        }
        if (req.music != undefined && req.music != '') {
            data.music = req.music;
        }
        // if (req.folder_id != undefined && req.folder_id != '') {
        //     data.folder_id = req.folder_id;
        // }
        if (req.location_latitude != undefined && req.location_latitude != '') {
            data.location_latitude = req.location_latitude;
            data.location_longitude = req.location_longitude;
            data.location_full_address = req.location_full_address;
            data.location = req.location;
        }
        postsModel.create(data, async (err, result) => {
            if (!err) {
                if (req.type != 'text') {
                    //add media to post
                    if (Object.keys(req.media).length > 0) {
                        social_model.addMediaToPost({ post_id: result._id, user_id: req.user_id, media: req.media }, (err, mediaData) => {
                            if (err) {
                                console.log("Error : ", err);
                                callback(responseCode.OPERATION_FAILED, t('somthing_went_wrong'), null)
                            } else {
                                if (req.hashtag_ids.length > 0) {
                                    /* add hashtag to text post */
                                    let hashtag_data = {
                                        post_id: result._id,
                                        user_id: req.user_id,
                                        type: 'Post',
                                        hashtag_ids: req.hashtag_ids
                                    }
                                    social_model.addHashTags(hashtag_data, (err, hashtagData) => {
                                        if (err) {
                                            console.log("addHashTags Error : ", err);
                                            callback(responseCode.OPERATION_FAILED, t('somthing_went_wrong'), null)
                                        } else {
                                            callback(responseCode.SUCCESS, t('post_added'), null);
                                        }
                                    });
                                } else {
                                    callback(responseCode.SUCCESS, t('post_added'), null);
                                }
                            }
                        })
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('somthing_went_wrong'), null)
                    }
                } else {
                    try {
                        let folder_data = {
                            user_id: req.user_id,
                            post_id: result._id,
                        }
                        /* add post in default folder  */
                        let addPostFolder = await addPostInDefaultFolder(folder_data);
                        if (req.folder_id != undefined && req.folder_id != '') {
                            folder_data.folder_id = req.folder_id;
                            let addPostAnotherFolder = await addPostInFolder(folder_data);

                        }
                        /* add hashtag to text post */
                        if (req.hashtag_ids != undefined && req.hashtag_ids.length > 0) {
                            let hashtag_data = {
                                post_id: result._id,
                                user_id: req.user_id,
                                type: 'Post',
                                hashtag_ids: req.hashtag_ids
                            }
                            let add_hashtag = await social_model.addHashTags(hashtag_data);
                        }
                        /* add post tag users to text post */
                        if (req.post_tag_users != undefined && req.post_tag_users.length > 0) {
                            let posttag_user_data = {
                                post_id: result._id,
                                user_id: req.user_id,
                                type: 'Post',
                                post_tag_users: req.post_tag_users
                            }
                            let add_posttag_user = await social_model.addPostTagUser(posttag_user_data);

                        }
                        /* add post categories to text post */
                        if (req.post_category_id != undefined && req.post_category_id.length > 0) {
                            let post_category_data = {
                                post_id: result._id,
                                user_id: req.user_id,
                                type: 'Post',
                                post_category_id: req.post_category_id
                            }
                            let add_posttag_user = await social_model.addPostCategory(post_category_data);

                        }
                        callback(responseCode.SUCCESS, t('post_added'), null);
                    } catch (error) {
                        console.log("addHashTags Error : ", error);
                        callback(responseCode.OPERATION_FAILED, t('somthing_went_wrong'), null)
                    }
                }
            } else {
                // console.log(err)
                callback(responseCode.OPERATION_FAILED, t('somthing_went_wrong'), null)
            }
        })
    },
    addMediaToPost: (req, callback) => {
        let sql = `update tbl_post_media set ? where post_id = ? and user_id=?`;
        con.query(sql, [{ is_delete: 1 }, req.post_id, req.user_id], (err, result) => {
            if (!err) {
                let newData = [];
                for (let i in req.media) {
                    newData.push([req.user_id, req.post_id, req.media[i]]);
                }
                con.query(`INSERT INTO tbl_post_media (user_id, post_id, media) VALUES ?`, [newData], (err, _post_media_result) => {
                    if (!err) {
                        callback(undefined, _post_media_result);
                    } else {
                        callback(err, undefined);
                    }
                })
            } else {
                callback(err, undefined);
            }
        })
    },
    addPostInFolder: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                let folder_post = await folderPostModel.create({
                    user_id: req.user_id,
                    folder_id: req.folder_id,
                    post_id: req.post_id
                });
                resolve(folder_post);
            } catch (error) {
                reject(error);
            }
        });
    },
    addPostInDefaultFolder: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                folderModel.find({ name: '' + moment().format('YYYY') + '', user_id: req.user_id }, async (err, _folder) => {
                    if (err) {
                        reject(err);
                    } else if (_folder.length > 0) {
                        let folder_post = await folderPostModel.create({
                            user_id: req.user_id,
                            folder_id: _folder[0]._id,
                            post_id: req.post_id
                        });
                        resolve(folder_post);
                    } else {
                        let folder_data = await folderModel.create({
                            user_id: req.user_id,
                            name: "" + moment().format('YYYY')
                        });
                        let folder_post = await folderPostModel.create({
                            user_id: req.user_id,
                            folder_id: folder_data._id,
                            post_id: req.post_id
                        });
                        resolve(folder_post);
                    }
                })
            } catch (error) {
                reject(error);
            }
        });
    },
    addHashTags: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                postsHashtag.updateOne({ post_id: req.post_id, user_id: req.user_id }, { is_delete: 1 }, (err, result) => {
                    if (!err) {
                        let addData = [];
                        req.hashtag_ids.map((item) => {
                            addData.push({ user_id: req.user_id, post_id: req.post_id, type: req.type, hashtag_id: item });
                        })
                        postsHashtag.create(addData, (err, _post_hashtags_result) => {
                            if (!err) {
                                resolve(_post_hashtags_result);
                            } else {
                                reject(err);
                            }
                        })
                    } else {
                        reject(err);
                    }
                })
            } catch (e) {
                reject("data_not_found");
            }
        });

    },
    // addHashTags: (req, callback) => {
    //     postsHashtag.updateOne({ post_id: req.post_id, user_id: req.user_id }, { is_delete: 1 }, (err, result) => {
    //         if (!err) {
    //             let addData = [];
    //             req.hashtag_ids.map((item) => {
    //                 addData.push({ user_id: req.user_id, post_id: req.post_id, type: req.type, hashtag_id: item });
    //             })
    //             console.log("addData", addData)
    //             postsHashtag.create(addData, (err, _post_hashtags_result) => {
    //                 if (!err) {
    //                     callback(undefined, _post_hashtags_result);
    //                 } else {
    //                     callback(err, undefined);
    //                 }
    //             })
    //         } else {
    //             callback(err, undefined);
    //         }
    //     })
    // },
    // addPostTagUser: async (req, callback) => {
    //     postTagUsers.deleteOne({ post_id: req.post_id }, (err, result) => {
    //         if (!err) {
    //             let addData = [];
    //             req.post_tag_users.map((item) => {
    //                 addData.push({ user_id: item, post_id: req.post_id });
    //             })
    //             postTagUsers.create(addData, (err, _post_hashtags_result) => {
    //                 if (!err) {
    //                     callback(undefined, _post_hashtags_result);
    //                 } else {
    //                     callback(err, undefined);
    //                 }
    //             })
    //         } else {
    //             callback(err, undefined);
    //         }
    //     })
    // },
    addPostTagUser: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                postTagUsers.deleteMany({ post_id: req.post_id, user_id: req.user_id }, (err, result) => {
                    if (!err) {
                        let addData = [];
                        req.post_tag_users.map((item) => {
                            addData.push({ user_id: req.user_id, post_id: req.post_id, tag_user_id: item });
                        })
                        postTagUsers.create(addData, (err, _post_hashtags_result) => {
                            if (!err) {
                                resolve(_post_hashtags_result);
                            } else {
                                reject(err);
                            }
                        })
                    } else {
                        reject(err);
                    }
                })
            } catch (e) {
                reject("data_not_found");
            }
        });
    },
    addPostCategory: async (req) => {
        return new Promise(async (resolve, reject) => {
            try {
                postCategory.deleteMany({ post_id: req.post_id, user_id: req.user_id }, (err, result) => {
                    if (!err) {
                        let addData = [];
                        req.post_category_id.map((item) => {
                            addData.push({ user_id: req.user_id, post_id: req.post_id, category_id: item });
                        })
                        postCategory.create(addData, (err, _post_category_result) => {
                            if (!err) {
                                resolve(_post_category_result);
                            } else {
                                reject(err);
                            }
                        })
                    } else {
                        reject(err);
                    }
                })
            } catch (e) {
                reject("data_not_found");
            }
        });
    },

    /**
   * @Descriprion API method for create story 
   */
    getPost: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            postsModel.aggregate([
                {
                    $match: {
                        is_active: "1",
                        is_delete: "0",
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'

                    }
                },
                {
                    "$unwind": "$profile"
                },
                {
                    $lookup: {
                        from: 'post_hatshtags',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postHashtags'
                    }
                },

                {
                    $lookup: {
                        from: 'hashtags',
                        localField: 'postHashtags.hashtag_id',
                        foreignField: '_id',
                        as: 'hashtags'
                    }
                },
                {
                    $lookup: {
                        from: 'post_likes',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $lookup: {
                        from: 'post_comments',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postComments'
                    }
                },
                {
                    $lookup: {
                        from: 'post_viewers',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postViewers'
                    }
                },
                {
                    $lookup: {
                        from: 'collection_posts',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'collectionPost'
                    }
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followInfo'
                    }
                },
                {
                    $addFields: {
                        totalLikes: { $size: '$likes' },
                        totalComments: { $size: '$postComments' },
                        totalViewers: { $size: '$postViewers' },
                        totalSave: { $size: '$collectionPost' },
                        userLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$likes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } }
                        // Add more fields for other emoji types as needed
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        _id: 1,
                        user_id: 1,
                        type: 1,
                        // message: 1,
                        location: 1,
                        description: 1,
                        music: 1,
                        privacy: 1,
                        status: 1,
                        is_active: 1,
                        is_delete: 1,
                        viewCount: 1,
                        createdAt: 1,
                        location_latitude: 1,
                        location_longitude: 1,
                        location_full_address: 1,
                        profile: {
                            username: '$profile.username',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $concat: [constant.s3Bucket_profile, {
                                    $ifNull: ['$profile.profile_image', 'default-avatar.png']
                                }]
                            },
                            email: '$profile.email'
                        },
                        hashtags: {
                            $map: {
                                input: '$hashtags',
                                as: 'hash_tag',
                                in: {
                                    _id: '$$hash_tag._id', // Include the _id field from the joined 'posts' collection
                                    hashtag: '$$hash_tag.hashtag' // Include the 'name' field from the joined 'posts' collection
                                }
                            }
                        },
                        totalLikes: 1,
                        userLiked: 1,
                        totalComments: 1,
                        totalViewers: 1,
                        totalSave: 1,
                        userLikeEmoji: {
                            $arrayElemAt: [{
                                $map: {
                                    input: '$reaction',
                                    as: 'reactions',
                                    in: {
                                        _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                        image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                    }
                                }
                            },
                                0]
                        },
                        isFollowed: 1
                        // Project other fields from the Post model as needed
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_posts => {
                if (_posts.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _posts);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            })
        } catch (error) {
            console.log(error)
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    postDetail: async (req, callback) => {
        try {
            let viewer = await social_model.postView(req);
            const objectId = mongoose.Types.ObjectId(req.post_id);
            postsModel.aggregate([
                {
                    $match: {
                        _id: objectId,
                        is_active: "1",
                        is_delete: "0",
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'

                    }
                },

                {
                    $lookup: {
                        from: 'post_hatshtags',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postHashtags'
                    }
                },

                {
                    $lookup: {
                        from: 'hashtags',
                        localField: 'postHashtags.hashtag_id',
                        foreignField: '_id',
                        as: 'hashtags'
                    }
                },
                {
                    $lookup: {
                        from: 'post_likes',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $lookup: {
                        from: 'post_comments',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postComments'
                    }
                },
                {
                    $lookup: {
                        from: 'post_viewers',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'postViewers'
                    }
                },
                {
                    $lookup: {
                        from: 'collection_posts',
                        localField: '_id',
                        foreignField: 'post_id',
                        as: 'collectionPost'
                    }
                },
                {
                    "$unwind": "$profile"
                },
                {
                    $lookup: {
                        from: 'followers',
                        let: { authorID: '$profile._id' },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                            { $eq: ['$following_id', '$$authorID'] }
                                        ]
                                    }
                                }
                            }
                        ],
                        as: 'followInfo'
                    }
                },
                {
                    $addFields: {
                        totalLikes: { $size: '$likes' },
                        totalComments: { $size: '$postComments' },
                        totalViewers: { $size: '$postViewers' },
                        totalSave: { $size: '$collectionPost' },
                        userLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$likes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        userLikeEmoji: {
                            $cond: {
                                if: { $in: [req.user_id, '$likes.user_id'] },
                                then: { $concat: [constant.s3Bucket_emoji, 'reaction.image'] },
                                else: ''
                            }
                        },
                        isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } }

                        // Add more fields for other emoji types as needed
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        _id: 1,
                        user_id: 1,
                        type: 1,
                        // message: 1,
                        location: 1,
                        description: 1,
                        music: 1,
                        privacy: 1,
                        status: 1,
                        is_active: 1,
                        is_delete: 1,
                        createdAt: 1,
                        viewCount: 1,
                        location_latitude: 1,
                        location_longitude: 1,
                        location_full_address: 1,
                        profile: {
                            username: '$profile.username',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $concat: [constant.s3Bucket_profile, {
                                    $ifNull: ['$profile.profile_image', 'default-avatar.png']
                                }]
                            },
                            email: '$profile.email'
                        },
                        hashtags: {
                            $map: {
                                input: '$hashtags',
                                as: 'hash_tag',
                                in: {
                                    _id: '$$hash_tag._id', // Include the _id field from the joined 'posts' collection
                                    hashtag: '$$hash_tag.hashtag' // Include the 'name' field from the joined 'posts' collection
                                }
                            }
                        },
                        totalLikes: 1,
                        userLiked: 1,
                        totalViewers: 1,
                        totalSave: 1,
                        totalComments: 1,
                        userLikeEmoji: {
                            $arrayElemAt: [{
                                $map: {
                                    input: '$reaction',
                                    as: 'reactions',
                                    in: {
                                        _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                        image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                    }
                                }
                            },
                                0]
                        },
                        isFollowed: 1
                        // Project other fields from the Post model as needed
                    }
                },

            ]).then(_posts => {
                // console.log("_posts", _posts)
                if (_posts.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _posts[0]);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, []);
        }
    },
    /**
     * @description This function will be used to get all recent locations
     * 
    */
    recentLocations: (req, callback) => {
        try {
            postsModel.find({ user_id: req.user_id, is_delete: "0" }, {
                _id: 0, location: 1, location_full_address: 1, location_latitude: 1, location_longitude: 1
            }).sort({ createdAt: -1 }).skip(0)
                .limit(10).exec((err, results) => {
                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, [])
                    } else if (results.length > 0) {
                        callback(responseCode.SUCCESS, t('data_found'), results);
                    } else {
                        callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                    }
                })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, [])

        }
    },
    /**
    * API method for post view 
    */
    postView: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                let post_viewer = await PostViewer.find({ viewer_id: req.user_id, post_id: req.post_id });
                if (post_viewer.length == 0) {
                    let post_viewer_data = {
                        viewer_id: req.user_id,
                        post_id: req.post_id
                    }
                    let inserted_data = await PostViewer.create(post_viewer_data);
                    let post_data = await postsModel.findByIdAndUpdate(req.post_id, {
                        $inc: { viewCount: 1 }, // Increment the view count by 1
                    }, { new: true });
                    resolve(post_data)
                } else {
                    resolve([])

                }
            } catch (error) {
                reject(error)
            }
        })
    },
    getPostViewer: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        const objectId = mongoose.Types.ObjectId(req.post_id);
        PostViewer.aggregate([
            {
                $match: {
                    post_id: objectId
                }
            },
            {
                $lookup: {
                    from: 'post_likes',
                    let: { post_id: '$post_id', user_id: '$viewer_id' },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $or: [
                                        { $eq: ['$post_id', '$$post_id'] },
                                        { $eq: ['$user_id', '$$user_id'] }
                                    ]
                                }
                            }
                        }
                    ],
                    as: 'likes'
                }
            },
            {
                $lookup: {
                    from: 'reactions',
                    localField: 'likes.like_id',
                    foreignField: '_id',
                    as: 'reaction'
                }
            },
            {
                $lookup: {
                    from: 'post_comments',
                    localField: '_id',
                    foreignField: 'post_id',
                    as: 'postComments'
                }
            },
            {
                $lookup: {
                    from: 'users',
                    localField: 'viewer_id',
                    foreignField: '_id',
                    as: 'profile'

                }
            },
            {
                $unwind: '$profile'
            },
            {
                $lookup: {
                    from: 'followers',
                    let: { authorID: '$profile._id' },
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $and: [
                                        { $eq: ['$follower_id', req.user_id] }, // Check if the current user follows the author
                                        { $eq: ['$following_id', '$$authorID'] }
                                    ]
                                }
                            }
                        }
                    ],
                    as: 'followInfo'
                }
            },
            {
                $addFields: {
                    userLiked: {
                        $cond: {
                            if: { $in: ['$profile._id', '$likes.user_id'] },
                            then: true,
                            else: false
                        }
                    },
                    userLikeEmoji: {
                        $cond: {
                            if: { $in: ['$profile._id', '$likes.user_id'] },
                            then: { $concat: [constant.s3Bucket_emoji, 'reaction.image'] },
                            else: []
                        }
                    },
                    isFollowed: { $cond: { if: { $gt: [{ $size: '$followInfo' }, 0] }, then: true, else: false } }
                    // Add more fields for other emoji types as needed
                }
            },
            {
                $sort: { createdAt: -1 }
            },
            {
                $project: {
                    _id: 1,
                    post_id: 1,
                    profile: {
                        user_id: '$profile._id',
                        username: '$profile.username',
                        first_name: '$profile.first_name',
                        last_name: '$profile.last_name',
                        profile_image:
                        {
                            $concat: [constant.s3Bucket_profile, {
                                $ifNull: ['$profile.profile_image', 'default-avatar.png']
                            }]
                        },
                        email: '$profile.email'
                    },
                    userLikeEmoji: {
                        $arrayElemAt: [{
                            $map: {
                                input: '$reaction',
                                as: 'reactions',
                                in: {
                                    _id: '$$reactions._id', // Include the _id field from the joined 'posts' collection
                                    image: { $concat: [constant.s3Bucket_emoji, '$$reactions.image'] } // Include the 'name' field from the joined 'posts' collection
                                }
                            }
                        }, 0]
                    },
                    isFollowed: 1,
                    userLiked: 1

                }
            },
            { $skip: start },
            { $limit: limit }
        ], (err, post_viewer) => {
            if (err) {
                callback(responseCode.OPERATION_FAILED, err, [])
            } else {
                if (post_viewer.length > 0) {
                    PostViewer.countDocuments({ post_id: objectId }, (err, count) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, [])
                        } else {
                            callback(responseCode.SUCCESS, t('data_found'), { totalViewers: count, viewersData: post_viewer })
                        }
                    });
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), { totalViewers: 0, viewersData: [] })
                }
            }
        })
    },
    /**
    * API method for like/unlike Post 
    */
    likePost: (req, callback) => {
        try {
            postLike.find({ post_id: req.post_id, user_id: req.user_id }, (err, result) => {
                console.log(result)
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    postLike.updateOne({ _id: result[0]._id }, { like_id: req.like_id }, (err, result) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('post_liked'), {})
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    postLike.create({ user_id: req.user_id, post_id: req.post_id, like_id: req.like_id }, (err, result) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('post_liked'), {})
                        }
                    })
                }
            })
        } catch (e) {
            console.log("Error in likeUnlikePost : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    unlikePost: (req, callback) => {
        try {
            postLike.find({ post_id: req.post_id, user_id: req.user_id }, (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    postLike.deleteOne({ _id: result[0]._id }, (err, result) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('post_unliked'), {})
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('something_went_wrong'), {});
                }
            })
        } catch (e) {
            console.log("Error in UnlikePost : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    addComment: (req, callback) => {
        try {
            let comment_data = {
                user_id: req.user_id,
                post_id: req.post_id,
                message: req.message
            }
            if (req.parentComment != undefined && req.parentComment != '') {
                comment_data.parentComment = req.parentComment;
            }
            postComments.create(comment_data, (err, result) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    callback(responseCode.SUCCESS, t('comment_created'), {});
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    getComment: (req, callback) => {
        try {
            let page = (req.page != undefined && req.page > 0) ? req.page : 1;
            let limit = constant.itemPerPage;
            let start = ((page - 1) * constant.itemPerPage);
            const objectId = mongoose.Types.ObjectId(req.post_id);
            postComments.aggregate([
                { $match: { post_id: objectId, parentComment: null } },
                {
                    $lookup: {
                        from: 'users',
                        localField: 'user_id',
                        foreignField: '_id',
                        as: 'profile'
                    }
                },
                {
                    "$unwind": "$profile"
                },
                {
                    $lookup: {
                        from: 'comment_likes',
                        localField: '_id',
                        foreignField: 'comment_id',
                        as: 'likes'
                    }
                },
                {
                    $lookup: {
                        from: 'reactions',
                        localField: 'likes.like_id',
                        foreignField: '_id',
                        as: 'reaction'
                    }
                },
                {
                    $addFields: {
                        totalLikes: { $size: '$likes' },
                        userLiked: {
                            $cond: {
                                if: { $in: [req.user_id, '$likes.user_id'] },
                                then: true,
                                else: false
                            }
                        },
                        // userLikeEmoji: {
                        //     $cond: {
                        //         if: { $in: [req.user_id, '$likes.user_id'] },
                        //         then: '$reaction.image',
                        //         else: ''
                        //     }
                        // }

                        // Add more fields for other emoji types as needed
                    }
                },
                {
                    $sort: { createdAt: -1 }
                },
                {
                    $project: {
                        profile: {
                            username: '$profile.username',
                            first_name: '$profile.first_name',
                            last_name: '$profile.last_name',
                            profile_image:
                            {
                                $concat: [constant.s3Bucket_profile, {
                                    $ifNull: ['$profile.profile_image', 'default-avatar.png']
                                }]
                            },
                            email: '$profile.email'
                        },
                        message: 1,
                        post_id: 1,
                        _id: 1,
                        parentComment: 1,
                        createdAt: 1,
                        updatedAt: 1,
                        totalLikes: 1,
                        userLiked: 1,
                        // userLikeEmoji: 1,
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(result => {
                if (result.length > 0) {
                    asyncLoop(result, (item, next) => {
                        postComments.aggregate([
                            {
                                $match: {
                                    post_id: objectId,
                                    parentComment: item._id
                                }
                            },
                            {
                                $lookup: {
                                    from: 'users',
                                    localField: 'user_id',
                                    foreignField: '_id',
                                    as: 'profile'

                                }
                            },
                            {
                                "$unwind": "$profile"
                            },
                            {
                                $lookup: {
                                    from: 'comment_likes',
                                    localField: '_id',
                                    foreignField: 'comment_id',
                                    as: 'likes'
                                }
                            },
                            {
                                $addFields: {
                                    totalLikes: { $size: '$likes' },
                                    userLiked: {
                                        $cond: {
                                            if: { $in: [req.user_id, '$likes.user_id'] },
                                            then: true,
                                            else: false
                                        }
                                    },
                                    // Add more fields for other emoji types as needed
                                }
                            },
                            {
                                $sort: { createdAt: -1 }
                            },
                            {
                                $project: {
                                    profile: {
                                        username: '$profile.username',
                                        first_name: '$profile.first_name',
                                        last_name: '$profile.last_name',
                                        profile_image:
                                        {
                                            $concat: [constant.s3Bucket_profile, {
                                                $ifNull: ['$profile.profile_image', 'default-avatar.png']
                                            }]
                                        },
                                        email: '$profile.email'
                                    },
                                    message: 1,
                                    post_id: 1,
                                    _id: 1,
                                    createdAt: 1,
                                    comment_id: '$parentComment',
                                    updatedAt: 1,
                                    totalLikes: 1,
                                    userLiked: 1,
                                }
                            }
                        ]).then(replies => {
                            item.replies = replies
                            next();
                        })
                    }, () => {
                        callback(responseCode.SUCCESS, t('data_found'), result)
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, [])
        }
    },
    likeComment: (req, callback) => {
        try {
            commentLikeModel.find({ comment_id: req.comment_id, user_id: req.user_id }, (err, result) => {

                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    commentLikeModel.updateOne({ _id: result[0]._id }, { like_id: req.like_id }, (err, result) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('comment_liked'), {})
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    commentLikeModel.create({ user_id: req.user_id, comment_id: req.comment_id, like_id: req.like_id }, (err, result) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('comment_liked'), {})
                        }
                    })
                }
            })
        } catch (e) {
            console.log("Error in likeUnlikePost : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },
    unlikeComment: (req, callback) => {
        try {
            commentLikeModel.find({ comment_id: req.comment_id, user_id: req.user_id }, (err, result) => {

                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (result.length > 0) {
                    // Unlike the post
                    commentLikeModel.deleteOne({ _id: result[0]._id }, (err, result) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('comment_unliked'), {})
                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('something_went_wrong'), {});
                }
            })
        } catch (e) {
            console.log("Error in likeUnlikePost : ", e);
            callback(responseCode.OPERATION_FAILED, e, {});
        }

    },

    /**
        * @description API method for collection listing
        */
    collectionList: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        let responseData = [];
        collectionModel.find({ user_id: req.user_id, is_active: "1", is_delete: "0" }).sort({ createdAt: -1 }).skip(start)
            .limit(limit)
            .exec((_err, _result) => {
                // Your code here
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, []);
                } else {
                    if (_result.length > 0) {
                        asyncLoop(_result, (item, next) => {
                            collectionPostModel.aggregate([
                                {
                                    $match: { user_id: req.user_id, collection_id: item._id }
                                },
                                {
                                    $lookup: {
                                        from: 'posts',
                                        localField: 'post_id',
                                        foreignField: '_id',
                                        as: 'postData'
                                    }
                                },
                                {
                                    $lookup: {
                                        from: 'users',
                                        localField: 'postData.user_id',
                                        foreignField: '_id',
                                        as: 'profile'
                                    }
                                },
                                {
                                    "$unwind": "$profile"
                                },
                                {
                                    "$unwind": "$postData"
                                },
                                {
                                    $sort: { createdAt: -1 }
                                },
                                {
                                    $project: {
                                        profile: {
                                            username: '$profile.username',
                                            first_name: '$profile.first_name',
                                            last_name: '$profile.last_name',
                                            profile_image:
                                            {
                                                $concat: [constant.s3Bucket_profile, {
                                                    $ifNull: ['$profile.profile_image', 'default-avatar.png']
                                                }]
                                            },
                                            email: '$profile.email'
                                        },
                                        post_collection_id: '$_id',
                                        post_id: '$postData._id',
                                        type: '$postData.type',
                                        message: '$postData.message',
                                        location: '$postData.location',
                                        description: '$postData.description',
                                        location_latitude: '$postData.location_latitude',
                                        location_longitude: '$postData.location_longitude',
                                        location_full_address: '$postData.location_full_address'
                                    }
                                },
                                { $skip: 0 },
                                { $limit: 1 }
                            ], (_err, postData) => {
                                if (_err) {
                                    next(_err);
                                    return;
                                } else if (postData.length > 0) {
                                    responseData.push({
                                        _id: item._id,
                                        name: item.name,
                                        updatedAt: item.updatedAt,
                                        createdAt: item.createdAt,
                                        posts: postData[0]
                                    })
                                    next();
                                } else {
                                    responseData.push({
                                        _id: item._id,
                                        name: item.name,
                                        updatedAt: item.updatedAt,
                                        createdAt: item.createdAt,
                                        posts: {}
                                    })
                                    next();
                                }
                            })
                        }, (err) => {
                            if (!err) {
                                callback(responseCode.SUCCESS, t('data_found'), responseData);
                            } else {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            }
                        })
                    } else {
                        callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                    }
                }
            })
    },
    /**
     * @description API method for creating collection
     */
    addCollection: (req, callback) => {
        let addData = {
            name: req.name,
            user_id: req.user_id
        }
        collectionModel.create(addData, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('collection_added'), { _id: _result._id });
            }
        })
    },
    /**
     * @description API method for edit collection
     */
    editCollection: (req, callback) => {
        let addData = {
            name: req.name
        }
        collectionModel.updateOne({ _id: req.collection_id }, addData, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('collection_edited'), {});
            }
        })
    },
    /**
     * @description API method for delete collection
     */
    deleteCollection: (req, callback) => {
        collectionModel.updateOne({ _id: req.collection_id }, { is_delete: "1" }, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('collection_deleted'), {});
            }
        })
    },
    /**
      * @description API method for Save Post in Collection
      */
    savePostInCollection: (req, callback) => {
        let post_data = {
            collection_id: req.collection_id,
            user_id: req.user_id,
            post_id: req.post_id
        }
        collectionPostModel.create(post_data, (_err, _insertedData) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('post_saved_in_collection'), {});
            }
        })
    },
    /**
      * @description API method for Delete Post From Collection
      */
    deletePostFromCollection: (req, callback) => {
        let post_data = {
            _id: req.post_collection_id
        }
        collectionPostModel.deleteOne(post_data, (_err, deletedData) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('post_deleted_from_collection'), {});
            }
        })
    },
    /**
      * @description API method for get Post From Collection
      */
    getPostFromCollection: (req, callback) => {
        let page = (req.page != undefined && req.page > 0) ? req.page : 1;
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        const objectId = mongoose.Types.ObjectId(req.collection_id);
        collectionPostModel.aggregate([
            {
                $match: { user_id: req.user_id, collection_id: objectId }
            },
            {
                $lookup: {
                    from: 'posts',
                    localField: 'post_id',
                    foreignField: '_id',
                    as: 'postData'
                }
            },
            {
                $lookup: {
                    from: 'users',
                    localField: 'postData.user_id',
                    foreignField: '_id',
                    as: 'profile'
                }
            },
            {
                "$unwind": "$profile"
            },
            {
                "$unwind": "$postData"
            },
            {
                $sort: { createdAt: -1 }
            },
            {
                $project: {
                    profile: {
                        username: '$profile.username',
                        first_name: '$profile.first_name',
                        last_name: '$profile.last_name',
                        profile_image:
                        {
                            $concat: [constant.s3Bucket_profile, {
                                $ifNull: ['$profile.profile_image', 'default-avatar.png']
                            }]
                        },
                        email: '$profile.email'
                    },
                    post_collection_id: '$_id',
                    post_id: '$postData._id',
                    type: '$postData.type',
                    message: '$postData.message',
                    location: '$postData.location',
                    description: '$postData.description',
                    location_latitude: '$postData.location_latitude',
                    location_longitude: '$postData.location_longitude',
                    location_full_address: '$postData.location_full_address'
                }
            },
            { $skip: start },
            { $limit: limit }
        ], (_err, postData) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, []);
            } else if (postData.length > 0) {
                callback(responseCode.SUCCESS, t('data_found'), postData)
            } else {
                callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
            }
        })
    },
    /**
    * API method for create story 
    */
    addStory: (req, callback) => {
        if (req.medias.length > 0) {
            let addData = []
            req.medias.map((item) => {
                addData.push([req.user_id, item, req.privacy, moment().format('YYYY-MM-DD HH:mm:ss'), req.duration]);
            })
            let sql = `INSERT INTO tbl_stories set (user_id, media, privacy, add_time, duration) VALUES ?`
            con.query(sql, [addData], function (err, result) {
                if (!err) {
                    callback(responseCode.SUCCESS, t('story_added_success'), {});
                } else {
                    console.log("Error while adding data to stories : ", err);
                    callback(responseCode.OPERATION_FAILED, err, {});
                }
            });
        } else {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {});
        }
    },
    /**
    * API method for create story 
    */
    getStory: (req, callback) => {
        let sql = `SELECT * FROM tbl_stories WHERE user_id=? and is_delete=0`;
        con.query(sql, function (err, result) {
            if (!err) {
                if (result.length > 0) {
                    let responseData = [];
                    asyncLoop(result, (item, next) => {
                        if (item.privacy == 'public') {
                            item.isPrivate = false;
                            responseData.push(item);
                            next();
                        } else if (item.privacy == 'following') {
                            user_model.follower_list(req, (code, message, following) => {
                                if (code == 1) {

                                } else {
                                    next(message);
                                    return;
                                }
                            })
                        } else if (item.privacy == 'family_only') {

                        } else if (item.privacy == 'only_me') {

                        } else if (item.privacy == 'tribe') {

                        } else if (item.privacy == 'friends') {

                        } else if (item.privacy == 'profile') {

                        } else if (item.privacy == 'groups') {

                        }
                    }, () => {

                    })
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), []);
                }
            } else {
                console.log("Error while adding data to stories : ", err);
                callback(responseCode.OPERATION_FAILED, err, {});
            }
        });
    },
};

module.exports = social_model