const express = require('express');
const router = express.Router();
const Validator = require('Validator');
const user_model = require('./user_model')
const common = require('../../../utilities/common')
const validate_header = require('../../../middleware/validate-header-web')
const uniqueMiddleware = require('../../../middleware/uniqueMiddleware')
const httpStatusCode = require('http-status-codes');
const { t } = require('localizify');
/* routes are starting here */
router.get('/test', (req, res) => {
    let request = req.body;
    console.log(request)
    user_model.test(request, (code, message, response) => {
        let response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(res, response_data);
    })
})
router.post('/signup', uniqueMiddleware.uniqueEmail, uniqueMiddleware.uniqueMobile, uniqueMiddleware.uniqueUsername, (req, res) => {
    let request = req.body;
    console.log("request", request)
    let rules = {
        username: "required",
        email: "required",
        country_code: "required",
        phone: "required",
        password: "required",
        device_type: "required",
        device_token: "required",
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    let error = '';
    if (v.fails()) {
        const errors = v.getErrors();
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.signup(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 4 || code == "4") {
                response_data.data = response;
            }
    console.log("response_data", response_data)
            
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/verifyOTP', (req, res) => {
    let request = req.body;
    let rules = {
        // email: "required",
        otp: "required",
        device_type: "required",
        device_token: "required",
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.verifyOTP(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/setPin', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    console.log("object")
    let rules = {
        pin: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.setPin(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/editProfile', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        type: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {

        user_model.editProfile(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/resendOTP', (req, res) => {
    let request = req.body;

    user_model.resendOTP(request, (code, message, response) => {
        let response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(res, response_data);
    })
})
router.post('/changePassword', (req, res) => {
    let request = req.body;
    // request.user_id = req.login_user_id;
    // request.device_type = req.login_user_device_type;
    let rules = {
        password: "required",
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.changePassword(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/changePin', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        old_pin: "required",
        pin: "required",
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.changePin(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/enableBiometrics', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        biomatric_enable: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.enableBiometrics(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/signIn', (req, res) => {
    let request = req.body;
    let rules = {
        password: "required",
        device_type: "required",
        device_token: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.signIn(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/authenticateWithPin', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    console.log("object")
    let rules = {
        pin: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.authenticateWithPin(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/logout', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    user_model.logout(request, (code, message, response) => {
        let response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(res, response_data);
    })
})
router.post('/forgotPassword', (req, res) => {
    let request = req.body;
    // request.user_id = req.login_user_id;
    // request.device_type = req.login_user_device_type;
    user_model.forgotPassword(request, (code, message, response) => {
        let response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(res, response_data);
    })
})
router.post('/contactUs', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        first_name: "required",
        last_name: "required",
        email: "required",
        subject: "required",
        description: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.contactUs(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/getUsers', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    user_model.getUsers(request, (code, message, response) => {
        let response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(res, response_data);
    })
})
router.post('/followUnfollow', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        following_id: "required"/* Represents the user who is being followed. 	 */
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.followUnfollow(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/follower_list', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    user_model.follower_list(request, (code, message, response) => {
        let response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(res, response_data);
    })
})
router.post('/following_list', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    user_model.following_list(request, (code, message, response) => {
        let response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(res, response_data);
    })
})
router.post('/getFamily', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    user_model.getFamily(request, (code, message, response) => {
        let response_data = {
            code: code,
            message: message
        }
        if (code == 1 || code == "1") {
            response_data.data = response;
        }
        validate_header.send_response(res, response_data);
    })
})
router.post('/addFamily', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        family_data: "required|array"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.addFamily(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/deleteFamily', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        family_id: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.deleteFamily(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/addGroup', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        name: "required",
        image: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.addGroup(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/addMemberToGroup', (req, res) => {
    let request = req.body;
    request.user_id = req.login_user_id;
    request.device_type = req.login_user_device_type;
    let rules = {
        group_id: "required",
        users: "required|array"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.addMemberToChatRoom(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
router.post('/removeMemberFromGroup', (req, res) => {
    let request = req.body;
    // request.user_id = req.login_user_id;
    // request.device_type = req.login_user_device_type;
    let rules = {
        chat_room_id: "required",
        user_id: "required"
    };
    let message = {
        required: t('field_requird')
    }
    let v = Validator.make(request, rules, message);
    if (v.fails()) {
        const errors = v.getErrors();
        let error = '';
        for (let key in errors) {
            error = errors[key][0];
            break;
        }
        let response_data = {
            code: "0",
            message: error
        };
        validate_header.send_response(res, response_data, httpStatusCode.OK);
    } else {
        user_model.removeMemberFromGroup(request, (code, message, response) => {
            let response_data = {
                code: code,
                message: message
            }
            if (code == 1 || code == "1") {
                response_data.data = response;
            }
            validate_header.send_response(res, response_data);
        })
    }
})
module.exports = router;