const responseCode = require('../../../config/response-error-code');
const con = require('../../../config/database');
const common = require('../../../utilities/common');
const validate_header = require('../../../middleware/validate-header-web');
const moment = require('moment');
const constant = require('../../../config/constant');
const asyncLoop = require('node-async-loop')
const userModel = require('../../../model/user');
const contactModel = require('../../../model/contact');
const followersModel = require('../../../model/followers');
const userInerestsModel = require('../../../model/user_interests');
const userQuesAnsModel = require('../../../model/user_question_answers');
const userDeviceModel = require('../../../model/user_device_info');
const mongoose = require('mongoose');
const { t } = require('localizify');
let user_model = {
    api_user_list: (callback) => {
        userModel.aggregate([
            {
                $lookup: {
                    from: 'user_device_infos',     //collection name
                    localField: '_id',
                    foreignField: 'user_id',
                    as: 'userDeviceData'     //alias
                }
            },
            {
                $unwind: '$userDeviceData' // Deconstructs the array of joined documents into separate documents
            },
            {
                $project: {
                    username: 1,
                    nickname: 1,
                    first_name: 1,
                    last_name: 1,
                    dob: 1,
                    email: 1,
                    country_code: 1,
                    phone: 1,
                    gender: 1,
                    profile_image: { $concat: [constant.s3Bucket_profile, "$profile_image"] },
                    blood_type: 1,
                    pin: 1,
                    password: 1,
                    profile_type: 1,
                    signup_step: 1,
                    status: 1,
                    is_verify: 1,
                    token: '$userDeviceData.token', // Select specific field (name) from the joined user data
                    device_type: '$userDeviceData.device_type', // Select specific field (name) from the joined user data
                    device_token: '$userDeviceData.device_token' // Select specific field (name) from the joined user data
                }
            }
        ]).then(result => {
            if (result.length > 0) {
                // console.log("result", result)
                asyncLoop(result, (item, next) => {
                    item.password = (item.password != null && item.password != '') ? validate_header.dcryption(item.password) : ''
                    item.pin = (item.pin != null && item.pin != '') ? validate_header.dcryption(item.pin) : ''
                    // console.log("object", item)
                    next();
                }, () => {
                    callback(result);
                })

            } else {
                console.log("Error:", err);
                callback([]);

            }
        })

    },
    signup: (req, callback) => {
        let password = validate_header.encryption(req.password);
        let insert_data = {
            email: req.email,
            username: req.username,
            password: password,
            otp: '1111'
            // otp: Math.floor(100000 + Math.random() * 900000)
        }
        if (req.country_code != undefined && req.country_code != '') {
            insert_data.country_code = req.country_code
        }
        if (req.phone != undefined && req.phone != '') {
            insert_data.phone = req.phone
        }
        insert_data.referral_code = validate_header.randomTokenString(8);
        if (req.referral_code != undefined && req.referral_code != '') {
            userModel.findOne({ referral_code: req.referral_code }, function (err, userData) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    if (_data.length > 0) {
                        insert_data.referred_by = userData._id;
                    } else {
                        insert_data.referred_by = '';
                    }
                    userModel.create(insert_data, function (err, userObj) {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            let insert_device_data = {
                                user_id: userObj._id,
                                device_type: req.device_type,
                                device_token: req.device_token
                            }
                            userDeviceModel.create(insert_device_data, (err, userDeviceObj) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    common.getUserData({ user_id: userObj._id, device_type: req.device_type }, (err, user) => {
                                        if (err) {
                                            callback(responseCode.OPERATION_FAILED, err, {});
                                        } else {
                                            callback(responseCode.OTP_NOT_VERYFIED, t('verification_code_sent'), user);
                                        }
                                    })
                                }
                            })
                        }
                    })
                }
            })
        } else {
            userModel.create(insert_data, function (err, userObj) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    let insert_device_data = {
                        user_id: userObj._id,
                        device_type: req.device_type,
                        device_token: req.device_token
                    }
                    userDeviceModel.create(insert_device_data, (err, userDeviceObj) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            common.getUserData({ user_id: userObj._id, device_type: req.device_type }, (err, user) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    callback(responseCode.OTP_NOT_VERYFIED, t('verification_code_sent'), user);
                                }
                            })
                        }
                    })
                }
            })
        }

    },
    verifyOTP: (req, callback) => {
        if (req.email != undefined && req.email != '') {
            userModel.findOne({ email: req.email }, (err, user) => {
                if (err) {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
                } else if (user.status == 'Active') {
                    if (user.is_verify == '0') {
                        if (user.otp == req.otp) {
                            let updateData = {
                                otp: '',
                                is_verify: '1',
                                status: 'Active',
                            };
                            if (user.signup_step != 'DONE') {
                                updateData.signup_step = 'VERIFY';
                            }
                            userModel.updateOne({ _id: user._id }, updateData, function (err, data) {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    let updateData = {
                                        token: validate_header.randomTokenString(40),
                                        device_type: req.device_type,
                                        device_token: req.device_token
                                    };
                                    userDeviceModel.updateOne({ user_id: user._id, device_type: req.device_type }, updateData, function (err, data) {
                                        if (err) {
                                            callback(responseCode.OPERATION_FAILED, err, []);
                                        } else {
                                            common.getUserData({ user_id: user._id, device_type: req.device_type }, (err, user) => {
                                                if (err) {
                                                    callback(responseCode.OPERATION_FAILED, err, {});
                                                } else {
                                                    callback(responseCode.SUCCESS, t('verification_success'), user);
                                                }
                                            })
                                        }
                                    })
                                }
                            });
                        } else {
                            callback(responseCode.OPERATION_FAILED, t('invalid_verification_code'), {});
                        }
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('already_verified'), {});
                    }

                } else {
                    return callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {});
                }
            });
        } else if (req.phone != undefined && req.phone != '') {
            userModel.findOne({ phone: req.phone, country_code: req.country_code }, function (err, user) {
                if (!user[0]) {
                    return callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
                } else if (user.status == 'Active') {
                    if (user.is_verify == '0') {
                        if (user.otp == req.otp) {
                            let updateData = {
                                otp: '',
                                is_verify: '1',
                                status: 'Active',
                            };
                            if (user.signup_step != 'DONE') {
                                updateData.signup_step = 'VERIFY';
                            }
                            userModel.updateOne({ _id: user._id }, updateData, function (err, data) {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    updateData = {
                                        token: validate_header.randomTokenString(40),
                                        device_type: req.device_type,
                                        device_token: req.device_token
                                    };
                                    userDeviceModel.updateOne({ user_id: user._id, device_type: req.device_type }, updateData, function (err, data) {
                                        if (err) {
                                            callback(responseCode.OPERATION_FAILED, err, []);
                                        } else {
                                            common.getUserData({ user_id: user._id, device_type: req.device_type }, (err, user) => {
                                                if (err) {
                                                    callback(responseCode.OPERATION_FAILED, err, {});
                                                } else {
                                                    callback(responseCode.SUCCESS, t('verification_success'), user);
                                                }
                                            })
                                        }
                                    })
                                }
                            });
                        } else {
                            callback(responseCode.OPERATION_FAILED, t('invalid_verification_code'), {});
                        }
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('already_verified'), {});
                    }

                } else {
                    callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {});
                }

            });
        } else {
            callback(responseCode.OPERATION_FAILED, t('provide_any_one_details'), {});
        }
    },
    /*
     * API method for set pin authentication  
     */
    setPin: (req, callback) => {
        userModel.findById(req.user_id, function (err, user) {
            if (err) {
                callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
            } else {
                if (user.is_pin_set == '0') {
                    let password = validate_header.encryption(req.pin);
                    let params = {
                        is_pin_set: 1,
                        pin: password,
                        signup_step: 'DONE'
                    };
                    userModel.updateOne({ _id: user._id }, params, function (err, data) {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            common.getUserData({ user_id: user._id, device_type: req.device_type }, (err, user) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    callback(responseCode.SUCCESS, t('sinup_success'), user);
                                }
                            })
                        }
                    });
                } else {
                    callback(responseCode.OPERATION_FAILED, t('already_pin_set'));
                }
            }
        });
    },
    /* 
    * API method for signin and login with email or phone number or username and paswword
    */
    signIn: (req, callback) => {
        if (req.username != undefined && req.username != '') {
            userModel.find({
                $or: [
                    { 'username': req.username },
                    { 'email': req.username }
                ],
                is_delete: '0'
            }, (err, login_data) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (login_data.length < 1) {
                    callback(responseCode.NO_DATA_FOUND, t('invalid_credentials'), {});
                } else {
                    let userData = login_data[0];
                    if (userData.status == 'Active') {
                        /*  Check whether PIN has been set or not. If yes then check the entered PIN otherwise go to next step.*/
                        /*  
                        * Check whether PIN has been set or not.
                        * If yes then check the entered PIN is correct or not.
                        * If no then simply return the user profile without asking for PIN.
                        */
                        if (req.password == validate_header.dcryption(userData.password)) {
                            let updateData = {
                                device_type: req.device_type,
                                device_token: req.device_token
                            };
                            if (userData.signup_step == 'VERIFY' || userData.signup_step == 'DONE') {
                                updateData.token = validate_header.randomTokenString(40);
                            }
                            // console.log("updateData", updateData)
                            // console.log("userData", userData)
                            userDeviceModel.updateOne({ user_id: userData._id }, updateData, function (err, data) {
                                // console.log("err", err)
                                // console.log("data", data)
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    common.getUserData({ user_id: userData._id, device_type: req.device_type }, (err, user) => {
                                        if (err) {
                                            callback(responseCode.OPERATION_FAILED, err, {});
                                        } else {
                                            callback(responseCode.SUCCESS, t('login_success'), user);
                                        }
                                    })
                                }
                            });
                        } else {
                            callback(responseCode.OPERATION_FAILED, t('invalid_password'), {});
                        }
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {})
                    }
                }
            })
        } else {
            userModel.find({
                phone: req.phone,
                country_code: req.country_code,
                is_delete: '0'
            }, (err, login_data) => {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else if (login_data.length < 1) {
                    callback(responseCode.NO_DATA_FOUND, t('invalid_credentials'), {});
                } else {
                    let userData = login_data[0];
                    if (userData.status == 'Active') {
                        /*  Check whether PIN has been set or not. If yes then check the entered PIN otherwise go to next step.*/
                        /*  
                        * Check whether PIN has been set or not.
                        * If yes then check the entered PIN is correct or not.
                        * If no then simply return the user profile without asking for PIN.
                        */
                        if (req.password == validate_header.dcryption(userData.password)) {
                            let updateData = {
                                device_type: req.device_type,
                                device_token: req.device_token
                            };
                            if (userData.signup_step == 'VERIFY' || userData.signup_step == 'DONE') {
                                updateData.token = validate_header.randomTokenString(40);
                            }
                            userDeviceModel.updateOne({ user_id: userData._id }, updateData, function (err, data) {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    common.getUserData({ user_id: userData._id, device_type: req.device_type }, (err, user) => {
                                        if (err) {
                                            callback(responseCode.OPERATION_FAILED, err, {});
                                        } else {
                                            callback(responseCode.SUCCESS, t('login_success'), user);
                                        }
                                    })
                                }
                            });
                        } else {
                            callback(responseCode.OPERATION_FAILED, t('invalid_password'), {});
                        }
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {})
                    }
                }
            })
        }
    },
    /* 
    * API method for signin and login with  pin
    */
    authenticateWithPin: (req, callback) => {
        userModel.find({
            _id: req.user_id,
            is_delete: '0'
        }, (err, login_data) => {
            if (err) {
                callback(responseCode.OPERATION_FAILED, err, {});
            } else if (login_data.length < 1) {
                callback(responseCode.NO_DATA_FOUND, t('invalid_credentials'), {});
            } else {
                let userData = login_data[0];
                if (userData.signup_step == 'DONE') {
                    if (userData.status == 'Active') {
                        /*  Check whether PIN has been set or not. 
                        If yes then check the entered PIN otherwise go to next step.*/
                        /*  
                        * Check whether PIN has been set or not.
                        * If yes then check the entered PIN is correct or not.
                        * If no then simply return the user profile without asking for PIN.
                        */
                        if (req.pin == validate_header.dcryption(userData.pin)) {
                            common.getUserData({ user_id: userData._id, device_type: req.device_type }, (err, user) => {
                                if (err) {
                                    callback(responseCode.OPERATION_FAILED, err, {});
                                } else {
                                    callback(responseCode.SUCCESS, t('login_success'), user);
                                }
                            })
                        } else {
                            callback(responseCode.OPERATION_FAILED, t('invalid_pin'));
                        }
                    } else {
                        callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {})
                    }
                } else {
                    callback(responseCode.OPERATION_FAILED, t('signup_step_incomplete'), {})
                }
            }
        })
    },
    /**
     * API method for enable biometric authentication in mobile app.
     * @param {Object} req - request object
     * @param {Function} callback - callback function
     */
    enableBiometrics: (req, callback) => {
        let update_data = {
            is_biomatric_enable: req.biomatric_enable
        };
        userModel.updateOne({ _id: req.user_id }, update_data, (err, data) => {
            if (err) {
                callback(responseCode.OPERATION_FAILED, err, {});
            } else {
                common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, {});
                    } else {
                        callback(responseCode.SUCCESS, t('enable_biometric_success'), user);
                    }
                })
            }
        });
    },
    /**
     * API method for forgot password
    */
    forgotPassword: (req, callback) => {
        if (req.email != undefined && req.email != '') {
            // Validate email
            userModel.find({ email: req.email, is_delete: '0' }, (err, rows) => {
                if (rows.length > 0) {
                    if (rows[0].status == 'Active') {
                        // var otp = Math.floor(1000 + Math.random() * 9000);
                        // var message = "Your OTP to reset your password is " + otp;
                        let updateData = {
                            otp: '1111',
                            is_verify: 0,
                        };
                        userModel.updateOne({ _id: rows[0]._id }, updateData, (err, data) => {
                            if (err) {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            } else {
                                /* send otp on email */
                                callback(responseCode.SUCCESS, t('verification_code_sent'), { email: rows[0].email, phone: rows[0].phone, country_code: rows[0].country_code });
                            }
                        })
                        // smsService.sendSMS(rows[0].country_code + "" + rows[0].phone, message).then((resp) => {
                        //     callback(responseCode.SUCCESS, 'OTP has been sent successfully', { user_id: rows[0].id, country_code: rows[0].country_code})
                        // }).catch((error) => {
                        //     callback(responseCode.OTP_NOT_SENT, error, null);
                        // });
                    } else {
                        return callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {});
                    }
                } else {
                    callback(responseCode.OPERATION_FAILED, t('data_not_found'));
                }
            });
        } else if (req.phone != undefined && req.phone != '') {
            // Validate phone number
            userModel.find({ phone: req.phone, country_code: req.country_code, is_delete: '0' }, (err, rows) => {
                if (rows.length > 0) {
                    if (rows[0].status == 'Active') {
                        // let otp = Math.floor(1000 + Math.random() * 9000);
                        // let message = "Your OTP to reset your password is " + otp;
                        let updateData = {
                            otp: '1111',
                            is_verify: 0,
                        };
                        userModel.updateOne({ _id: rows[0]._id }, updateData, (err, data) => {
                            if (err) {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            } else {
                                /* send otp on email */
                                callback(responseCode.SUCCESS, t('verification_code_sent'), { email: rows[0].email, phone: rows[0].phone, country_code: rows[0].country_code });
                            }
                        })
                        // smsService.sendSMS(rows[0].country_code + "" + rows[0].phone, message).then((resp) => {
                        //     callback(responseCode.SUCCESS, 'OTP has been sent successfully', { user_id: rows[0].id, country_code: rows[0].country_code})
                        // }).catch((error) => {
                        //     callback(responseCode.OTP_NOT_SENT, error, null);
                        // });
                    } else {
                        return callback(responseCode.OPERATION_FAILED, t('account_is_deactivated'), {});
                    }
                } else {
                    callback(responseCode.OPERATION_FAILED, t('data_not_found'));
                }
            });
        } else {
            callback(responseCode.OPERATION_FAILED, t('provide_any_one_details'), {})
        }
    },
    /**
     * API method for logout from
      */
    logout: (req, callback) => {
        let update_data = {
            token: ''
        }
        userDeviceModel.updateOne({ user_id: req.user_id, device_type: req.device_type }, update_data, (err, result) => {
            if (err) {
                callback(responseCode.OPERATION_FAILED, err, null)
            } else {
                callback(responseCode.SUCCESS, t('logout_success'), {});
            }
        })
    },
    /*
    * API method for change password 
    */
    changePassword: (req, callback) => {
        if (req.email != undefined && req.email != '') {
            userModel.find({ email: req.email }, (err, user) => {
                if (!user) {
                    return callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
                } else {
                    let password = validate_header.encryption(req.password);
                    let params = {
                        password: password
                    };
                    userModel.updateOne({ _id: user[0]._id }, params, (err, data) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('password_change_success'), {});

                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    });
                }
            });
        } else if (req.phone != undefined && req.phone != '') {
            userModel.find({ phone: req.phone, country_code: req.country_code }, (err, user) => {
                if (!user[0]) {
                    return callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
                } else {
                    let password = validate_header.encryption(req.password);
                    let params = {
                        password: password
                    };
                    userModel.updateOne({ _id: user[0]._id }, params, (err, data) => {
                        if (!err) {
                            callback(responseCode.SUCCESS, t('password_change_success'), {});

                        } else {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        }
                    });
                }
            });
        } else {
            return callback(responseCode.OPERATION_FAILED, 'Please provide either email or phone number', null);
        }

    },

    /*
    * API method for change pin 
    */
    changePin: (req, callback) => {
        userModel.find({ _id: req.user_id }, (err, user) => {
            if (!user[0] || err) {
                return callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {});
            } else if (user[0].pin == validate_header.encryption(req.old_pin)) {
                let password = validate_header.encryption(req.pin);
                let params = {
                    pin: password
                };
                userModel.updateOne({ _id: req.user_id }, params, (err, data) => {
                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, {});
                    } else {
                        common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                            if (err) {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            } else {
                                callback(responseCode.SUCCESS, t('pin_change_success'), user);
                            }
                        })
                    }
                });
            } else {
                callback(responseCode.OPERATION_FAILED, t('incorrect_old_pin'));
            }
        });
    },
    /**
     * API method for resend otp
      */
    resendOTP: (req, callback) => {
        if (req.email != undefined && req.email != '') {
            let email = req.email;
            //checking whether the user is already exist or not
            userModel.find({ email: email }, (err, result) => {
                if (result && result.length > 0) {
                    // let otp = Math.floor(1000 + Math.random() * 9000);
                    // let message = 'Your One Time Password is :' + otp;
                    // sendEmail("Reset Your Password", "Hi," + message + "Best Regards,  Team");
                    // sendMessage("Reset Your Password", "Hi," + message + "Best Regards, Team");
                    let update_data = {
                        otp: '1111',
                        is_verify: 0
                    }
                    userModel.updateOne({ _id: result[0]._id }, update_data, (err, data) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            delete result[0].otp;
                            callback(responseCode.SUCCESS, t('otp_resend'), result[0]);
                        }
                    });
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {})
                }
            });
        } else if (req.phone != undefined && req.phone != '') {
            //checking whether the user is already exist or not
            userModel.find({ phone: req.phone, country_code: req.country_code, }, (err, result) => {
                if (result && result.length > 0) {
                    // let otp = Math.floor(1000 + Math.random() * 9000);
                    // let message = 'Your One Time Password is :' + otp;
                    // sendEmail("Reset Your Password", "Hi," + message + "Best Regards,  Team");
                    // sendMessage("Reset Your Password", "Hi," + message + "Best Regards, Team");
                    let update_data = {
                        otp: '1111',
                        is_verify: 0
                    }
                    userModel.updateOne({ _id: result[0]._id }, update_data, (err, data) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            delete result[0].otp;
                            callback(responseCode.SUCCESS, t('otp_resend'), result[0]);
                        }
                    });
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), {})
                }
            });
        } else {
            callback(responseCode.OPERATION_FAILED, t('provide_any_one_details'), {})
        }
    },
    /**
     * API method for edit profile
     */
    editProfile: (req, callback) => {
        let update_data = {};
        if (req.type == 'personal_info') {
            if (req.first_name != undefined && req.first_name != '') {
                update_data.first_name = req.first_name;
            }
            if (req.last_name != undefined && req.last_name != '') {
                update_data.last_name = req.last_name;
            }
            if (req.nickname != undefined && req.nickname != '') {
                update_data.nickname = req.nickname;
            }
            if (req.profile_image != undefined && req.profile_image != '') {
                update_data.profile_image = req.profile_image;
            }
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_dob_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'dob') {
            if (req.dob != undefined && req.dob != '') {
                update_data.dob = req.dob;
            }
            // console.log(update_data)
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_dob_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'my_interest') {
            if (req.interests.length > 0) {
                userInerestsModel.updateMany({ user_id: req.user_id }, { is_delete: '1' }, (err, result) => {
                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, null);
                    } else {
                        let insert_data = [];
                        req.interests.map((value) => {
                            insert_data.push({ user_id: req.user_id, interest_id: value });
                        });
                        userInerestsModel.create(insert_data, (err, result) => {
                            if (!err) {
                                common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                                    if (err) {
                                        callback(responseCode.OPERATION_FAILED, err, {});
                                    } else {
                                        callback(responseCode.SUCCESS, t('edit_interest_success'), user);
                                    }
                                })
                            } else {
                                console.log("Error while adding questions : ", err);
                                callback(responseCode.OPERATION_FAILED, err, {});
                            }
                        })
                    }
                })
            } else {
                callback(responseCode.OPERATION_FAILED, t('please_provide_interests'), {});
            }

        } else if (req.type == 'gender') {

            if (req.gender != undefined && req.gender != '') {
                update_data.gender = req.gender;
            }
            if (req.gender != undefined && req.gender != '' && req.gender == '2') {
                update_data.other_gender = req.other_gender;
            }
            if (req.gender != undefined && req.gender != '' && req.gender == '3') {
                update_data.add_other_gender = req.add_other_gender;
            }
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_gender_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'profile') {
            if (req.profile_type != undefined && req.profile_type != '') {
                update_data.profile_type = req.profile_type
            }
            if (req.secondary_email != undefined && req.secondary_email != '') {
                update_data.secondary_email = req.secondary_email
            }
            if (req.secondary_country_code != undefined && req.secondary_country_code != '') {
                update_data.secondary_country_code = req.secondary_country_code
            }
            if (req.secondary_phone != undefined && req.secondary_phone != '') {
                update_data.secondary_phone = req.secondary_phone
            }
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_profile_type_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'blood') {
            if (req.blood_type != undefined && req.blood_type != '') {
                update_data.blood_type = req.blood_type
            }
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_blood_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'personality') {
            if (req.personality_type != undefined && req.personality_type != '') {
                update_data.personality_type = req.personality_type
            }
            userModel.updateOne({ _id: req.user_id }, update_data, function (err, data) {
                if (err) {
                    callback(responseCode.OPERATION_FAILED, err, {});
                } else {
                    common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                        if (err) {
                            callback(responseCode.OPERATION_FAILED, err, {});
                        } else {
                            callback(responseCode.SUCCESS, t('edit_profile_success'), user);
                        }
                    })
                }
            });
        } else if (req.type == 'question') {
            if (req.questions != undefined && req.questions.length > 0) {
                userQuesAnsModel.updateMany({ user_id: req.user_id }, { is_delete: '1' }, (err, result) => {
                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, null);
                    } else {
                        let insert_data = [];
                        req.questions.map((value) => {
                            insert_data.push({ user_id: req.user_id, question_id: value.question, answer_id: value.answer });
                        });
                        userQuesAnsModel.create(insert_data, (err, result) => {
                            if (!err) {
                                common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                                    if (err) {
                                        callback(responseCode.OPERATION_FAILED, err, {});
                                    } else {
                                        callback(responseCode.SUCCESS, t('edit_question_success'), user);
                                    }
                                })
                            } else {
                                console.log("Error while adding questions : ", err);
                                callback(responseCode.OPERATION_FAILED, err, {});
                            }
                        })
                    }
                })
            } else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {});
            }
        } else if (req.type == 'love_language') {
            // console.log("object", req)
            if (req.love_language != undefined && req.love_language.length > 0) {
                userModel.updateOne({ _id: req.user_id }, { love_language: req.love_language }, function (err, data) {

                    if (err) {
                        callback(responseCode.OPERATION_FAILED, err, {});
                    } else {
                        common.getUserData({ user_id: req.user_id, device_type: req.device_type }, (err, user) => {
                            // console.log("err", err)
                            // console.log("data", data)
                            if (err) {
                                callback(responseCode.OPERATION_FAILED, err, {});
                            } else {
                                callback(responseCode.SUCCESS, t('edit_love_language_success'), user);
                            }
                        })
                    }
                });
            }
            else {
                callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {})
            }
        }
    },
    /**
     * API method for contactUs
      */
    contactUs: (req, callback) => {
        let insert_data = {
            user_id: req.user_id,
            first_name: req.first_name,
            last_name: req.last_name,
            email: req.email,
            subject: req.subject,
            description: description
        }
        contactModel.create(insert_data, (err, result) => {
            if (!err) {
                callback(responseCode.SUCCESS, t('contactus_success'), {});
            } else {
                console.log("Error in Contact us : " + JSON.stringify(err));
                callback(responseCode.OPERATION_FAILED, err, null);
            }
        });
    },
    /**
     * API method for follow / unfollow
     */
    followUnfollow: (req, callback) => {
        const userID = mongoose.Types.ObjectId(req.user_id);
        const objectId = mongoose.Types.ObjectId(req.following_id);
        followersModel.find({ follower_id: userID, following_id: objectId }, (_err, _following_result) => {
            // console.log("_following_result", _following_result)
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else if (_following_result.length > 0) {
                /* Do unfollow  operation */
                followersModel.deleteOne({ _id: _following_result[0]._id }, (_err, _result) => {
                    if (_err) {
                        callback(responseCode.OPERATION_FAILED, _err, {});
                    } else {
                        callback(responseCode.SUCCESS, t('unfollowed_success'), {});
                    }
                })
            } else {
                /* Do follow  operation */
                let insData = {
                    follower_id: req.user_id,
                    following_id: req.following_id
                };
                followersModel.create(insData, (_err, _result) => {
                    if (_err) {
                        callback(responseCode.OPERATION_FAILED, _err, {});
                    } else {
                        callback(responseCode.SUCCESS, t('followed_success'), {});
                    }
                })
            }
        })
    },
    /**
    * API method for follower list 
    */
    follower_list: (req, callback) => {
        if (req.page <= 0) {
            let page = 1;
        } else {
            let page = req.page;
        }
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        try {
            followersModel.aggregate([
                {
                    $match: {
                        following_id: req.user_id,
                        is_delete: '0'
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: '_id',
                        foreignField: 'follower_id',
                        as: 'followUser'
                    }
                },
                {
                    $unwind: '$followUser' // Deconstructs the array of joined documents into separate documents
                },
                {
                    $project: {
                        _id: 1,
                        follower_id: 1,
                        first_name: '$followUser.first_name',
                        last_name: '$followUser.last_name',
                        email: '$followUser.email',
                        nickname: '$followUser.nickname',
                        username: '$followUser.username',
                        profile_type: '$followUser.profile_type',
                        profile_image: { $concat: [constant.s3Bucket_profile, "$followUser.profile_image"] }
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    /**
    * API method for following list 
    */
    following_list: (req, callback) => {
        if (req.page <= 0) {
            let page = 1;
        } else {
            let page = req.page;
        }
        let limit = constant.itemPerPage;
        let start = ((page - 1) * constant.itemPerPage);
        try {
            followersModel.aggregate([
                {
                    $match: {
                        follower_id: req.user_id,
                        is_delete: '0'
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: '_id',
                        foreignField: 'following_id',
                        as: 'followUser'
                    }
                },
                {
                    $unwind: '$followUser' // Deconstructs the array of joined documents into separate documents
                },
                {
                    $project: {
                        _id: 1,
                        following_id: 1,
                        first_name: '$followUser.first_name',
                        last_name: '$followUser.last_name',
                        email: '$followUser.email',
                        nickname: '$followUser.nickname',
                        username: '$followUser.username',
                        profile_type: '$followUser.profile_type',
                        profile_image: { $concat: [constant.s3Bucket_profile, "$followUser.profile_image"] }
                    }
                },
                { $skip: start },
                { $limit: limit }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    /**
   * API method for get all users  
   */
    getUsers: (req, callback) => {
        try {
            userModel.aggregate([
                {
                    $lookup: {
                        from: 'followers',
                        localField: '_id',
                        foreignField: 'following_id',
                        as: 'followers'
                    }
                },
                {
                    $match: {
                        _id: { $ne: req.user_id },
                        followers: { $size: 0 } // Filter users with no followers
                    }
                }
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },

    /**
    * API method for add family members 
    */
    addFamily: (req, callback) => {
        if (family_data.length > 0) {
            let addData = [];
            family_data.map((item) => {
                addData.push({ user_id: req.user_id, family_member_id: item.family_member_id, relationship_id: item.relationship_id });
            })
            familyModel.create(addData, (_err, _result) => {
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, {});
                } else {
                    callback(responseCode.SUCCESS, t('family_member_added'), {});
                }
            })
        } else {
            callback(responseCode.OPERATION_FAILED, t('something_went_wrong'), {});
        }
    },
    /**
   * API method for get family members.
   */
    getFamily: (req, callback) => {
        try {
            followersModel.aggregate([
                {
                    $match: {
                        user_id: req.user_id,
                        is_delete: '0'
                    }
                },
                {
                    $lookup: {
                        from: 'users',
                        localField: '_id',
                        foreignField: 'family_member_id',
                        as: 'User'
                    }
                },
                {
                    $unwind: '$User' // Deconstructs the array of joined documents into separate documents
                },
                {
                    $lookup: {
                        from: 'relationships',
                        localField: '_id',
                        foreignField: 'relationship_id',
                        as: 'relationship'
                    }
                },
                {
                    $unwind: '$relationship' // Deconstructs the array of joined documents into separate documents
                },
                {
                    $project: {
                        family_id: '$_id',
                        relationship_name: '$relationship.name',
                        first_name: '$User.first_name',
                        last_name: '$User.last_name',
                        email: '$User.email',
                        nickname: '$User.nickname',
                        username: '$User.username',
                        profile_type: '$User.profile_type',
                        profile_image: { $concat: [constant.s3Bucket_profile, "$User.profile_image"] }
                    }
                },
            ]).then(_results => {
                if (_results.length > 0) {
                    callback(responseCode.SUCCESS, t('data_found'), _results);
                } else {
                    callback(responseCode.NO_DATA_FOUND, t('data_not_found'), [])
                }
            })
        } catch (error) {
            callback(responseCode.OPERATION_FAILED, error, {});
        }
    },
    /**
 * API method for remove family members 
 */
    deleteFamily: (req, callback) => {
        familyModel.updateOne({ _id: req.family_id }, { is_delete: 1 }, (_err, _result) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('family_member_deleted'), {});
            }
        })
    },
    /*
    * API method for create group
    */
    addGroup: (req, callback) => {
        let sql = `insert into tbl_chat_room set ?`;
        let data = {
            type: 'Group',
            isGroupChat: 1,
            name: req.name,
            image: req.image,
            created_by: req.user_id,
            user1: req.user_id,
        };
        con.query(sql, (err, result) => {
            if (!err) {
                //add member to chat room
                let arr = [];
                arr.push(req.user_id);
                user_model.addMemberToChatRoom({
                    users: arr,
                    group_id: result.insertId,
                    is_admin: 1,
                }, (code, msg, data) => {
                    if (code == 1) {
                        callback(responseCode.CREATED, t('group_create_successfully'), {});
                    } else {
                        callback(code, msg, {});
                    }
                });
            } else {
                callback(responseCode.OPERATION_FAILED, err, {});
            }
        });
    },
    addMemberToChatRoom: (req, callback) => {
        let insertData = []
        asyncLoop(req.users, (item, next) => {
            insertData.push([req.group_id, item.user_id, (req.is_admin != undefined ? 1 : 0)]);
        }, () => {
            con.query(`INSERT INTO tbl_chat_users set (chat_room_id,user_id,is_admin) VALUES ?`, [insertData], (_err, _result) => {
                if (_err) {
                    callback(responseCode.OPERATION_FAILED, _err, {});
                } else {
                    callback(responseCode.SUCCESS, t('member_added'), {});
                }
            })
        })
    },
    removeMemberFromGroup: (req, callback) => {
        let sql = `DELETE FROM tbl_chat_users WHERE chat_room_id =? AND user_id =?`;
        con.query(sql, [req.group_id, req.user_id], (_err, res) => {
            if (_err) {
                callback(responseCode.OPERATION_FAILED, _err, {});
            } else {
                callback(responseCode.SUCCESS, t('member_deleted'), {});
            }
        })
    },
};

module.exports = user_model