let config = require('../config/config');
let constant = require('../config/constant');
const asyncLoop = require('node-async-loop')
const { t } = require('localizify');
const con = require('../config/database');
let validate_header = require('../middleware/validate-header');
let httpStatusCode = require('http-status-codes')
const userModel = require('../model/user');
const userQuesAnsModel = require('../model/user_question_answers');
const userDeviceModel = require('../model/user_device_info');
const userInerestsModel = require('../model/user_interests');
const loveLanguagesModel = require('../model/love_languages');
const notificationModel = require('../model/notification');
const notiLibrary = require('../config/notification')
const mongoose = require('mongoose');
/* create common function in this file */
let common = {
    // this function is used to get user's data
    getUserData: async (req, callback) => {
        // code here
        userModel.aggregate([
            {
                $match: {
                    _id: req.user_id
                }
            },
            {
                $lookup: {
                    from: 'user_device_infos',     //collection name
                    localField: '_id',
                    foreignField: 'user_id',
                    as: 'userDeviceData'     //alias
                }
            },
            {
                $unwind: '$userDeviceData' // Deconstructs the array of joined documents into separate documents
            },

            {
                $project: {
                    username: 1,
                    nickname: 1,
                    first_name: 1,
                    last_name: 1,
                    dob: 1,
                    email: 1,
                    country_code: 1,
                    phone: 1,
                    is_biomatric_enable: 1,
                    zip_code: 1,
                    gender: 1,
                    profile_image: { $concat: [constant.s3Bucket_profile, "$profile_image"] },
                    blood_type: 1,
                    profile_type: 1,
                    status: 1,
                    signup_step: 1,
                    love_language: 1,
                    is_verify: 1,
                    referral_code: 1,
                    personality_type: 1,
                    secondary_email: 1,
                    other_gender: 1,
                    add_other_gender: 1,
                    token: '$userDeviceData.token' // Select specific field (name) from the joined user data
                }
            }
        ]).then(result => {
            // console.log("result", result)
            userQuesAnsModel.aggregate([
                { $match: { user_id: result[0]._id, is_delete: '0' } },
                {
                    $lookup: {
                        from: 'questions', //collection name
                        localField: 'question_id',
                        foreignField: '_id',
                        as: 'userQuestionData'
                    }
                },
                {
                    $unwind: '$userQuestionData' // Deconstructs the array of joined documents into separate documents
                },
                {
                    $lookup: {
                        from: 'questions_answers', //collection name
                        localField: 'answer_id',
                        foreignField: '_id',
                        as: 'answersData'     //alias
                    }
                },
                {
                    $unwind: '$answersData' // Deconstructs the array of joined documents into separate documents
                },
                {
                    $project: {
                        question_id: '$userQuestionData._id',
                        user_answer_id: '$answersData._id',
                        question: '$userQuestionData.question',
                        user_answer: '$answersData.answer',
                    }
                }
            ]).then(quesResult => {
                result[0]['questions'] = quesResult
                userInerestsModel.aggregate([
                    { $match: { user_id: result[0]._id, is_delete: '0' } },
                    {
                        $lookup: {
                            from: 'interests',     //collection name
                            localField: 'interest_id',
                            foreignField: '_id',
                            as: 'userInterestsData'     //alias
                        }
                    },
                    {
                        $unwind: '$userInterestsData' // Deconstructs the array of joined documents into separate documents
                    },
                    {
                        $project: {
                            interest_id: 1,
                            name: '$userInterestsData.name',
                            image: { $concat: [constant.s3Bucket_interest, "$userInterestsData.image"] }
                        }
                    }
                ]).then(interestResult => {
                    result[0]['interest'] = interestResult;
                    if (result[0].love_language.length > 0) {
                        loveLanguagesModel.find({ _id: { $in: result[0].love_language } }, {
                            _id: 1,
                            name: 1,
                            description: 1,
                            image: { $concat: [constant.s3Bucket_loveLanguage, "$image"] },
                            activeImage: { $concat: [constant.s3Bucket_loveLanguage, "$activeImage"] }
                        }, (err, loveLanguage) => {
                            if (err) {
                                callback(err, undefined);
                            } else {
                                result[0]['love_language'] = loveLanguage;
                                callback(undefined, result[0]);
                            }
                        })
                    } else {
                        result[0]['love_language'] = [];
                        callback(undefined, result[0]);
                    }

                }).catch((err) => {
                    console.error("Error in getting user info : ", err);
                    callback(err, undefined);
                })
            }).catch((err) => {
                console.error("Error in getting user info : ", err);
                callback(err, undefined);
            })
        }).catch((err) => {
            console.error("Error in getting user info : ", err);
            callback(err, undefined);
        })
    },
    sendNotificationForAllDevices: async (req, callback) => {
        try {
            console.log("req", req)
            if (req.receiver_id + "" != req.sender_id + "") {
                // let userData = await common.userDetails(req.receiver_id);
                let noti_data = {
                    receiver_id: req.receiver_id,
                    receiver_type: "User",
                    sender_id: req.sender_id,
                    sender_type: "User",
                    title: req.title,
                    message: req.message,
                    notification_tag: req.tag,
                    image: req.image_url
                };
                if (req.post_id != undefined && req.post_id != '') {
                    noti_data.post_id = req.post_id;
                }
                if (req.comment_id != undefined && req.comment_id != '') {
                    noti_data.comment_id = req.comment_id;
                }
                if (req.like_id != undefined && req.like_id != '') {
                    noti_data.like_id = req.like_id;
                }
                if (req.reel_id != undefined && req.reel_id != '') {
                    noti_data.reel_id = req.reel_id;
                }
                if (req.follow_id != undefined && req.follow_id != '') {
                    noti_data.follow_id = req.follow_id;
                }
                if (req.collaborate_id != undefined && req.collaborate_id != '') {
                    noti_data.collaborate_id = req.collaborate_id;
                }
                if (req.goal_id != undefined && req.goal_id != '') {
                    noti_data.goal_id = req.goal_id;
                }
                if (req.goal_request_id != undefined && req.goal_request_id != '') {
                    noti_data.goal_request_id = req.goal_request_id;
                }

                notificationModel.create(noti_data, (_err, _data) => {
                    if (_err) {
                        console.log("Common notificationModel.create Error: ", _err)
                        callback(t('something_went_wrong'), undefined);
                    } else {
                        userDeviceModel.find({ user_id: req.receiver_id, token: { $ne: "" } }, { user_id: 1, token: 1, device_token: 1, device_type: 1, voip_token: 1 }, (err, _user_data) => {
                            console.log("sendNotificationForAllDevices _user_data", _user_data)
                            if (err) {
                                console.log("Common err ", _err)
                                callback(err, undefined);
                            } else if (_user_data.length > 0) {

                                asyncLoop(_user_data, (item, next) => {
                                    let message_data = {
                                        device_token: item.device_token,
                                        device_type: item.device_type,
                                        title: req.title,
                                        message: req.message,
                                        tag: req.tag,
                                        image_url: req.image_url,
                                    }

                                    if (req.post_id != undefined && req.post_id != '') {
                                        message_data.post_id = req.post_id;
                                    }
                                    if (req.comment_id != undefined && req.comment_id != '') {
                                        message_data.comment_id = req.comment_id;
                                    }
                                    if (req.like_id != undefined && req.like_id != '') {
                                        message_data.like_id = req.like_id;
                                    }
                                    if (req.reel_id != undefined && req.reel_id != '') {
                                        message_data.reel_id = req.reel_id;
                                    }
                                    if (req.follow_id != undefined && req.follow_id != '') {
                                        message_data.follow_id = req.follow_id;
                                    }
                                    if (req.collaborate_id != undefined && req.collaborate_id != '') {
                                        message_data.collaborate_id = req.collaborate_id;
                                    }
                                    if (req.goal_id != undefined && req.goal_id != '') {
                                        message_data.goal_id = req.goal_id;
                                    }
                                    if (req.goal_request_id != undefined && req.goal_request_id != '') {
                                        message_data.goal_request_id = req.goal_request_id;
                                    }
                                    console.log("message_data", message_data)
                                    notiLibrary.sendPush(item.device_token, message_data, (status) => {
                                        if (status) {
                                            console.log(status)
                                            next();
                                        } else {
                                            next();
                                        }
                                    })

                                }, () => {
                                    // all done so return the response
                                    callback(undefined, true);

                                })


                            } else {
                                callback(undefined, true);
                            }
                        })
                    }
                })
            } else {
                callback(undefined, true);
            }

        } catch (error) {
            callback(t('something_went_wrong'), undefined);
        }

    },
    userDetails: async (req, callback) => {
        return new Promise(async (resolve, reject) => {
            try {
                const objectId = mongoose.Types.ObjectId(req.receiver_id);
                let userData = await userModel.find({ _id: objectId }, {
                    _id: 1,
                    username: 1,
                    nickname: 1,
                    first_name: 1,
                    last_name: 1,
                });
                if (userData.length >= 0) {
                    resolve(userData[0])
                } else {
                    reject(t('somthing_went_wrong'));
                }
            } catch (error) {
                reject(error)
            }
        })
    },

};
module.exports = common;